<?php

namespace HCM\FPA\Calculos\Ferias;

use HCM\FPA\Calculos\Geral\Calculo;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaPtregraEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\FPA\Entity\FpaParamfolhahEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaFormcalculoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use HCM\FPA\Calculos\Geral\FGTS;
use HCM\FPA\Calculos\Geral\INSS;
use HCM\FPA\Calculos\Geral\IRRF;

class CalculoFerias extends Calculo {

    public function calculaFerias($vinculo, $tp_calc_mov, $contador, $nrGozoFerias, $Idnaopagaradic){
        
        if($this->dataSet['DSORDEM'] == "LISTA_LOTE"){
            if(!is_array($this->dataSet['DSINICIO'])){
                $nrlotevincs = explode(',',$this->dataSet['DSINICIO']);
            } else {
                $nrlotevincs = $this->dataSet['DSINICIO'];
            }
            
            foreach($nrlotevincs as $nrlotevinc){
                $vinculoEncontrado = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTEVINC)->findOneBy(array('nrorg' => $this->_nrorg, 'nrlotevinc' =>  $nrlotevinc, 'nrvinculom' => $vinculo->getNrvinculom()));

                if(!empty($vinculoEncontrado)){
                    $this->_nrlotevinc = $nrlotevinc;
                    continue;
                }
            }
        }
        
        $calcula = $this->_inicializaVariaveis($vinculo, $tp_calc_mov);
        $this->nrGozoFerias = $nrGozoFerias;
        
        if(!$calcula){
            return;
        }
        
        if ($tp_calc_mov->retornaItensCalculo($vinculo) === false) {
            return;
        }
        
        try {
            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
            $this->_itens_calculo_principal =  $obj_tp_calc_mov->getInstancia();
            $this->_itens_calculo_principal->setNrTipoCalculo($this->_nr_tipo_calculo);
        } catch(Exception $e) {
            $message = $e->getMessage();
            $message .= ' - CALCULO.PHP';
            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
        }
        
        $this->geraIdedmdev($vinculo);

        if ($this->_nr_tipo_calculo !== FpatipocalculoEntity::CALC_FECHAMENTO &&
            $this->_nr_tipo_calculo !== FpatipocalculoEntity::CALC_PROVISAO_CONTABIL){
            $liquido = $this->_controleLiquido();
            if ($liquido == true) {
                MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui liquido.', MsgCalculo::ERRO);
                return;
            }
        }

        $this->_controleLiquidoParcial();
        
        $this->_inicializaVariaveisRegra();
        
        $this->_calculo_regra->geraVariaveis();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_INICIO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );

        $this->_verificaTrabalhoEfetivo($vinculo);
        
        MemoriaCalculo::getInstance()->criaCabecalho($this->entityManager, $this->hcmParameters, $this->_vinculo,$this->_nr_tipo_calculo,$this->_sit_func_resumo,$this->_salario,$this->_dias_uteis,$this->_dias_rsr,
            $this->_mes_competencia, $this->_dt_pagamento, $this->_nrOcorrenciaCalculo,$this->_obj_tipo_calculo->getNmtipocalculo(),$this->_gozo_ferias);
        
        $this->_calculaRemuneracaoBase($vinculo, $this->_itens_calculo_principal, $this->_nr_tipo_calculo);
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_ANTES_CALC_ESPECIFICO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        $this->_executaFerias($vinculo,$tp_calc_mov);
        
        $this->_geraPensao();
        
        $this->_calculaTotais();
        
        if($this->_nr_tipo_calculo!=FpatipocalculoEntity::CALC_13SAL) {
            $this->_calculo_regra->executaPontoRegra(
                $this->_idDebug, FpaPtregraEntity::PTREGRA_FINAL, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
            );
        }
        
        MemoriaCalculo::getInstance()->setMovimentos();
        MemoriaCalculo::getInstance()->setIncidencias();
        
        $arquivoMemoria = MemoriaCalculo::getInstance()->getNmArquivo();
        
        $lista = null;
        
        if($arquivoMemoria){
            $lista['ARQUIVOMEMORIA'] = FPA::getTempDirectory().'/'.$arquivoMemoria;
        }
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            return $lista;
        }
        
        if ($this->_simulacao === false) {
            $tp_calc_mov->persisteItensCalculo($this->nrGozoFerias, $this->_nrlotevinc, $this->_idedmdev);
            return $lista;
        } else {
            $lista['SIMULACAO'] = $tp_calc_mov->retornaListaItensCalculo(
                $this->_mes_competencia, $this->_eventom, $vinculo
            );
            //Vinculo
            if(count($lista) > 0){
                $lista['SIMULACAO'][0][] = array(
                    'MESOCORRENCIA' => $vinculo->getNrvinculom(),
                    'TIPOEVENTO' => $vinculo->getNmvinculom(),
                    'NRFILHO' => $vinculo->getNrvinculom(),
                    'NREVENTOM' => 'Matricula: '.$vinculo->getCdmatricula(),
                    'NMEVENTOH' => 'Salario: ' .number_format($this->_salario, 2, ',','.'),
                    'UNIDREFEEVEN' => 'Admissão: '.substr($vinculo->getDtadmissaovinc(), 0, 10),
                    'VRREFFOLHA' => null,
                    'IDEVENDEMON' => null,
                    'VREVENTOFOLHA' => null,
                    'NRORDEIMPRES' => null,
                    'ORDEMTIPOEVENTO' => null,
                    'NRVINCULOM' => null,
                    'NRCALCULOFOLHAVINC' => null
                );    
            }
            
            return $lista;
        }
    }
}
