<?php

namespace HCM\FPA\Calculos\Geral;

use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpaeventohEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaincideventoEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\Util\MsgCalculo;
use HCM\Util\DateUtil;

class CalculoRateio extends TeknisaRegra {

    protected $_itens_calculo;
    
    private $_calculo;
    private $_mesOcorrenciaEventoCorrente;
    private $_referenciaEventoCorrente;
    private $_valorEventoCorrente;
    private $_ocorrenciasEventoCorrente;
    private $_nrEventoCorrente;
    private $_eventoCorrente;

    public function __construct(){

        $rateioEvaluator = new RateioEvaluator($this);
        parent::__construct($rateioEvaluator);
    }

    public function geraVariaveis(){
        $this->setVar('MES_COMPETENCIA', '"'.substr($this->_calculo->getMesCompetencia(), 0, 10).'"');
        $this->setVar('DIAS_TRABALHO', $this->_calculo->getDiasNormais());
        $this->setVar('DIAS_FERIAS', $this->_calculo->getDiasFerias());
        $this->setVar('DIAS_AFASTADO', $this->_calculo->getDiasAfastados());
        $this->setVar('JORNADA_DIARIA', $this->_calculo->getJornadaDiaria());
        
    }

    public function retornaValorPorCondicao($condicao, $valorSeVerdadeiro, $valorSeFalso) {
        if ($condicao) {
            return $valorSeVerdadeiro;
        } else if ($valorSeFalso !== false && $valorSeFalso !== null) {
            return $valorSeFalso;
        } else {
            return ABORT;
        }
    }

    private function _atribuiVariavelEventoCorrente($nrEvento){
        $this->setVar('EVENTO_CORRENTE', $nrEvento);
    }

    public function getCalculo(){
        return $this->_calculo;
    }
    
    public function retornaDataInicioUltMovimentacao($nrTipoEstrutura){
        $nrvinculom = $this->_calculo->getVinculo()->getNrvinculom();
        $nrorg = $this->_calculo->getNrorg();
        $dia_final = $this->getUltimoDiaMes();
        $dataInicioMovimentacao = $this->buscarDataInicioMovimentacao($nrorg, $nrTipoEstrutura, $nrvinculom, $dia_final);
        
        return $dataInicioMovimentacao;
    }
    
    private function buscarDataInicioMovimentacao($nrorg, $nrTipoEstrutura, $nrvinculom, $dia_final){
        $gpemovimentacao = $this->_calculo->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacao($nrorg, $nrTipoEstrutura, $nrvinculom);
    
        foreach($gpemovimentacao as $movimentacao){
            $dataFimMovimentacao = $movimentacao->getDtfimmoviment();
            $dataInicioMovimentacao = $movimentacao->getDtinimoviment()->format('d/m/Y');
            
            if(is_null($dataFimMovimentacao) || $dataFimMovimentacao >= $dia_final){
                return $dataInicioMovimentacao;
            }
        }
        return null;
    }
    
    public function retornaSalarioMinimo(){
        $nrValorReferenciaM = 1; #Referencia do valorReferenciaMestre de salario minimo
        $nrorg = $this->_calculo->getNrorg();
        
        $valorReferencias = $this->_calculo->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAH)->findByNrvalorreferenciam($nrValorReferenciaM);
        
        $vrreferencia = 0;
        
        if(!is_null($valorReferencias)){
            foreach($valorReferencias as $valorReferencia){
                $vrreferencia = $valorReferencia->getVrreferencia() > $vrreferencia ? $valorReferencia->getVrreferencia() : $vrreferencia;
            }
        }
        
        return $vrreferencia;
    }
    
    public function retornaTipoJornada()
    {
        $nrEscalaTrabM = $this->_calculo->getVinculo()->historico->getNrescalatrabm();
        
        $nrTipoJornada = $this->_calculo->entityManager->getRepository(Repositories::GPE_ESCALATRABM)
        ->retornaEscalaTrab($nrEscalaTrabM, $this->_calculo->getNrorg())
        ->historico->getNrtpjornada();
        return $nrTipoJornada;
    }

    public function retornaValorIncidencia($tipo_incidencia){
        $mes_competencia = $this->_calculo->getMesCompetencia();
        $valor = $this->_itens_calculo->calculaIncidencias(
            $tipo_incidencia, $mes_competencia, $this->_calculo->getEventom()
        );
        
        if(!$valor or is_nan($valor)){
            $valor = 0;
        }
        
        return $valor;
    }

    public function retornaReferenciaIncidencia($tipo_incidencia){
        $mes_competencia = $this->_calculo->getMesCompetencia();
        $valor = $this->_itens_calculo->calculaIncidencias(
            $tipo_incidencia, $mes_competencia, $this->_calculo->getEventom()
        );
        $referencia = $this->_itens_calculo->getTotalReferencias();
        
        if(!$referencia or is_nan($referencia)){
            $referencia = 0;
        }
        
        return $referencia;
    }

    public function retornaValorEvento($nr_evento){
        $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
        $valor = $this->_itens_calculo->retornaValorPorEvento($evento);
        
        if(!$valor or is_nan($valor)){
            $valor = 0;
        }
        
        return $valor;
    }

    public function retornaReferenciaEvento($nr_evento){
        $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
        $referencia = $this->_itens_calculo->retornaReferenciaPorEvento($evento);
        
        if(!$referencia or is_nan($referencia)){
            $referencia = 0;
        }
        
        return $referencia;
    }

    public function getDiasCompetencia(){
        $dia_inicial=$this->getDataCompetencia();
        $dia_final=$this->getUltimoDiaMes();
        
        $dias_reais = DateUtil::getQtdeDiasEntre($dia_inicial, $dia_final) + 1;
        return $dias_reais;
    }

    public function retornaContadorEvento($nr_evento){
        if ((int)$nr_evento === (int)$this->_nrEventoCorrente) {
            return $this->_ocorrenciasEventoCorrente;
        } else {
            $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
            $contador = $this->_itens_calculo->retornaContadorPorEvento($evento);
            return $contador;
        }
    }

    public function verificaExistenciaEvento ($nr_evento){
        $evento = $this->_calculo->getEventom()->retornaEvento($nr_evento);
        $mes_competencia = $this->_calculo->getMesCompetencia();
        $indice = $this->_itens_calculo->procuraEvento($evento, $mes_competencia);

        $verifica = true;
        
        if ($indice === -1) {
            $verifica = false;
        }
        return $verifica;
    }

    public function atribuiValor($valor){
        $this->_valorEventoCorrente = $valor;
    }

    public function atribuiReferencia($referencia){
        $this->_referenciaEventoCorrente = $referencia;
    }

    public function atribuiOcorrencias($ocorrencias){
        $this->_ocorrenciasEventoCorrente = $ocorrencias;
    }

    public function totalizaValorHistorico($listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia = null){
        $valor = $this->_totalizaHistoricoPorTipo(
            $listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia, FpaitecalcfolhaEntity::VALOR
        );
        
        if(!$valor or is_nan($valor)){
            $valor = 0;
        }
        
        return $valor;
    }

    public function totalizaReferenciaHistorico($listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia = null){

        $referencia = $this->_totalizaHistoricoPorTipo(
            $listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia, FpaitecalcfolhaEntity::REFERENCIA
        );
        
        if(!$referencia or is_nan($referencia)){
            $referencia = 0;
        }
        
        return $referencia;
    }

    private function _totalizaHistoricoPorTipo($listaNrEvento, $nrTipoMovimento, $deslocamento, $mesIncidencia, $tipo){

        if ($mesIncidencia ===  null) {
            $mesIncidencia = $this->_calculo->getMesCompetencia();
        } else {
            $mesIncidencia = DateUtil::setDay($mesIncidencia, 1);
        }
        $lista = $this->_retornaItensCalculosHistorico($listaNrEvento, $nrTipoMovimento, $mesIncidencia, $deslocamento);

        foreach ($lista as $item) {
            if(!$item[$tipo] or is_nan($item[$tipo])){
                $item[$tipo] = 0;
            }
            $valor += $item[$tipo];
        }
        
        return $valor;
    }
    
    private function _retornaItensCalculosHistorico($listaNrEvento, $nrTipoMovimento, $mesIncidencia, $deslocamento){

        $dtInicial = $mesIncidencia;
        if ($deslocamento > 0) {
            $dtInicial = DateUtil::addSubIntevalo($dtInicial, $deslocamento, DateUtil::MES);
        }

        $dtFinal = $mesIncidencia;

        $linha = $this->_calculo->getMesCompetencia()->monthsAfter($mesIncidencia);
        $coluna = $deslocamento;

        if (count($this->_calculo->listaCalculada[$linha][$coluna]) > 0) {
            $listaNrCalcFolha = $this->_calculo->listaCalculada[$linha][$coluna];
        } else {
            $calculoFolha = new FpacalculofolhaEntity();

            $listaNrCalcFolha = $calculoFolha->retornaCalculoFolhaPeriodo($this->_calculo->getNrorg(),
                                $nrTipoMovimento, $dtInicial, $dtFinal);

            $this->_calculo->listaCalculada[$linha][$coluna] = $listaNrCalcFolha;
        }

        $itemCalculo = new FpaitecalcfolhaEntity();
        $listaItensCalculos = $itemCalculo->totalizaItensHistorico(
            $this->_calculo->getVinculo(), $listaNrCalcFolha, $listaNrEvento
        );
        return $listaItensCalculos;

    }

    public function totalizaProvento(){
        $total = $this->_retornaValorTotalProventoDesconto();
        return $total[FpaeventomEntity::IDTIPOEVENTO_PROVENTO];
    }

    public function totalizaDesconto(){
        $total = $this->_retornaValorTotalProventoDesconto();
        return $total[FpaeventomEntity::IDTIPOEVENTO_DESCONTO];
    }

    private function _retornaValorTotalProventoDesconto(){
        $total = $this->_itens_calculo->retornaTotais($this->_calculo->getEventom());
        return $total;
    }

    public function retornaDadosVinculo($descricao){
        $objeto = $this->_calculo->getVinculo();
        return $this->_retornaDados($descricao, $objeto);

    }

    private function _retornaDados($descricao, $objeto){

        $nomeColuna = strtolower($descricao);
        $nomeColuna[0] = strtoupper($nomeColuna[0]);
        $metodoVerifica = 'get'.$nomeColuna;
        $metodo = 'get'.$nomeColuna.'();';
        $resultado = '';
        $objetoMestre = $objeto;
        
        if ($metodoVerifica != 'getNrdiasapurvales' && method_exists($objetoMestre, $metodoVerifica)) {
            $resultado = 'return $objeto->'.$metodo;
        } else {
            if (method_exists($objetoMestre->historico, $metodoVerifica)) {
                $resultado = 'return $objeto->historico->'.$metodo;
            } 
        }
        return eval($resultado);

    }

    public function executaRegra($codigo, $debug = false){
        $valor = parent::executaRegra($codigo, $debug);
        return $valor;
    }

    public function getDiasEntre($dataInicial, $dataFinal) {
        return DateUtil::getQtdeDiasEntre($dataInicial, $dataFinal);
    }

    public function getDataCompetencia() {
        return $this->_calculo->getMesCompetencia();
    }

    public function getUltimoDiaMes() {	
		return DateUtil::getUltimoDiaDoMes($this->_calculo->getMesCompetencia());
    }

    public function getTomador($nrestrutgeren)  {
        $nrorg = FPA::getNrOrgTrabalho();

        return Gpevinculom::getTomador($nrorg, $nrestrutgeren);
    }

    public function converteHoraEmDecimal($valor_hora_minuto){
        $horas = (int)$valor_hora_minuto;
        $minutos = $valor_hora_minuto-$horas;
        if($minutos>0){
            $decimal=$minutos/60*100;
        }
        $valor_hora_decimal=$horas+$decimal;
        return $valor_hora_decimal;
    }
    
    public function retornaIdadeVinculo(){
        $vinculo = $this->_calculo->getVinculo();
        
        $pessoa = $this->_calculo->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaAtual($vinculo->getNrorg(), $vinculo->getNrpessoa(), $this->_calculo->getMesCompetencia());
        
        $idade=0;
        if($pessoa){
            $dtnascpessoa = DateUtil::getDataDeString($pessoa->historico->getDtnascpessoa());
            
            $dtatual = DateUtil::getDataAtual();
             
            $anoNasc = $dtnascpessoa->format('Y');
			$anoComp = $dtatual->format('Y');
			$idade = abs($anoNasc - $anoComp); 
            
        }
        return $idade;
    }
    
    public function retornaTrabalhoEmAnos(){
        $vinculo= $this->_calculo->getVinculo();
        $dtadmissao= $vinculo->getDtadmissaovinc();
        $anosTrabalho=0;
        
        if($dtadmissao){
            $dtatual = DateUtil::getDataAtual();
            $dtUltdiames = $this->getUltimoDiaMes();
			
			$anosTrabalho = DateUtil::getQtdeAnosEntre($dtadmissao, $dtUltdiames); 
			//$anosTrabalho = abs($anoAdm - $anoAt); 
        }
       
       return $anosTrabalho;
    }
    
     public function retornaValEvenOcorrencias($nr_evento){
        
        $ocorrencias = array(1,2,3,4,5);
        $valor = 0;
        
        foreach($ocorrencias as $ocorrencia){
            
            if($this->_calculo->_nrOcorrenciaCalculo == $ocorrencia){
                continue;
            }
            
            $calc_folha = $this->_calculo->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                    $this->_calculo->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1, $ocorrencia, $this->_calculo->_mes_competencia, $this->_calculo->_mes_competencia
                )[0];
                
            if($calc_folha){
                $itemCalcCompetencia = new FpaitecalcfolhaEntity($this->_calculo->entityManager, $this->_calculo->hcmParameters, $calc_folha->getNrcalculofolha());
                $itemCalcCompetencia->retornaItensPorVinculo($this->_calculo->_vinculo->getNrorg(), $this->_calculo->_vinculo->getNrvinculom(), $calc_folha->getNrcalculofolha());
                
                $eventom = $this->_calculo->_eventom->retornaEvento($nr_evento);
                $indice = $itemCalcCompetencia->procuraEvento($eventom);
                $evento = $itemCalcCompetencia->getItem($indice);
                
                if($evento){
                    $valor += $evento->getVreventofolha();
                }
            }            
        }
        
        if(!$valor or is_nan($valor)){
            $valor = 0;
        }
        
        return $valor;
    }

}