<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 22/07/2016
 * Time: 16:25
 */

namespace HCM\FPA\Controllers;

use Folha\Entities\FpaTipomoviment;
use HCM\FPA\Services\ItensCalculoService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use HCM\Geral\Entity\GpevinculomEntity;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Util\FPA;

class ItensCalculoController extends Crud {

    protected $dataSourceName = 'fpa_itecalcfolha';
    protected $dataSourceName_get = 'getItecalcfolha';
    protected $dataSourceName_getEstrutura = 'getItecalcfolhaPorEstrutura';

    protected $parameterBagColumns = array(
        'P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_NRTIPOMOVIMENT', 'P_NRTPMODALIDCAL', 'P_RESCISAO', 'P_ULTIMODIAMES', 'P_NRCALCULOFOLHA', 'P_NRESTRUTLEGAL', 'P_NRESTRUTGEREN',
        'NRESTRUTURA_BENEFICIO', 'NMVINCULOM', 'NRTPBENEFICIO_LIST', 'NRBENEFICIOM_LIST', 'P_NRESTRUTLEGALOPER', 'P_NRESTRUTGERENOPER', 'P_NRTIPOESTRUTGEREN', 'P_NRTIPOESTRUTOMADOR',
        'P_DTOCORRENCIA_INI', 'P_DTOCORRENCIA_FIM', 'NRLOTEVINC_LIST'
        );

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ItensCalculoService $itensCalculoService, EntityManager $entityManager){
        $this->parameterBag          = $parameterBag;
        $this->hcmParameters         = $hcmParameters;
        $this->itensCalculoService   = $itensCalculoService;
        $this->entityManager         = $entityManager;
        $this->vinculomEntity        = new GpevinculomEntity($entityManager, $hcmParameters);
        parent::__construct($dataSourceManager);
    }

    public function getItecalcfolha (DTO\Request\Filter $request, DTO\Response $response) {

        set_time_limit(0);
        
        $nrtipomoviment = null;
        $nrestrutura_beneficio = null;
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        foreach($conditions as &$condition) {
            if($condition['columnName'] == 'P_NRTIPOMOVIMENT') {
                $nrtipomoviment = $condition['value'];
            }
            if($condition['columnName'] == 'NRESTRUTURA_BENEFICIO') {
                $nrestrutura_beneficio = $condition['value'];
            }
            // O operador recebido do front é '=' ao invés de 'IN'
            if($condition['columnName'] == 'NRVINCULOM') {
                $condition['operator'] = 'IN';
            }
            if($condition['columnName'] == 'NRTPBENEFICIO_LIST') {
                $condition['value'] = implode(",", $condition['value']);
            }
            if($condition['columnName'] == 'NRBENEFICIOM_LIST') {
                $condition['value'] = implode(",", $condition['value']);
            }
            if($condition['columnName'] == 'NRLOTEVINC_LIST') {
                $condition['value'] = implode(",", $condition['value']);
            }
        }

        $filterData = new FilterCriteria(
            $filterData->getDataSourceName(),
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );
        
        $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutlegalOper());
        $filterData->addCondition('P_NRESTRUTGERENOPER', '=', FPA::getNrestrutgerenOper());
        $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
        $filterData->addCondition('P_DTMESCOMPETENC','=',$this->hcmParameters->getDtCompetencia());
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());

        $filterData->addCondition('P_NRTIPOESTRUTGEREN', '=', $this->hcmParameters->getTipoEstruturaGerencialOrg());
        $filterData->addCondition('P_NRTIPOESTRUTOMADOR', '=', $this->hcmParameters->getTipoEstruturaTomadorOrg());
        
        if($nrtipomoviment){
            $filterData->addCondition('P_NRCALCULOFOLHA','=',$this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment));
            
            if($nrtipomoviment==21){
                $filterData->addCondition('P_RESCISAO','=','T');
            }
        }
        //$filterData->addCondition('P_RESCISAO','=','N');
        $nrestrutura_beneficio ? null : $filterData->addCondition('NRESTRUTURA_BENEFICIO', '=', null);
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $newFilterData->setPageSize(99999999999);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        /*$newDataSet = array();
        foreach($dataSet->getRows() as $data){
            $data['NRVINCULOM'] = strval($data['NRVINCULOM']);
            $data['VINCULO_GROUPED'] = $data['NMVINCULOM'].' - '.$data['NRVINCULOM'];
            array_push($newDataSet, $data);
        }*/
        $response->addDataSet($dataSet);
    }
    
    public function getItecalcfolhaPorEstrutura (DTO\Request\Filter $request, DTO\Response $response) {

        set_time_limit(0);
        

        $filterData = $request->getFilterCriteria();
        foreach($filterData->getConditions() as $condition) {
            if($condition['columnName'] == 'P_NRTIPOMOVIMENT') {
                $nrtipomoviment = $condition['value'];
                $filterData->addCondition('P_NRCALCULOFOLHA','=',$this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment));
            }
        }
        
        $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        $filterData->setDataSourceName($this->dataSourceName_getEstrutura);
        $filterData->setPageSize(9999);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getItecalcfolhaPorCompetencia(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("getItecalcfolhaPorCompetencia");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function setItecalcfolha (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $msg = array();
            $disponibilidade = true;
            //Busca valor de referencia de bloqueio edição de eventos totais.
            $valorReferenciaTotal = $this->itensCalculoService->verificaValorReferenciaBloqueiaEdicao();
            $valorReferenciaEncargo = $this->itensCalculoService->verificaValorReferenciaBloqueiaEncargo();
            $valorReferenciaComplementoLiquido = $this->itensCalculoService->verificaValorReferenciaBloqueiaComplementoLiquido();
            foreach ($request->getDataSet()->getRows() as &$row) {

                if($row['__is_new'] == false && ($valorReferenciaTotal == 1 && $row['IDTIPOEVENTO'] == 'TOTAL')){
                    $msg[] = array('msg' => 'Operação Bloqueada. Não é possível fazer a edição do evento: '.$row['NREVENTOM'].' do tipo TOTAL.');
                    $disponibilidade = false;
                }
                
                if($row['__is_new'] == false && ($valorReferenciaComplementoLiquido == 1 && $row['NREVENTOM'] == 760)){
                    $msg[] = array('msg' => 'Operação Bloqueada. Não é possível fazer a edição do evento: '.$row['NREVENTOM'].' complemento de líquido.');
                    $disponibilidade = false;
                }
                
                if($row['__is_new'] == false && ($valorReferenciaEncargo == 1 && $row['IDTIPOEVENTO'] == 'ENCARGO')){
                    $msg[] = array('msg' => 'Operação Bloqueada. Não é possível fazer a edição do evento: '.$row['NREVENTOM'].' do tipo ENCARGO.');
                    $disponibilidade = false;
                }

                if (isset($row['NRTIPOVINCULOM'])) {
                    $bloq = $this->itensCalculoService->verificaEventoBloqueado($row['NRORG'], $row['NREVENTOM'], $row['NRTIPOVINCULOM']);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $row['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                if ($disponibilidade === true) {
                    if(array_key_exists('NRTIPOMOVIMENT1', $row)){
                        $row['NRTIPOMOVIMENT'] = $row['NRTIPOMOVIMENT1'];
                    }
                    if(array_key_exists('VRREFFOLHA_inline', $row)){
                        $row['__is_new'] = false;
                    }
                    
                    if (array_key_exists('NRCALCULOFOLHA', $row)) {
                        if ($row['NRCALCULOFOLHA'] === '') {
                            if ($row['NRTIPOMOVIMENT'] == 100) {
                                // Lançamento de eventos Processo Trabalhista
                                $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT'], $row['DTMESCOMPETENC']);
                            } else {
                                $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                            }
                        }
                    } else {
                        if ($row['NRTIPOMOVIMENT'] == 100) {
                            // Lançamento de eventos Processo Trabalhista
                            $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT'], $row['DTMESCOMPETENC']);
                        } else {
                            $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                        }
                    }
                    
                    $idevendemoncalc = array_key_exists('IDEVENDEMONCALC', $row) ? $row['IDEVENDEMONCALC'] : 'N';
                    // Somente para o vale transporte é permitido fazer alterações mesmo que já exista líquido.

                    if (($row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_VALE_TRANSPORTE && $row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_BENEFICIO) && $idevendemoncalc != 'S') {
                        $liquido = $this->itensCalculoService->validaLiquido($row['NRTIPOMOVIMENT'], $row['NRCALCULOFOLHA'], $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);
                        if ($liquido) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Message::TYPE_ERROR));
                            return;
                        }
                    }
                    
                    if($row['__is_new']) {
                        $item = $this->itensCalculoService->retornaItemCalculo($row['NRORG'], $row['NRCALCULOFOLHA'], $row['NREVENTOM'], $row['NRVINCULOM'], $row['DTOCORRENCIA']);
                        if (is_object($item)) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_ITEM_CALC_UNIQ_ERROR), Message::TYPE_ERROR);
                            return;
                        }
                    }
    
                    if (array_key_exists('NRITEMCALCFOLHA', $row)) {
                        if ($row['NRITEMCALCFOLHA'] === '') {
                            $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                        }
                    } else {
                        $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                    }
                    
                    if(!array_key_exists('NRGOZOFERIAS', $row)){
                        $row['NRGOZOFERIAS'] = null;
                    }
                    
                    
                    $idEventoDemonst = $this->itensCalculoService->getIdeventodemonst(0, $row['NREVENTOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);
                    
                    if($idEventoDemonst){
                        $row['IDEVENDEMONCALC'] = $idEventoDemonst[0]['IDEVENTODEMONST'];
                    }
                    
                    $row['QTOCORPROXPERI'] = isset($row['QTOCORPROXPERI']) ? $row['QTOCORPROXPERI'] : 1;
                    
                    parent::save($request, $response);
                }
            }
            if (count($msg) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setItecalcfolhaInline (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            error_reporting(E_ERROR | E_PARSE);
            $msg = array();
            //Busca valor de referencia de bloqueio edição de eventos totais.
            $valorReferenciaTotal = $this->itensCalculoService->verificaValorReferenciaBloqueiaEdicao();
            $valorReferenciaEncargo = $this->itensCalculoService->verificaValorReferenciaBloqueiaEncargo();
            $valorReferenciaComplementoLiquido = $this->itensCalculoService->verificaValorReferenciaBloqueiaComplementoLiquido();
            foreach ($request->getDataSet()->getRows() as &$rows) {
                foreach ($rows as &$row) {  
                    
                    if($row['__is_new'] == false && ($valorReferenciaTotal == 1 && $row['IDTIPOEVENTO'] == 'TOTAL')){
                        $msg[] = array('msg' => 'Operação Bloqueada. Não é possível fazer a edição do evento: '.$row['NREVENTOM'].' do tipo TOTAL.');
                        continue;
                    }
                    
                    if($row['__is_new'] == false && ($valorReferenciaComplementoLiquido == 1 && $row['NREVENTOM'] == 760)){
                        $msg[] = array('msg' => 'Operação Bloqueada. Não é possível fazer a edição do evento: '.$row['NREVENTOM'].' complemento de líquido.');
                        continue;
                    }
                    
                    if($row['__is_new'] == false && ($valorReferenciaEncargo == 1 && $row['IDTIPOEVENTO'] == 'ENCARGO')){
                        $msg[] = array('msg' => 'Operação Bloqueada. Não é possível fazer a edição do evento: '.$row['NREVENTOM'].' do tipo ENCARGO.');
                        continue;
                    }
                    
                    if(array_key_exists('NRTIPOMOVIMENT1', $row)){
                        $row['NRTIPOMOVIMENT'] = $row['NRTIPOMOVIMENT1'];
                    }
                    if(array_key_exists('VRREFFOLHA_inline', $row)){
                        $row['__is_new'] = false;
                    }
                    $idevendemoncalc = array_key_exists('IDEVENDEMONCALC', $row) ? $row['IDEVENDEMONCALC'] : 'N';
                    
                    if (array_key_exists('NRCALCULOFOLHA', $row)) {
                        if ($row['NRCALCULOFOLHA'] === '') {
                            $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                        }
                    } else {
                        $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($row['NRTIPOMOVIMENT']);
                    }
                    // Somente para o vale transporte é permitido fazer alterações mesmo que já exista líquido.
                    if ($row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_VALE_TRANSPORTE  && $idevendemoncalc != 'S') {
                        $liquido = $this->itensCalculoService->validaLiquido($row['NRTIPOMOVIMENT'], $row['NRCALCULOFOLHA'], $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);
                        if ($liquido) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Message::TYPE_ERROR));
                            return;
                        }
                    }
                    
                    if($row['__is_new']) {
                        $item = $this->itensCalculoService->retornaItemCalculo($row['NRORG'], $row['NRCALCULOFOLHA'], $row['NREVENTOM'], $row['NRVINCULOM'], $row['DTOCORRENCIA']);
                        if (is_object($item)) {
                            $response->addMessage(new Message(Messages::HCM01000_FPA_ITEM_CALC_UNIQ_ERROR), Message::TYPE_ERROR);
                            return;
                        }
                    }
                    
                    if (array_key_exists('NRITEMCALCFOLHA', $row)) {
                        if ($row['NRITEMCALCFOLHA'] === '') {
                            $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                        }
                    } else {
                        $row['NRITEMCALCFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true);
                    }
                    
                    $this->itensCalculoService->updateItensCalc($row);
                    
                    $message = new \Zeedhi\Framework\DTO\Response\Message('Registro salvo com sucesso', DTO\Response\Message::TYPE_MESSAGE);
                    $response->addMessage($message);
                }
            }
            
            if (count($msg) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
            
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function zerarCalculoRecisorio (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            error_reporting(E_ERROR | E_PARSE);
            foreach ($request->getDataSet()->getRows() as & $row) {
                
                
                    $nrtipomoviment     = 1;
                    
                    $dtmescompetenc     = $this->hcmParameters->getDtCompetencia();
                    
                    $vrreffolha = $row['VRREFFOLHA'];
                    $row['VRREFFOLHA'] = 0;
                    
                    $vreventofolha = $row['VREVENTOFOLHA'];
                    $row['VREVENTOFOLHA'] = 0;
                    
                    $nrcalculofolha = $row['NRCALCULOFOLHA'];
                    $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment);
                    
                    $row['DTRESCISAOFIM'] = DateUtil::getDataDeString($row['DTRESCISAOFIM'], DateUtil::FORMATO_BRASILEIRO, true);
                    
                    $row['DTRESCISAOINI'] = DateUtil::getDataDeString($row['DTRESCISAOINI'], DateUtil::FORMATO_BRASILEIRO, true);

                    $vinculosRescindidos = $this->vinculomEntity->retornaVinculosDemitidosMesCompetencia($row['NRORG'], $dtmescompetenc, $row['DSINICIO'], $row['DSFIM'],  $row['DTRESCISAOINI'], 
                    $row['DTRESCISAOFIM'], $row['DSORDEM'],  null, false);
                    
                    $datapagamento = $row['DTPAGAMENTO']; 
                    
                    if(empty($datapagamento))
                    {
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Operação Bloqueada! A data de pagamento não foi preenchida', DTO\Response\Message::TYPE_ERROR));
                        return $response;
                    }
                    
                 if($vinculosRescindidos != null){
                    $insereEventos = $this->itensCalculoService->validaVinculos($row, $vinculosRescindidos);
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
                }else if($vinculosRescindidos == null){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Não foram encontrados vínculos rescindidos para a competência', DTO\Response\Message::TYPE_ERROR));
                }
            }
            
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $idevendemoncalc = array_key_exists('IDEVENDEMONCALC', $row) ? $row['IDEVENDEMONCALC'] : 'N';
                // Beneficio (Soluções) Somente para o vale transporte é permitido fazer exclusão mesmo que já exista líquido.
                if (($row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_VALE_TRANSPORTE && $row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_BENEFICIO) && $idevendemoncalc != 'S') {
                    $liquido = $this->itensCalculoService->validaLiquido($row['NRTIPOMOVIMENT'], $row['NRCALCULOFOLHA'], $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);

                    if ($liquido) {
                        $response->addNotification(new Notification(Messages::HCM01000_FPA_BLOQ_OPER_VINC_LIQ_ERROR, Notification::TYPE_ERROR));
                        return;
                    }
                }
            }

            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    // TODO: Refatorar
    public function deleteMultiplosIteCalcFolha(DTO\Request\DataSet $request, DTO\Response $response)
    {
      try {
        set_time_limit(0);
        $mensagensExclusao = [];
        $itensToDelete = [];
        $nrVinculom = null;

        foreach ($request->getDataSet()->getRows() as &$row) {
          $idevendemoncalc = array_key_exists('IDEVENDEMONCALC', $row) ? $row['IDEVENDEMONCALC'] : 'N';

          if (($row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_VALE_TRANSPORTE && $row['NRTIPOMOVIMENT'] != FpaTipomoviment::TP_MOV_BENEFICIO) && $idevendemoncalc != 'S') {
            $liquido = $this->itensCalculoService->validaLiquido($row['NRTIPOMOVIMENT'], $row['NRCALCULOFOLHA'], $row['NRVINCULOM'], $this->hcmParameters->getDtCompetencia(), $row['NRORG']);

            if ($liquido) {
              if ($row['NRVINCULOM'] != $nrVinculom) {  
                $mensagensExclusao[] = ['msg' => 'Operação bloqueada. Vínculo '. $row["NMVINCULOM"] . ' possui líquido.'];
              }

              $nrVinculom = $row['NRVINCULOM'];

              continue;
            }
          }

          $itensToDelete[] = $row['NRITEMCALCFOLHA'];
        }

        $sizeOfItensToDelete = count($itensToDelete);
        $setOfItensToDelete = [];
        $countToDelete = 0;
        
        if ($sizeOfItensToDelete > 0) {
            for ($index = 0; $index < $sizeOfItensToDelete; $index++) {
                $setOfItensToDelete[] = $itensToDelete[$index];
                $countToDelete++;
                
                if ($sizeOfItensToDelete-1 == $index || $countToDelete == 500) {
                    $this->itensCalculoService->deleteItens($setOfItensToDelete);
                    $setOfItensToDelete = [];
                    $countToDelete = 0;
                }
            }
        }
        
        if (!empty($mensagensExclusao)) {
          $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('mensagensExclusao', $mensagensExclusao));
        } else {
          $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
        }
      } catch (\Exception $e) {
          $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
      }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}