<?php

namespace HCM\FPA\Entity;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\TkEntity;
use HCM\Util\Messages;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpealtesitufuncEntity;

class FpagozoferiasEntity extends TkEntity
{
    private $_mes_1a_parte;
    private $_dias_1a_parte;
    private $_mes_2a_parte;
    private $_dias_2a_parte;
    private $_dias_abono;
    private $_dtInicioGozo;
    private $_dtFimGozo;
    private $_nr_ferias_preparado;
    
    const IDADIANT13SALAR_NAO = 'N';
    const IDADIANT13SALAR_SIM = 'S';

    public function __construct($entityManager, $hcmParameters){
        $this->hcmParameters = $hcmParameters;
        $this->_mes_1a_parte = null;
        $this->_mes_2a_parte = null;
        $this->_dtInicioGozo = null;
        $this->_dtFimGozo = null;
        $this->fpaGozoFerias = Repositories::FPA_GOZOFERIAS;
        $this->alteSituFuncEntity = new GpealtesitufuncEntity($entityManager, $hcmParameters);
        $this->feriasEntity = new FpaferiasEntity($entityManager, $hcmParameters);
        $this->msg = array();
        
        parent::__construct(null,$entityManager);
    }

    public function getNrFeriasPreparado(){
        if ($this->_nr_ferias_preparado > 0) {
            return $this->_nr_ferias_preparado;
        } else {
            return null;
        }
    }

    public function setNrFeriasPreparado($nrFeriasPreparado){
        $this->_nr_ferias_preparado = $nrFeriasPreparado;
    }

    public function getMes1aParte(){
        return $this->_mes_1a_parte;
    }

    public function getDias1aParte(){
        return $this->_dias_1a_parte;
    }

    public function getMes2aParte(){
        return $this->_mes_2a_parte;
    }

    public function getDias2aParte(){
        return $this->_dias_2a_parte;
    }

    public function getDiasAbono(){
        return $this->_dias_abono;
    }

    public function getDtInicioGozo(){
        return $this->_dtInicioGozo;
    }

    public function getDtFimGozo(){
        return $this->_dtFimGozo;
    }
    public function getNrgozoferiasAtual(){
        return $this->_nrGozoFerias;
    }

    public function retornaGozoFerias($nr_org, $nr_vinculo){

        $this->resetItems();
        $resultado = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaItem($nr_org, $nr_vinculo);

        if (count($resultado) > 0) {
            foreach ($resultado as $entity_obj) {
                if(!$entity_obj->getDtInigozoferias()){
                    MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui periodo de gozo de ferias sem data de inicio e fim.', MsgCalculo::ERRO);
                    continue;
                }
                
                $this->addItem($entity_obj, $entity_obj->getNrgozoferias());
            }
        }
    }
    
    public function retornaGozoFeriasNrGozo($nrorg, $nrGozoFerias){
        return $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaGozoFeriasNrGozo($nrorg, $nrGozoFerias);
    }
    
    public function retornaGozosFeriasEntreDatas($nr_org, $nr_vinculo, $dtIni, $dtFim){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaGozosFeriasEntreDatas($nr_org, $nr_vinculo, $dtIni, $dtFim);
        return $resultado;
    }

    public function procuraGozoFerias($dataInicioPeriodo){
        $dataInicioPeriodo = DateUtil::getDataDeString(substr($dataInicioPeriodo, 0, 10));
        foreach ($this->getItems() as $item) {
            if(!empty($dataInicioPeriodo)) {
                $dataIniGozo = DateUtil::getDataDeString(substr($item->getDtinigozoferias(), 0, 10));
                if ($dataInicioPeriodo->format('d/m/Y') === $dataIniGozo->format('d/m/Y')) {
                    return $item;
                }
            } else {
                return null;
            }
        }
    }
    
    public function procuraGozoFeriasSemAdicional(){
        $gozos = [];
        foreach ($this->getItems() as $item) {
            if($item->getIdnaopagaadic() == 'S') {
                $gozos[] = $item;
            }
        }
        return $gozos;
    }

    public function procuraGozoFeriasAnterior($dataInicioPeriodo,$nrferias){
        foreach ($this->getItems() as $item) {
            if ($dataInicioPeriodo > $item->getDtfimgozoferias()
                && $nrferias == $item->getNrferias()) {
                return $item;
            }
        }
    }
    
    public function validacaoESocial ($nrchave, $editarexcluir){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $nrtipoTabela = 2;

        //busca por eventos S-2230 que possuam a chave igual ao número do gozo de férias que está querendo ser modificado
        $query = $this->entityManager->getConnection()->prepare("
SELECT 
  *
FROM
  ESO_ENVIOCADASTROS EC
  JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                      AND (EV.NRORG = EC.NRORG OR EV.NRORG = :P_NRORG_PADRAO)
WHERE
  EV.NMEVENTO = :P_NMEVENTO
  AND (EC.NRTABELA = :P_NRTABELA OR (EC.NRTABELA IS NULL AND :P_NRTABELA IS NULL))
  AND EC.NRORG = :P_NRORG
  AND EC.NRCHAVE = :P_NRCHAVE
  AND EC.NRTPAMBIENTE = 1
  AND EC.IDATIVO = 'S'
        ");
            
        $query->execute(
            array(
                "P_NRTABELA"      => $nrtipoTabela,
                "P_NRCHAVE"       => $nrchave,
                "P_NMEVENTO"      => "S-2230",
                "P_NRORG"         => $nrorg,
                "P_NRORG_PADRAO"  => $nrorgpadrao
            )
        );
        
        $itens = $query->fetchAll();
        

        //busca por eventos S-3000 para os eventos encontrados, caso o campo NRENVIOEXCL esteja preenchido
        foreach($itens as $item){
            if($item['NRENVIOEXCL'] !== null && $item !== null) {
                $query2 = $this->entityManager->getConnection()->prepare("
SELECT
  *
FROM
  ESO_ENVIOCADASTROS EC
  JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                      AND (EV.NRORG = EC.NRORG OR EV.NRORG = :P_NRORG_PADRAO)
WHERE
  EC.NRORG = :P_NRORG
  AND (EC.NRTABELA = :P_NRTABELA OR (EC.NRTABELA IS NULL AND :P_NRTABELA IS NULL))
  AND EC.NRCHAVE = :P_NRCHAVE
  AND EC.NRENVIO = :P_NRENVIO
  AND EC.NRTPAMBIENTE = 1
  AND EC.IDATIVO = 'S'
            ");
                
                 $query2->execute(
                    array(
                        "P_NRCHAVE"       => $item['NRCHAVE'],
                        "P_NRENVIO"       => $item['NRENVIOEXCL'],
                        "P_NRTABELA"      => $item['NRTABELA'],
                        "P_NRORG"         => $nrorg,
                        "P_NRORG_PADRAO"  => $nrorgpadrao
                    )
                );
                
                $itens2 = $query2->fetchAll();
            }
        }
        
        $resposta['EDICAO'] = true; $resposta['EXCLUSAO'] = true;
        
        //verifica se existe o evento S-3000 concluído para permitir a exclusão
        if (isset($itens2)) {
            foreach ($itens2 as $item2) {
                if ($item2['IDSTATUS'] === 'E' || $item2['IDSTATUS'] === 'R' || $item2['IDSTATUS'] === 'P' || $item2['IDSTATUS'] === 'M') {
                    $resposta['EDICAO'] = false;
                    $resposta['EXCLUSAO'] = false;
                } else if ($item2['IDSTATUS'] === 'C' || $item2['IDSTATUS'] === 'A') {
                    $resposta['EDICAO'] = true;
                    $resposta['EXCLUSAO'] = true;
                }
                if($resposta['EXCLUSAO'] === false || $resposta['EDICAO'] === false)
                    break;
            }
        } else if (count($itens) > 0) {
            foreach($itens as $item){
                if ($item['IDSTATUS'] === 'E' || $item['IDSTATUS'] === 'R' || $item['IDSTATUS'] === 'P') {
                    $resposta['EDICAO'] = true;
                    $resposta['EXCLUSAO'] = false;
                } else if ($item['IDSTATUS'] === 'C' || $item['IDSTATUS'] === 'A' || $item['IDSTATUS'] === 'M') {
                    $resposta['EDICAO'] = false;
                    $resposta['EXCLUSAO'] = false;
                }
                if($resposta['EXCLUSAO'] === false || $resposta['EDICAO'] === false)
                    break;
            }
        }
        
        if($editarexcluir == 'editar'){
            return $resposta['EDICAO'];
        } else if ($editarexcluir == 'excluir'){
            return $resposta['EXCLUSAO'];
        }
    }

    public function validaGozoFerias($dtIniPeriodo,$dtfimPeriodo){
        foreach ($this->getItems() as $item) {
            if (($dtIniPeriodo >= $item->getDtinigozoferias() && $dtIniPeriodo <= $item->getDtfimgozoferias())
                || ($dtfimPeriodo >= $item->getDtinigozoferias() && $dtfimPeriodo <= $item->getDtfimgozoferias())
                || ($item->getDtinigozoferias() >= $dtIniPeriodo && $item->getDtinigozoferias() <= $dtfimPeriodo)
                || ($item->getDtfimgozoferias() >= $dtIniPeriodo && $item->getDtfimgozoferias() <= $dtfimPeriodo)) {
                return false;
            }
        }
        return true;
    }

    public function procuraGozoFeriasIdColet(TkDateTime $dataInicioPeriodo,$identificador){
        foreach ($this->getItems() as $item) {
            if (($dataInicioPeriodo->equals($item->getDtinigozoferias(), TkDateTime::FORMAT_DATE) === true)
                && $item->getDsgozoferias()==$identificador) {
                return $item;
            }
        }
    }

    public function retornaPeriodoFerias ($data_ini, $data_fim){
        $aux = new TkDateTime($data_fim, TkDateTime::FORMAT_DATE);
        $aux2 = new TkDateTime($data_ini, TkDateTime::FORMAT_DATE);
        $periodo = $aux2->daysAfter($aux);
        return $periodo;
    }

    private function _inicializaArrayFerias(array $ferias, $diasPeriodo){
        for ($i=0; $i <= $diasPeriodo; $i++) {
            $ferias[$i] = -1;
        }

        return $ferias;
    }

    public function incluiGozoFerias(
        $nrOrg, $nrFerias, $dtIniGozo, $dtFimGozo, $qtdDiasAbono, $dtRetornoFerias, $idAdiant13Sal = 'N',$dsgozoferias=null
    ){
        try{
            $dtIniGozo = DateUtil::getDataDeString(substr($dtIniGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true);
            $dtFimGozo = DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true);
            $gozoFerias = new $this->fpaGozoFerias;
            
			$gozoFerias->setNrgozoferias(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_GOZOFERIAS', 1, 12, true));
            $gozoFerias->setNrorg($nrOrg);
            $gozoFerias->setNrferias($nrFerias);
            $gozoFerias->setDtinigozoferias($dtIniGozo);
            $gozoFerias->setDtfimgozoferias($dtFimGozo);
            $gozoFerias->setQtdiasabonoferi($qtdDiasAbono);
            $gozoFerias->setDtretornoferias($dtRetornoFerias);
            $gozoFerias->setIdadiant13salar($idAdiant13Sal);
            $gozoFerias->setDsgozoferias($dsgozoferias);
            $gozoFerias->setDtinclusao(DateUtil::getDataAtual());
            $gozoFerias->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $gozoFerias->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $gozoFerias->setDtultatu(DateUtil::getDataAtual());
            $gozoFerias->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $gozoFerias->setCdoperultatu($this->hcmParameters->getCdoperador());
            
            $this->entityManager->persist($gozoFerias);
            $this->entityManager->flush($gozoFerias);

            unset($gozoFerias);

        }catch(Exception $e){
            throw $e;
        }
    }

    public function retornaDiasFerias($data_ini_per, $data_fim_per){
        $ferias = array();

        $date_aux = $data_fim_per;
        $dias_periodo = DateUtil::getQtdeDiasEntre($data_ini_per, $date_aux) + 1;
        
        $ferias = $this->_inicializaArrayFerias($ferias, $dias_periodo);
        
        $itens = $this->getItems();
        if (count($itens) > 0){
            foreach ($itens as $item) {
                if ($item->getDtinigozoferias() !== false) {
                    $data_ini_aux = $item->getDtinigozoferias();
                    if ($data_ini_aux > $data_fim_per) {
                        // Afastamento comecou depois do periodo tratado.
                        continue;
                    }

                    try{
                        $data_fim_aux = $item->getDtfimgozoferias();
                    }catch (\Exception $e){
                        $data_fim_aux = null;
                    }

                    if (($data_fim_aux === null)
                        || ($data_fim_aux > $data_fim_per)
                    ) {
                        $data_fim_aux = $data_fim_per;
                        $this->_dtInicioGozo = $data_ini_aux;
                        $this->_dtFimGozo = $data_fim_aux;
                    }
                    
                    if ((($data_ini_aux > $data_ini_per)
                        ||($data_ini_aux->format('d/m/Y') == $data_ini_per->format('d/m/Y')))
                        &&(($data_ini_aux < $data_fim_per)
                        ||($data_ini_aux->format('d/m/Y') == $data_fim_per->format('d/m/Y')))
                    ) {
                        $this->_dtInicioGozo = $data_ini_aux;
                        $this->_dtFimGozo = $data_fim_aux;
                        $dia_ini = DateUtil::getQtdeDiasEntre($data_ini_per, $data_ini_aux);
                        
                        if ($dia_ini < 0){
                            $dia_ini = 0;
                        }
                        
						if(DateUtil::getQtdeDiasEntre($data_ini_aux, $data_fim_aux) == 0){
						    $qtd_dias = 1;
						} else{
						    $qtd_dias = DateUtil::getQtdeDiasEntre($data_ini_aux, $data_fim_aux) + 1;
						}
						for ($i=0; $i < $qtd_dias; $i++) {
                            $ferias[$i+$dia_ini] = 1;
                        }
                    } elseif (($data_ini_aux < $data_ini_per)
                        && (($data_fim_aux < $data_fim_per) ||  ($data_fim_aux->format('d/m/Y') == $data_fim_per->format('d/m/Y')))
                    ) {
                        if (($data_fim_aux > $data_ini_per)
                            || ($data_fim_aux->format('d/m/Y') == $data_ini_per->format('d/m/Y'))
                        ) {
                            $this->_dtInicioGozo = $data_ini_aux;
                            $this->_dtFimGozo = $data_fim_aux;
                        
							if(DateUtil::getQtdeDiasEntre($data_ini_per, $data_fim_aux) == 0){
							    $qtd_dias = 1;
							} else{
							    $qtd_dias = DateUtil::getQtdeDiasEntre($data_ini_per, $data_fim_aux) + 1;
							}
							
                            for ($i=0; $i < $qtd_dias; $i++) {
                                $ferias[$i] = 1;
                            }
                        } else {
                            true;
                        }
                    }
                }
            }
        } else {
           $this->_dtInicioGozo = DateUtil::getDataAtual(true);
           $this->_dtFimGozo = DateUtil::getDataAtual(true);
        }
        return $ferias;
    }

    public function trataGozoFerias($mes_competencia, $ferias, $dtIni = null, $dtFim = null){
        
        $pri_dia_mes = DateUtil::setDay(DateUtil::getDataDeString($mes_competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true), 1);
        
        $ult_dia_mes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mes_competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true));

        $pri_dia_mes_seg = DateUtil::adicionaIntervalo($pri_dia_mes, 1, DateUtil::MES);
        
        $ult_dia_mes_seg = DateUtil::getUltimoDiaDoMes(DateUtil::adicionaIntervalo($mes_competencia, 1, DateUtil::MES));
        
        $pri_dia_mes_seg_seg = DateUtil::adicionaIntervalo($pri_dia_mes_seg, 1, DateUtil::MES);
        
        $this->_dias_1a_parte = null;
        $this->_dias_2a_parte = null;
        $this->_dias_abono = 0;
        $this->_nrGozoFerias = null;
        
        $gozoFerias = $this->feriasEntity->getGozoFeriasDatas($ferias->getNrFerias(), $dtIni, $dtFim);
        
        foreach ($gozoFerias as $item) {
            $this->_nrGozoFerias = $item->getNrgozoFerias();
            $this->_dtInicioGozo = DateUtil::getDataDeString($item->getDtinigozoferias(), DateUtil::FORMATO_BRASILEIRO, true);
            $this->_dtFimGozo = DateUtil::getDataDeString($item->getDtfimgozoferias(), DateUtil::FORMATO_BRASILEIRO, true);
            $data_ini_aux = DateUtil::getDataDeString(substr($item->getDtinigozoferias(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $data_fim_aux = DateUtil::getDataDeString(substr($item->getDtfimgozoferias(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            if ($data_ini_aux < $pri_dia_mes) {
                
                $this->_mes_1a_parte = $mes_competencia;
                $this->_dias_1a_parte = 0;
                $this->_mes_2a_parte = $mes_competencia;
                
                if(DateUtil::getQtdeDiasEntre($pri_dia_mes, $data_fim_aux) == 0){
                    $this->_dias_2a_parte = 1;
                } else{
                    $this->_dias_2a_parte = DateUtil::getQtdeDiasEntre($pri_dia_mes, $data_fim_aux) + 1;
                }

                $this->_dias_abono = 0;
            }
            else {
                $this->_dias_abono = $item->getQtdiasabonoferi();
                if ($data_ini_aux < $ult_dia_mes
                    || $data_ini_aux->format('d/m/Y') === $ult_dia_mes->format('d/m/Y')
                ) {
                    $this->_mes_1a_parte = $mes_competencia;
                    if ($data_fim_aux > $ult_dia_mes) {

                        if(DateUtil::getQtdeDiasEntre($data_ini_aux, $ult_dia_mes) == 0){
                            $this->_dias_1a_parte = 1;
                        }else{
                            $this->_dias_1a_parte = DateUtil::getQtdeDiasEntre($data_ini_aux, $ult_dia_mes) + 1;
                        }

                        $this->_mes_2a_parte = $pri_dia_mes_seg;

                        if(DateUtil::getQtdeDiasEntre($pri_dia_mes_seg, $data_fim_aux) == 0){
                            $this->_dias_2a_parte = 1;
                        } else{
                            $this->_dias_2a_parte = DateUtil::getQtdeDiasEntre($pri_dia_mes_seg, $data_fim_aux) + 1;
                        }

                    } else {
                        $this->_dias_1a_parte = DateUtil::getQtdeDiasEntre($data_ini_aux, $data_fim_aux) + 1;
                        $this->_mes_2a_parte = $mes_competencia;
                        $this->_dias_2a_parte = 0;
                    }
                } else {
                    $this->_mes_1a_parte = $pri_dia_mes_seg;
                    if ($data_fim_aux > $ult_dia_mes_seg) {
                        $this->_dias_1a_parte = DateUtil::getQtdeDiasEntre($data_ini_aux, $ult_dia_mes_seg) + 1;
                        $this->_mes_2a_parte = $pri_dia_mes_seg_seg;
                        if(DateUtil::getQtdeDiasEntre($pri_dia_mes_seg_seg, $data_fim_aux) == 0){
                            $this->_dias_2a_parte = 1;
                        } else{
                            $this->_dias_2a_parte = DateUtil::getQtdeDiasEntre($pri_dia_mes_seg_seg, $data_fim_aux) + 1; 
                        }

                    } else {
                        $this->_dias_1a_parte = DateUtil::getQtdeDiasEntre($data_ini_aux, $data_fim_aux) + 1;
                        $this->_mes_2a_parte = $pri_dia_mes_seg;
                        $this->_dias_2a_parte = 0;
                    }
                }
            }
        }
    }

    public function retornaNrFeriasPeloGozo($inicio_periodo_saida, $fim_periodo_saida){
        $this->_nr_ferias_preparado = null;
        foreach ($this->getItems() as $item) {
            $data_ini_aux = DateUtil::getDataDeString($item->getDtinigozoferias(), DateUtil::FORMATO_BRASILEIRO, true);
            
            if (DateUtil::isBetween($data_ini_aux, $inicio_periodo_saida, $fim_periodo_saida)) {
                $this->_nr_ferias_preparado = $item->getNrFerias();
                return true;
            }
        }
        return false;
    }

    public function retornaItemPorNrFerias($nrferias, $nrorg){
        return Fpagozoferias::retornaItemPorNrFerias($nrferias, $nrorg);
    }

    public function insereGozoFerias($nrorg, $nrferias, $dtinigozo, $dtfimgozo, $qtddiasabono, $dtretornoferias,
                                     $idadiant13sal)
    {

        $this->setNrorg($nrorg);
        $this->setNrferias($nrferias);
        $this->setDtinigozoferias($dtinigozo);
        $this->setDtfimgozoferias($dtfimgozo);
        $this->setQtdiasabonoferi($qtddiasabono);
        $this->setDtretornoferias($dtretornoferias);
        $this->setIdadiant13salar($idadiant13sal);
    }

    public function getDiasAbonados() {
        $diasabono=$this->getQtdiasabonoferi();
        return $diasabono;
    }

    public function deletaGozoFerias($nrGozoFerias,$nrorgTrab,$nrFerias){
        $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->deletaGozoFerias($nrGozoFerias, $nrorgTrab, $nrFerias);
    }
    
    public function deletaGozoFeriasInicioFim($vinculo, $dtIniGozo, $dtFimGozo){
        $nrorg = $vinculo->getNrorg();
        
        $gozoFerias = $this->feriasEntity->retornaFeriasPeriodo(
            $nrorg,
            $vinculo->getNrvinculom(),
            DateUtil::getDataDeString(substr($dtIniGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true),
            $dtFimGozo = DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true)
        );
        
        if (empty($gozoFerias)) {
            return false;
        }
        
        $nrGozoFerias = $gozoFerias[0]['NRGOZOFERIAS'] ?? null;
        $nrFerias = $gozoFerias[0]['NRFERIAS'] ?? null;
        
        $validationResponse = $this->validateDeleteGozoFerias($nrorg, $nrGozoFerias);
        if ($validationResponse['status']) {
            $this->deletaGozoFerias($nrGozoFerias, $nrorg, $nrFerias);
        }
        
        return $validationResponse;
    }
    
    public function validateDeleteGozoFerias($nrorgTrab, $nrGozoFerias){
        $response = array('status' => false, 'message' => null);
        
        if ($this->existemCalculosDeFolhaAssociadosGozo($nrorgTrab, $nrGozoFerias)) {
            $response['message'] = Messages::HCM01000_FPA_GOZOFERIAS_EXCLUSAO_POSSUI_CALCULO;
            return $response;
        }
        
        $response['status'] = true;
        return $response;
    }

    public function persisteGozoFeriasAutomatico($vinculos, $dtIniGozo, $dtFimGozo, $idadiant13salar = 'N', $diasabono = null, $dsgozoferias = null, $nrferias = null,$dtAviso = null, $nrDiasFerias = null, $idnaopagaadic = null){
        try{
            $mensagem = array();
            $dtIniGozo = DateUtil::getDataDeString(substr($dtIniGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true);
            $dtFimGozo = DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true);
            $dtretorno = DateUtil::adicionaIntervalo($dtFimGozo, 1);
            
            $this->entityManager->beginTransaction();
    
            foreach($vinculos as $key => $vinculo) {
                // Bloqueio férias
                $vinculoRescindido = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $vinculo->getNrvinculom(), "nrorg" => $this->hcmParameters->getNrorgTrabalho()))->getDtrescisaovinc();
                
                if ($vinculoRescindido) {
                    $mensagem[] = array('msg' => 'O vínculo '.$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom().' está rescindido!');
                    continue;
                }
                
                //Bloqueia adicionar gozo de ferias se o vinculo ja possuir cadastro de gozo entre este periodo.
                $gozoFeriasJaCadastrado = $this->retornaGozosFeriasEntreDatas($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), $dtIniGozo, $dtFimGozo);
                
                if ($gozoFeriasJaCadastrado) {
                    $mensagem[] = array('msg' => 'Operação Bloqueada!! O vínculo '.$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom().' Já possui um período de gozo de férias que contempla um ou mais dias deste período!');
                    continue;
                }
                
                // End
                if($this->validaPersisteGozoFerias($vinculo,$dtIniGozo,$dtFimGozo)) {
                    $this->feriasEntity->retornaFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());					

                    $ferias = $this->feriasEntity->procuraFerias($nrferias);

                    if ($ferias){
						$dsgozoferias = '';
						$item = new $this->fpaGozoFerias;
						$item->setNrorg($vinculo->getNrorg());
						$item->setNrferias($ferias->getNrferias());
						$item->setDtinigozoferias($dtIniGozo);
						$item->setDtfimgozoferias($dtFimGozo);
						$item->setQtdiasabonoferi($diasabono);
						$item->setIdadiant13salar($idadiant13salar);
						$item->setIdnaopagaadic($idnaopagaadic);
						$item->setDtretornoferias($dtretorno);
						$item->setDtavisaoprevio($dtAviso);
						$item->getQtdiasferias($nrDiasFerias);
						$item->setDsgozoferias($dsgozoferias);
						$item->setNrgozoferias(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_GOZOFERIAS', 1, 12, true));
					    $item->setDtinclusao(DateUtil::getDataAtual());
                        $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                        $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
                        $item->setDtultatu(DateUtil::getDataAtual());
                        $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $item->setCdoperultatu($this->hcmParameters->getCdoperador());
					    
						$this->entityManager->persist($item);
						$this->entityManager->flush($item);
						$msg = 'Adicionado para o Vínculo '.$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom().' o Período de Gozo de '.substr($dtIniGozo,0,10).' a '.substr($dtFimGozo,0,10);
						$msg .= $diasabono > 0 ? ' dias Abono: '.$diasabono : null;
						$mensagem[] = array('msg' => $msg);
					} else {
						$mensagem[] = array('msg' => 'Vínculo '.$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom().' não possui período aquisitivo para inserir o gozo!'); 					
					}
                }
            }
            $this->msg = array_merge($this->msg, $mensagem);
            
            $this->entityManager->commit();
            return $this->msg;
        }catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function retrocedeGozoFeriasAutomatico($vinculos, $dtIniGozo, $dtFimGozo) {
        try {
            $dtIniGozo = DateUtil::getDataDeString(substr($dtIniGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true);
            $dtFimGozo = DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true);
            
            $this->entityManager->beginTransaction();
            
            foreach($vinculos as $key => $vinculo) {
                $ferias = $this->feriasEntity->retornaFeriasPeriodo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $dtIniGozo, $dtFimGozo);					
                
                //verifica se há algum evento S-2230 em andamento ou sem um S-3000 concluído para possibilitar a alteração do dado
                $mensagem = $this->validacaoESocial($ferias[0]['NRGOZOFERIAS'], 'excluir');
                
                if ($mensagem === true) {
                    if ($ferias) {
                        $gozoFeriasDeletadoResponse = $this->deletaGozoFeriasInicioFim($vinculo, $dtIniGozo, $dtFimGozo);
                        if ($gozoFeriasDeletadoResponse['status']) {
                            $this->msg[] = array('msg' => 'Vínculo '.$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom().' teve gozo de férias retrocedido com sucesso!');
                        } else {
                            $this->msg[] = array('msg' => 'Os registros de gozo de férias do vínculo '.$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom(). ' NÃO foram excluídos. ' . $gozoFeriasDeletadoResponse['message']);
                        }
                        
                    } else {
                        $this->msg[] = array('msg' => 'Vínculo '.$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom().' não possui gozo no período selecionado!');
                    }
                } else {
                    $mensagem = Messages::evaluateMessage(MESSAGES::HCM00000_EDITAR_EXCLUIR_REGISTRO_AFASTAMENTO, $ferias[0]['NRGOZOFERIAS']);
                    $this->msg[] = array("msg" => $mensagem);
                }
            }
            
            $this->entityManager->commit();            
            return $this->msg;
            
        } catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    
    public function existemCalculosDeFolhaAssociadosPeriodoGozo($vinculo, $dtIniGozo, $dtFimGozo) {
        $gozoFerias = $this->feriasEntity->retornaFeriasPeriodo(
            $vinculo->getNrorg(),
            $vinculo->getNrvinculom(),
            DateUtil::getDataDeString(substr($dtIniGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true),
            $dtFimGozo = DateUtil::getDataDeString(substr($dtFimGozo,0,10), DateUtil::FORMATO_BRASILEIRO, true)
        );
        
        if (empty($gozoFerias)) {
            return false;
        }
        
        return $this->existemCalculosDeFolhaAssociadosGozo($vinculo->getNrorg(), $gozoFerias[0]['NRGOZOFERIAS']);
    }
    
    public function existemCalculosDeFolhaAssociadosGozo($nrorgTrab, $nrGozoFerias) {
        $itens = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findBy(
            array(
                'nrorg'          => $nrorgTrab,
                'nrgozoferias'   => $nrGozoFerias,
            )
        );
        
        $nrCalculosFolha = array_map(function ($item) {
            return $item->getNrcalculofolha();
        }, $itens);
        
        $nrCalculosFolha = array_unique($nrCalculosFolha);
        
        $calculos = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findBy(
            array(
                'nrorg'          => $nrorgTrab,
                'nrtipomoviment' => '2',
                'nrcalculofolha' => $nrCalculosFolha
            )
        );
        
        $existemCalculosGozoFerias = count($calculos) > 0;
        return $existemCalculosGozoFerias;
    }

   public function validaPersisteGozoFerias($vinculo, $dtinigozo, $dtfimGozo){
        $this->msg = array();
        if ($this->alteSituFuncEntity->verificaEstabilidade($vinculo->getNrorg(), $this->hcmParameters->getDtCompetencia(), $vinculo->getNrvinculom(), $dtinigozo) ||
            $this->alteSituFuncEntity->verificaVinculoAfastado($vinculo->getNrorg(), $vinculo->getNrvinculom(), $dtinigozo))
        {
            $this->msg[] = array('msg' => "O vínculo ".$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom(). " está com a situação funcional inconsistente impossibilitando-o de tirar férias.", "IDTIPOMSG" => 'Erro');
            return false;
        }
        
        $this->retornaGozoFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());
        $itemGozo = $this->validaGozoFerias($dtinigozo,$dtfimGozo);
        if(!$itemGozo){
            $this->msg[] = array('msg' => "Já existe um período de gozo lançado para o vínculo ".$vinculo->getNrvinculom().' '.$vinculo->getNmvinculom()." coincidindo com o periodo: ".substr($dtinigozo, 0, 10)." - ".substr($dtfimGozo, 0, 10), "IDTIPOMSG" => 'Erro');
            return false;
        }
        return true;
    }

    
    public function getMensagensGozoInvalido(){
        return $this->msg; 
    }
}