<?php

namespace HCM\FPA\Repository;

use Doctrine\ORM\EntityManager;

class ContaContabilRepository {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function verificaExclusao($nrtamnivel, $cdcontctbl,$nrorg) {
        $params                 = array();
        $params['P_NRTAMNIVEL'] = $nrtamnivel;
        $params['P_CDCONTCTBL'] = $cdcontctbl;
        $params['P_NRORG'] = $nrorg;
   
        $sql = "SELECT CDCONTCTBL
                FROM CONTCTBL
                WHERE   (CDNVCONTCTBL = :P_NRTAMNIVEL + 1) 
                    AND (CDCONTCTBL like :P_CDCONTCTBL || '%')
                    AND NRORG = :P_NRORG
                    ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function retornaMaiorConta($pai, $nrorg) {
        $params                 = array();
        $params['PAI']   = $pai;
        $params['NRORG'] = $nrorg;
   
        $sql = "SELECT MAX(CDCONTCTBL) AS CDCONTCTBL FROM CONTCTBL
                    WHERE CDCONTANTE = :PAI
                        AND NRORG =  :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function validaSubContas($cdcontctbl, $nrseqgrupo, $cdcontacorre, $nrseqnatu) {
        $params                   = array();
        $params['P_CDCONTCTBL']   = $cdcontctbl;
        $params['P_NRSEQGRUPO']   = $nrseqgrupo;
        $params['P_CDCONTACORRE'] = $cdcontacorre;
        $params['P_NRSEQNATU']    = $nrseqnatu;
   
        $sql = "SELECT NRSEQUENCIAL
                FROM CONTCTBLCORRE
                WHERE CDCONTCTBL = :P_CDCONTCTBL
                    AND NRSEQGRUPO = :P_NRSEQGRUPO
                    AND CDCONTACORRE = :P_CDCONTACORRE
                    AND NRSEQNATU = :P_NRSEQNATU";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function verificaContaContabil($cdcontctbl, $nrorg) {
        $params                   = array();
        $params['CDCONTCTBL']   = $cdcontctbl;
        $params['NRORG']    = $nrorg;
   
        $sql = "SELECT *
                FROM CONTCTBL
                WHERE CDCONTCTBL = :CDCONTCTBL
                    AND NRORG    = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function validaCodigoReduzido($cdcontredu, $nrorg) {
        $params               = array();
        $params['CDCONTREDU'] = $cdcontredu;
        $params['NRORG']      = $nrorg;
   
        $sql = "SELECT *
                FROM CONTCTBL
                WHERE CDCONTREDU = :CDCONTREDU
                    AND NRORG    = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function cadastroAutoIdentificadorPadrao($idpadrrel, $containi, $contafin, $nrorg) {
        $params                = array();
        $params['P_IDPADRREL'] = $idpadrrel;
        $params['P_CONTAINI']  = $containi;
        $params['P_CONTAFIN']  = $contafin;
        $params['NRORG']       = $nrorg;
        
        $sql = " UPDATE CONTCTBL
                    SET IDPADRREL    = :P_IDPADRREL
                    WHERE CDCONTCTBL BETWEEN :P_CONTAINI AND :P_CONTAFIN
                        AND NRORG        = :NRORG 
                        AND IDTIPOCONT   = 'A'" ;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function cadastroAutoContaVertical($contabase, $containi, $contafin, $nrorg) {
        $params                   = array();
        $params['P_CONTABASE']    = $contabase;
        $params['P_CONTAINICIAL'] = $containi;
        $params['P_CONTAFINAL']   = $contafin;
        $params['NRORG']          = $nrorg;
        
        $sql = "UPDATE CONTCTBL
                    SET CDCONTCTBLANALI = :P_CONTABASE
                    WHERE CDCONTCTBL BETWEEN :P_CONTAINICIAL AND :P_CONTAFINAL
                        AND NRORG        = :NRORG 
                        AND IDCONTCCUS <> 'R'  
                        AND CDNVCONTCTBL <> '1'" ;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function cadastroAutoContaReferencialEcd($contantiga, $contnova, $nrorg) {
        $params                 = array();
        $params['P_CONTNOVA']   = $contnova;
        $params['P_CONTANTIGA'] = $contantiga;
        $params['NRORG']        = $nrorg;
        
        $sql = "UPDATE CONTCTBL
                SET CDCONTPLANREFE   = :P_CONTNOVA
                WHERE CDCONTPLANREFE = :P_CONTANTIGA
                    AND NRORG        = :NRORG " ;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function cadastroAutoContaReferencialAux($contantiga, $contnova, $nrorg) {
        $params                   = array();
        $params['P_CONTNOVA']   = $contnova;
        $params['P_CONTANTIGA'] = $contantiga;
        $params['NRORG']        = $nrorg;
        
        $sql = "UPDATE CONTCTBL
                SET CDCONTREFEAUX   = :P_CONTNOVA
                WHERE CDCONTREFEAUX = :P_CONTANTIGA
                AND NRORG           = :NRORG ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function salvaMascara ($mascara, $nrorg) {
        $params = array('P_CDPICTCCTB' => $mascara, 'P_NRORG' => $nrorg);
        
        $sql = <<<SQL
UPDATE PARAMGERAL SET CDPICTCCTB = :P_CDPICTCCTB WHERE NRORG = :P_NRORG
SQL;
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getMascara ($nrorg) {
        $params= array('P_NRORG' => $nrorg);
   
        $sql = "SELECT CDPICTCCTB FROM PARAMGERAL WHERE NRORG = :P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

}