<?php
namespace HCM\FPA\Services;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Geral\Controllers\BirtReportController;
use HCM\Geral\Services\VinculoServices;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;
use HCM\Geral\Controllers\EnviarMensagemController;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\Geral\Controllers\RelatoriosHcmController;
use HCM\FPA\Repository\FechamentoMensalRepository;
use HCM\Geral\Services\IntegracaoDigitalCertisignService;
use HCM\Util\NewFileServer\FileServerClasses\Util\MyCurl;
use HCM\Geral\Services\EnviarMensagemService;
use Doctrine\ORM\EntityManager;

class FechamentoMensalService {
    
    const Alfabetica    = 'ALFABETICA';
    const Data_Admissao = 'DATA_ADMISSAO';
    const Matricula     = 'MATRICULA';
    const Estrutura     = 'ESTRUTURA';
    protected $dataSourceGetFechamentoMensal = 'fpaFormCalculo_getFechamentoMensal';
    protected $dataSourceSetFechamentoMensal = 'fpaFormCalculo_setFechamentoMensal';
    protected $parameterBagColumns = array('NRORG', 'CDOPERADOR', 'IDTPFORMCALCULO', 'NRORGPADRAO');
    protected $entityManager;

    public function __construct (EntityManager $entityManager, ParameterBag $parameterBag, BirtReportController $birtReportController,
        VinculoServices $vinculoServices, EnviarMensagemController $enviarMensagemController, HCMParameters $hcmParameters, RelatoriosHcmController $relatoriosHcmController, 
        FechamentoMensalRepository $fechamentoMensalRepository, IntegracaoDigitalCertisignService $integracaoDigitalCertisignService, EnviarMensagemService $enviarMensagemService){ 
        $this->parameterBag               = $parameterBag;
        $this->birtReportController       = $birtReportController;
        $this->vinculoServices            = $vinculoServices;
        $this->enviarMensagemController   = $enviarMensagemController;
        $this->hcmParameters              = $hcmParameters;
        $this->relatoriosHcmController    = $relatoriosHcmController;
        $this->fechamentoMensalRepository = $fechamentoMensalRepository;
        $this->certisignService           = $integracaoDigitalCertisignService;
        $this->entityManager = $entityManager;
        $this->enviarMensagemService = $enviarMensagemService;
    }

    /**
     * Função auxiliar cujo propósito é criar o relatório para o vínculo informado.
     */
    public function criarHolerite($directoryTemp, $vinculo, $mesCompetencia, $nrOrg, &$returnSemLiquido, &$row, $hcmParameters){
        $row->getRow()['NRVINCULOM'] = $vinculo->getNrvinculom();
        $competencia = substr(str_replace('/', '', $mesCompetencia), 2);
        $possuiCalculo = $this->vinculoServices->verificaCalculoMovimentoVinculo($vinculo->getNrvinculom(), $nrOrg, $mesCompetencia, $row->getRow()['NRTPMODALIDCAL'], $row->getRow()['OCORRECAL'], $row->getRow()['NRTIPOMOVIMENT']);            
        if (!$possuiCalculo) {
            if(!is_null($returnSemLiquido)){
                $returnSemLiquido .= ' ';
            }
            $returnSemLiquido .= $vinculo->getNrvinculom().', ';
            return 'sem_calculo';
        }                
        
        // novo response para gerar url do relatório
        $responseReport = new DTO\Response();
        $this->birtReportController->createRemoteReport($row, $responseReport);
        $messages    = $responseReport->getMessages();
        $reportUrl   = end($messages)->getMessage();
        $fileName = $vinculo->getNrvinculom().'_'.$row->getRow()['NMTIPOMOVIMENT'].'_'.$competencia;
        $carregaRelatorio = self::carregaRelatorio($reportUrl, $vinculo->getNrvinculom());
        
        $pessoaEntity = new GpepessoaEntity(\Setforms2\Shared\Registry::getEntityManager(), $hcmParameters);
        
        $pessoa = $pessoaEntity->retornaPessoaAtual($vinculo->getNrpessoa());
        $senha = $pessoa->historico->getNrcpfpessoa();
        
        file_put_contents($directoryTemp.$fileName.'.pdf', $carregaRelatorio);
        
        /*$z = new \ZipArchive();
        $zipArquive = $z->open($directoryTemp.$fileName.'.zip', \ZipArchive::CREATE);
        $cmd = 'zip -P '.$senha.' -r '. $directoryTemp.$fileName.'.zip '.$directoryTemp.$fileName.'.pdf';
        system($cmd);
        unlink($directoryTemp.$fileName.'.pdf');
        return $fileName.'.zip';*/
        
        return $fileName.'.pdf';
    }
    
    public function getUrlRelatorio($nrVinculom, $fileType, $filtro){
        $urlReport = $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, $fileType, null, null);
        $urlReport = str_replace(" ", "%20", $urlReport);
        return $urlReport;
    }
    
    public function gerarRelatorioGenerico($urlReport, $fileName, $nrVinculom, $extension){
        $fileNamePath = $this->hcmParameters->getTempDirectory().'/'.$fileName. '_'. $nrVinculom . '_'. '.'.$extension;
        $fileContents = file_get_contents($urlReport);
        file_put_contents($fileNamePath, $fileContents);
        // file_put_contents($fileNamePath, 'teste');
        return $fileNamePath;
    }
    
    public function enviarRelatorioGenericoViaEmail($row, $nrOrg, $mesCompetencia, $vinculo, $fileNamePath){
        $this->enviarMensagemController->enviarEmailDemonstrativo($row->getRow()['MENSAGEM'], $nrOrg, $mesCompetencia, $vinculo->getNrvinculom(), $vinculo->historico->getNrestrutlegal(), $fileNamePath, NULL, 'ignoraTabelaOperador');        
    }

    public function enviarRelatorioGenericoViaCertiSign($row, $nrOrg, $mesCompetencia, $vinculo, $fileNamePath){
        $dataSet = $row->getRow();
        
        $rowAux = $this->generateAuxRow($fileNamePath, $dataSet, $nrOrg, $vinculo, '');
        
        $return = $this->certisignService->saveAssinaturaEletronica($rowAux);
    }
    
    /**
     * Função auxiliar responsável por fazer o envio do holerite para o vínculo informado
     */
    public function enviarHoleriteParaVinculo($directoryTemp, $row, $nrOrg, $mesCompetencia, $vinculo, $fileName, &$returnVinculo, &$returnDsEmailOper, &$returnAmbos, &$returnSemLiquido, &$validaTabelaOper){
        $return = $this->enviarMensagemController->enviarEmailDemonstrativo($row->getRow()['MENSAGEM'], $nrOrg, $mesCompetencia, $vinculo->getNrvinculom(), $vinculo->historico->getNrestrutlegal(), $directoryTemp.$fileName, NULL, 'ignoraTabelaOperador');
        if (is_bool($return)) {
            unlink($directoryTemp.$fileName);
            $returnVinculo[] = $vinculo->getNmvinculom();
        } else {
            $returnConst = null;
            if(strlen($return) > 17)
                $returnConst = substr($return, 0, 17);
            if($return == 'DSEMAILOPER'){
                if(!is_null($returnDsEmailOper)){
                    $returnDsEmailOper .= ' ';
                }
                $returnDsEmailOper.= $vinculo->getNmvinculom();
            } else if ($return == 'AMBOS'){
                if(!is_null($returnAmbos)){
                    $returnAmbos .= ' ';
                }
                $returnAmbos .= $vinculo->getNmvinculom();
            } else if($returnConst == 'Email do operador'){
                $validaTabelaOper .= $return;
                $validaTabelaOper.='<br>';
            } 
        }        
    }
    
    /**
     * Função auxiliar responsável por fazer o envio do holerite para o vínculo informado para o CertiSign
     */
    public function enviarHoleriteParaVinculoCertiSign($directoryTemp, $row, $nrOrg, $mesCompetencia, $vinculo, $fileName, &$returnVinculo, &$returnDsEmailOper, &$returnAmbos, &$returnSemLiquido, &$validaTabelaOper){
        $dataSet = $row->getRow();
        
        $rowAux = $this->generateAuxRow($directoryTemp, $dataSet, $nrOrg, $vinculo, $fileName);
        
        $return = $this->certisignService->saveAssinaturaEletronica($rowAux);
        
        $result = true;
        if(gettype($nrOrg) != 'NULL' && gettype($vinculo->getNrvinculom()) != 'NULL'){
            $oper = $this->enviarMensagemService->getOperadorVinculo($nrOrg, $vinculo->getNrvinculom());
            if (!$oper) {
                $result = 'AMBOS';
            } else {
                if (is_null($oper['DSEMAILOPER'])) {
                    $result = 'DSEMAILOPER';
                }
            }         
        }

        if (is_bool($result)) {
            unlink($directoryTemp.$fileName);
            $returnVinculo[] = $vinculo->getNmvinculom();
        } else {
            $returnConst = null;
            if(strlen($result) > 17)
                $returnConst = substr($result, 0, 17);
            if($result == 'DSEMAILOPER'){
                if(!is_null($returnDsEmailOper)){
                    $returnDsEmailOper .= ' ';
                }
                $returnDsEmailOper.= $vinculo->getNmvinculom();
            } else if ($result == 'AMBOS'){
                if(!is_null($returnAmbos)){
                    $returnAmbos .= ' ';
                }
                $returnAmbos .= $vinculo->getNmvinculom();
            } else if($returnConst == 'Email do operador'){
                $validaTabelaOper .= $result;
                $validaTabelaOper.='<br>';
            } 
        }
    }
    
    /**
     * Função auxiliar responsável por enviar os holerites para o responsável da Estrutura Gerencial
     */
    public function enviarHoleritesParaResponsavelInformal($directoryTemp, $row, $nrOrg, $mesCompetencia, $groupEstruturasG, &$sucesso){
        foreach ($groupEstruturasG as $key=>$estruturaG) {
            $zipName = $this->ziparHolerites($directoryTemp, $key, $estruturaG);
            if( ($zipName == 'abrir o') || ($zipName == 'adicionar no') ){
                return true;
            }else{
                if(count($estruturaG['email'])>1){
                    foreach($estruturaG['email'] as $email){
                        $return = $this->enviarMensagemController->enviarEmailDemonstrativo($row->getRow()['MENSAGEM'], NULL, $mesCompetencia, NULL, $estruturaG['estruturaL'], $directoryTemp.$zipName, $estruturaG, 'ignoraTabelaOperador');
                    }
                }else{
                    $return = $this->enviarMensagemController->enviarEmailDemonstrativo($row->getRow()['MENSAGEM'], NULL, $mesCompetencia, NULL, $estruturaG['estruturaL'], $directoryTemp.$zipName, $estruturaG, 'ignoraTabelaOperador');
                }
                if(!empty($sucesso)){
                    $sucesso .= ' ';
                }
                $sucesso.= $key.', ';                    
                unlink($directoryTemp.$zipName);
            }    
        }
        if(!empty($sucesso))
            $sucesso = substr($sucesso, 0, -2);
        return false;    
    }

    /**
     * Função auxiliar cujo objetivo é compactar os muitos holeries, antes de enviar para 
     * o responsável informal da estrutura gerencial
     */
    public function ziparHolerites($directoryTemp, $key, $estruturaG){
        $holerites = explode(',', $estruturaG['holerites']);
        array_pop($holerites);
        $zip = new \ZipArchive();
        $zipName = $key."_"."holerites.zip";
        if (!$zip->open($directoryTemp.$zipName, \ZipArchive::CREATE)) {
            return 'abrir o';
        }   
        foreach ($holerites as $holerite) {
            if(!$zip->addFile($directoryTemp.$holerite, $holerite)) {
                $zip->close();
                return 'adicionar no';
            }            
        }
        $zip->close();
        return $zipName;
    }
    
    /**
     * Função auxiliar com o propósito de percorrer o array de holerites, e exclui-los
     * do servidor.
     */
    public function excluirArquivos($directoryTemp, $groupEstruturasG){
        foreach ($groupEstruturasG as $estrutura) {
            $holerites = explode(',', $estrutura['holerites']);
            array_pop($holerites);
            foreach ($holerites as $holerite) {
                unlink($directoryTemp.$holerite);
            }
        }
    }
    
    public function carregaRelatorio($url, $nrvinculom) {
        $cURLRequest = new cURLRequest($url);
        $report = $cURLRequest->request('');

        $requestInfo = $cURLRequest->getRequestInfo();    
        if ($requestInfo['http_code'] !== HTTPInterface::STATUS_CODE_OK) {
            throw new \Exception(Messages::evaluateMessage(Messages::HCM01000_FPA_ERROR_SAVE_REPORT_VINCULO, $nrvinculom), 1);
        }  
        
        return $report;
    }
    
    public function possuiPonto($nrorg, $nrvinculom, $nrperiodoapuracao){
        return $this->fechamentoMensalRepository->possuiPonto($nrorg, $nrvinculom, $nrperiodoapuracao);
    }

    public function generateAuxRow($directoryTemp, $row, $nrOrg, $vinculo, $fileName) {
        if(substr($directoryTemp, 0, 5) == '/tmp/' && strlen($directoryTemp) > 5) {
            $fileContent = file_get_contents($directoryTemp);
            $fileName = substr($directoryTemp, 5, strlen($directoryTemp));
        } else {
            $fileContent = file_get_contents($directoryTemp.$fileName);
        }
        
        $fileContent = 'data:application/pdf;base64,'.base64_encode($fileContent);
        $nrPessoa = $vinculo->getNrpessoa();

        $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy([
            "nrpessoa" => $nrPessoa,
            "nrorg" => $nrOrg,
        ]);

        $nrparcnegocio = $pessoa->getNrparcnegocio();
        
        $rowAux = array(
            "NRVINCULOM" => isset($row['NRVINCULOM']) ? $row['NRVINCULOM'] : $vinculo->getNrvinculom(),
            "NRORG" => $nrOrg,
            "FILE" => $fileContent,
            "IDTIPOASSINATURA" => $row['IDTIPOASSINATURA'],
            "NMFILE" => $fileName,
            "NMDOCUMENTO" => isset($row['NMDOCUMENTO']) ? $row['NMDOCUMENTO'] : $fileName,
            "IDATIVO" => "S",
            "IDSATUS" => "P",
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRCATEGDOCUMENTO" => $row['NRCATEGDOCUMENTO']
        );
        
        return $rowAux;
    }

    public function processConditionValue($condition) {
        return $condition['value'];
    }

    public function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}