<?php

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\{ DataSet, FilterCriteria, Manager, ParameterBag };
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use HCM\Geral\Services\{ CrudService, QuadroVagasService };
use HCM\Util\{ DateUtil, HCMParameters, Messages, NovoCodigo, Repositories, Util };

class QuadroVagasController extends Crud {

    protected $dataSourceName_get = 'getQuadroVagas';
    protected $dataSourceName_set = 'setQuadroVagas';
    
    protected $parameterBagColumns = array('NRORG', 'P_NRORG_PADRAO', 'P_NRORG', 'P_DTMESCOMPETENC', 'QTD_VINC_ATIVOS_OCUPACAO', 'QTD_VAGAS');

    public function __construct (Manager $dataSourceManager, EntityManager $entityManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, CrudService $crudService, QuadroVagasService $quadroVagasService){
        $this->entityManager = $entityManager;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
		$this->crudService = $crudService;
		$this->quadroVagasService = $quadroVagasService;
        parent::__construct($dataSourceManager);
    }

    public function getQuadroVagas(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName_get);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $this->processNewRowsData($dataSet->getRows(), $filterData);
            $response->addDataSet(new DataSet($filterData->getDataSourceName(), $rows));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function processNewRowsData(array $rows, FilterCriteria $filterData): array {
        $alteredRows = $this->quadroVagasService->includeNewRowsData($rows);
        return $this->quadroVagasService->filterNewRowsData($alteredRows, $filterData);
    }
    
    public function setQuadroVagas(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = $this->dataSourceName_set;
            $rows = $request->getDataSet()->getRows();
            
            if ($this->quadroVagasService->areRowsInvalid($rows, $response)) {
                return;
            }
            
            foreach($rows as &$row) {
                if ($row['__is_new']) {
                    $row['NRQUADROVAGAS'] = NovoCodigo::geraCodigo($row['NRORG'], 'GPE_QUADROVAGAS', 1, 12, true);
                }
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cadAutoQuadroVagas(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $arraylistMensagemRetorno = [];
            
            foreach($request->getDataSet()->getRows() as $row){
                $addousubstituiQuadroVagas =$row['ADDOUSUBSTCADAUTO'];
                $nrorg = $row['NRORG'];
                $cdproposta = $row['CDPROPOSTA'];
                $dtinivalidade = DateUtil::getDataDeString(substr($row['DTINIVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                $dtfimvalidade = DateUtil::getDataDeString(substr($row['DTFIMVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                $nrestruturam_list = $row['NRESTRUTURAM'];
                $qtdlimite = $row['QTDLIMITE'];
                $nrocupacaom = $row['NROCUPACAOM'];
                $nrquadrovagas =  $row['NRQUADROVAGAS'];
                $vrExame = $row['VREXAME'];
                $vrTotalMensalExame = $row['VRTOTALMENSALEXAME'];
                $nrescalatrabm = $row['NRESCALATRABM'] ? $row['NRESCALATRABM'] : null;
                
                $msgRetorno = $this->quadroVagasService->cadastroAutomatico($cdproposta,$dtinivalidade,$dtfimvalidade,$nrestruturam_list, $qtdlimite,$nrocupacaom, $nrorg, $nrquadrovagas, $addousubstituiQuadroVagas,  $vrExame, $vrTotalMensalExame, $nrescalatrabm);

                $arraylistMensagemRetorno = array_merge($arraylistMensagemRetorno, $msgRetorno);
            };

            $response->addDataSet(new DataSet("cod", $arraylistMensagemRetorno));


        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteQuadroVagas(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = $this->dataSourceName_set;
            
            foreach($request->getDataSet()->getRows() as &$row) {
                Util::deactivateRow($row);
            }
                
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function ativateQuadroVagas(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = $this->dataSourceName_set;
            
            foreach($request->getDataSet()->getRows() as &$row) {
                Util::activateRow($row);
            }
                
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}