<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\DataSet;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\RemuneracaoOutrasFontesService;
use HCM\Util\Util;

class RemuneracaoOutrasFontesController extends Crud {

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, RemuneracaoOutrasFontesService $remuneracaoOutrasFontesService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager; 
        $this->remuneracaoOutrasFontesService = $remuneracaoOutrasFontesService;
        parent::__construct($dataSourceManager);
    }

    public function getRemuneracaoOutrasFontes(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "remuneracaoOutrasFontes_get";
        $filterData = $request->getFilterCriteria();

        foreach ($filterData->getConditions() as $value) {
            if($value['columnName'] == "CDINSCOUTREMPR") {
                $cdInscOutrEmprValue = Util::removeOutrosCaracteresEspeciais($value['value']);
                
                $oldFilterDataProperties = Util::returnFilterDataProperties($filterData);
                Util::getValueInFilterDataSourceByName($filterData, ["CDINSCOUTREMPR"], true);
                $filterData = Util::createNewFilterData($oldFilterDataProperties[0], $filterData, $oldFilterDataProperties[2], $oldFilterDataProperties[3]);
                $filterData->addCondition('CDINSCOUTREMPR', 'LIKE_I', $cdInscOutrEmprValue);
            }
        }

        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }


    public function setRemuneracaoOutrasFontes(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "remuneracaoOutrasFontes";
            foreach ($request->getDataSet()->getRows() as $row) {
                $row['IDATIVO'] = $row['IDATIVO']?? 'S';
                $row['CDCODCATEG'] = $row['CDCODCATEG']?? NULL;
                $row['CDINSCOUTREMPR'] =  preg_replace('/\D/', '', $row['CDINSCOUTREMPR']);

                $existeRemuneracao = $this->remuneracaoOutrasFontesService->existeRemuneracao($row);

                if(!empty($existeRemuneracao) && ($existeRemuneracao['NRREMUNOUTREMPR'] != $row['NRREMUNOUTREMPR'] || !isset($row['NRREMUNOUTREMPR'])) ) {
                    $response->addNotification(new Notification("Já existe um registro de remuneração para esta pessoa, com a mesma competência, inscrição da fonte pagadora e tipo de movimento", Notification::TYPE_ALERT));
                    return;
                }
               
                if($row['__is_new'] === true){
                   $row['NRREMUNOUTREMPR'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_REMUNOUTREMPR', 1, 12, true); 
                }

                parent::save($request, $response);
            }
        }catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function deleteRemuneracaoOutrasFontes(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "remuneracaoOutrasFontes";
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error(empty($error) ? $e->getMessage() : $error, empty($error) ? $e->getCode() : 5));
        }  
    }

    public function baixarPlanilhaRemuneracaoOutrasFontes(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $file = $this->remuneracaoOutrasFontesService->baixarPlanilhaRemuneracaoOutrasFontes();
                
            if($file != null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaRemuneracaoOutrasFontes(DTO\Request\DataSet $request, DTO\Response $response)
    {
        set_time_limit(0);
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $messages = $this->remuneracaoOutrasFontesService->importarPlanilhaRemuneracaoOutrasFontes($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 