<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;


class AlteEscalaRepository {

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function retornaUltimaAlteEscalaPorDataInicio($nrorg, $nrvinculom, $dtiniescala)
    {
        try {
            $query = <<<SQL
SELECT *
  FROM GPE_ALTEESCALA
 WHERE NRORG = :P_NRORG
   AND NRVINCULOM = :P_VINCULOM
   AND DTINIESCALA = ( SELECT MAX(DTINIESCALA)
                         FROM GPE_ALTEESCALA B
                        WHERE B.NRORG = :P_NRORG
                          AND B.NRVINCULOM = :P_VINCULOM
                          AND B.DTINIESCALA <= :P_DTINIESCALA )
SQL;
            $params = array(
                'P_NRORG'       => $nrorg,
                'P_VINCULOM'    => $nrvinculom,
                'P_DTINIESCALA' => $dtiniescala,
            );
            return $this->connection->fetchAll($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaProximaAlteEscala($nrorg, $nrvinculom, $dtiniescala) {

        $params = array($nrorg, $nrvinculom, $dtiniescala);

        $query = <<<SQL
SELECT *
  FROM GPE_ALTEESCALA
 WHERE NRORG       = ?
   AND NRVINCULOM  = ?
   AND DTINIESCALA = (SELECT MIN(DTINIESCALA)
                        FROM GPE_ALTEESCALA B
                       WHERE B.NRORG       = GPE_ALTEESCALA.NRORG
                         AND B.NRVINCULOM  = GPE_ALTEESCALA.NRVINCULOM
                         AND B.DTINIESCALA > TO_DATE(?, 'DD/MM/YYYY'))
SQL;
        return $this->connection->fetchAll($query, $params);
    }
    
    public function getTurnoParOuImpar(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT *
            FROM GPE_TURNO
            WHERE NRORG = :P_NRORG
            AND NRESCALATRABM = :P_NRESCALATRABM
            AND ROWNUM = 1
            AND EXISTS (SELECT 1
                            FROM ( SELECT MAX(NRSEQHORARIO) NRSEQHORARIO
                                    FROM GPE_HORARIOTURNO
                                    WHERE NRORG       = :P_NRORG
                                    AND NRTURNO       = GPE_TURNO.NRTURNO
                                    AND IDATIVO       = 'S') AUX 
                            JOIN DATA DT ON DT.DTDATA = TO_DATE( :P_DIA )
                            JOIN GPE_HORARIOTURNO HT ON HT.NRORG        = :P_NRORG
                                        AND HT.NRTURNO      = GPE_TURNO.NRTURNO
                                        AND HT.IDATIVO      = 'S'
                                        AND HT.NRSEQHORARIO = NVL( NULLIF( MOD( TO_CHAR( trunc(DT.DTDATA,'MM')+:P_NUMBER - 4, 'J' ), AUX.NRSEQHORARIO ), 0 ), AUX.NRSEQHORARIO )
                            JOIN GPE_HORDIAH HDH ON HDH.NRORG = :P_NRORG 
                                    AND HDH.NRHORDIAM = HT.NRHORDIAM
                                    AND HDH.IDTIPOHORARIO <> 'F'
                                    AND HDH.DTMESCOMPETENC = (
                                                    (SELECT DTMESCOMPETENC FROM 
                                                        (SELECT 
                                                            HDHH.DTMESCOMPETENC 
                                                        FROM 
                                                            GPE_HORDIAH HDHH
                                                        WHERE 
                                                            HDHH.NRORG = HDH.NRORG 
                                                            AND HDHH.NRHORDIAM = HDH.NRHORDIAM 
                                                        ORDER BY HDHH.DTMESCOMPETENC DESC) 
                                                    WHERE ROWNUM = 1)
                                                )
                        )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $turnos = $std->fetch();

        return !empty($turnos) ? $turnos : []; 
    }

} 
