<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

use HCM\Util\DateUtil;

use HCM\Util\HCMParameters;
use HCM\Util\Util;

class SolicitacaoRepository {

    protected  $connection;

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getSolicitacao($nrorg, $nrvinculom, $cdoperador) {
        $sql = <<<SQL
/*SOLICITACOES INCLUIDAS PELO PROPRIO OPERADOR OU DO VINCULO DO OPERADOR*/
SELECT
    GPE_SOLICITACAO.*,
    GPE_VINCULOM.NMVINCULOM,
    GPE_PERFACESTIPOSOLI.NRNIVELAPROVASOLI,
    GPE_PERFACESTIPOSOLI.NRPERFACESAPROVPRI,
    GPE_PERFACESTIPOSOLI.NRPERFACESAPROVSEC
  FROM GPE_SOLICITACAO
  JOIN GPE_VINCULOM ON GPE_VINCULOM.NRORG = GPE_SOLICITACAO.NRORG
                   AND GPE_VINCULOM.NRVINCULOM = GPE_SOLICITACAO.NRVINCULOM
  LEFT JOIN GPE_PERFACESTIPOSOLI ON GPE_PERFACESTIPOSOLI.NRORG = GPE_SOLICITACAO.NRORG
                                AND GPE_PERFACESTIPOSOLI.NRTIPOSOLICITACAO = GPE_SOLICITACAO.NRTIPOSOLICITACAO
 WHERE GPE_SOLICITACAO.CDOPERINCLUSAO = :P_CDOPERADOR OR (GPE_VINCULOM.NRORG = :P_NRORG AND GPE_SOLICITACAO.NRVINCULOM = :P_NRVINCULOM)
UNION
/*SOLICITACOES DA ESTRUTURA GERENCIAL NO QUAL O OPERADOR E RESPONSAVEL E PERFIL DE ACESSO DE NIVEL 1*/
SELECT
    GPE_SOLICITACAO.*,
    GPE_VINCULOM.NMVINCULOM,
    GPE_PERFACESTIPOSOLI.NRNIVELAPROVASOLI,
    GPE_PERFACESTIPOSOLI.NRPERFACESAPROVPRI,
    GPE_PERFACESTIPOSOLI.NRPERFACESAPROVSEC
  FROM GPE_SOLICITACAO
  JOIN GPE_VINCULOM ON GPE_VINCULOM.NRORG = GPE_SOLICITACAO.NRORG
                   AND GPE_VINCULOM.NRVINCULOM = GPE_SOLICITACAO.NRVINCULOM
  JOIN GPE_VINCULOM ON GPE_VINCULOM.NRORG = GPE_SOLICITACAO.NRORG
                   AND GPE_VINCULOM.NRVINCULOM = GPE_SOLICITACAO.NRVINCULOM
  JOIN GPE_VINCULOH ON GPE_VINCULOH.NRORG = GPE_VINCULOM.NRORG
                   AND GPE_VINCULOH.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                   AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                       WHERE H1.NRORG = GPE_VINCULOH.NRORG
                                                         AND H1.NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                                                         AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN ESTRUTURAH ON ESTRUTURAH.NRORG = GPE_VINCULOH.NRORG
                 AND ESTRUTURAH.NRESTRUTURAM = GPE_VINCULOH.NRESTRUTGEREN
                 AND ESTRUTURAH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                   WHERE H1.NRORG = ESTRUTURAH.NRORG
                                                     AND H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN OPERADOR ON OPERADOR.NRORG = ESTRUTURAH.NRORG
               AND OPERADOR.NRPARCNEGOCIO = ESTRUTURAH.NRPARCRESPINF
               AND OPERADOR.CDOPERADOR = :P_CDOPERADOR
  LEFT JOIN GPE_PERFACESTIPOSOLI ON GPE_PERFACESTIPOSOLI.NRORG = GPE_SOLICITACAO.NRORG
                                AND GPE_PERFACESTIPOSOLI.NRTIPOSOLICITACAO = GPE_SOLICITACAO.NRTIPOSOLICITACAO
  LEFT JOIN PERFACESSPARC ON PERFACESSPARC.NRORG = OPERADOR.NRORG
                         AND PERFACESSPARC.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO
                         AND PERFACESSPARC.NRPERFILACESSO = GPE_PERFACESTIPOSOLI.NRPERFACESAPROVPRI
 WHERE PERFACESSPARC.NRPARCNEGOCIO IS NOT NULL
UNION
/*SOLICITACOES PARA O PERFIL DE ACESSO DE NIVEL 2 CASO O OPERADOR TENHA ESSE PERFIL*/
SELECT
    GPE_SOLICITACAO.*,
    GPE_VINCULOM.NMVINCULOM,
    GPE_PERFACESTIPOSOLI.NRNIVELAPROVASOLI,
    GPE_PERFACESTIPOSOLI.NRPERFACESAPROVPRI,
    GPE_PERFACESTIPOSOLI.NRPERFACESAPROVSEC
  FROM GPE_SOLICITACAO
  JOIN GPE_VINCULOM ON GPE_VINCULOM.NRORG = GPE_SOLICITACAO.NRORG
                   AND GPE_VINCULOM.NRVINCULOM = GPE_SOLICITACAO.NRVINCULOM
  JOIN OPERADOR ON OPERADOR.NRORG = GPE_VINCULOM.NRORG
               AND OPERADOR.CDOPERADOR = :P_CDOPERADOR
  LEFT JOIN GPE_PERFACESTIPOSOLI ON GPE_PERFACESTIPOSOLI.NRORG = GPE_SOLICITACAO.NRORG
                                AND GPE_PERFACESTIPOSOLI.NRTIPOSOLICITACAO = GPE_SOLICITACAO.NRTIPOSOLICITACAO
  LEFT JOIN PERFACESSPARC ON PERFACESSPARC.NRORG = OPERADOR.NRORG
                         AND PERFACESSPARC.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO
                         AND PERFACESSPARC.NRPERFILACESSO = GPE_PERFACESTIPOSOLI.NRPERFACESAPROVSEC
 WHERE PERFACESSPARC.NRPARCNEGOCIO IS NOT NULL;
SQL;
    }

    public function retornaVinculoFromOperador($nrorg, $cdoperador, $competencia)
    {
        $sql = <<<SQL
SELECT VM.*
  FROM GPE_VINCULOM VM
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN OPERADOR OP ON OP.NRORG = PE.NRORG
                  AND OP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                  AND OP.CDOPERADOR = :P_CDOPERADOR
 WHERE VM.NRORG = :P_NRORG 
 AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENCIA)
SQL;

        $params = array(
            'P_NRORG' => $nrorg,
            'P_CDOPERADOR' => $cdoperador,
            'P_DTMESCOMPETENCIA' =>$competencia
        );

        return $this->connection->fetchAll($sql, $params);
    }

    public function insereSolicitacao($nrsolicitacao, $nrtiposolicitacao, $nrvinculom, $dsobservacao) {

        $sql = "INSERT INTO GPE_SOLICITACAO
                  (NRORG, NRSOLICITACAO, NRTIPOSOLICITACAO, NRVINCULOM, DTSOLICITACAO, DSOBSSOLICITACAO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTULTATU, CDOPERULTATU, NRORGULTATU)
                VALUES
                  (:NRORG, :NRSOLICITACAO, :NRTIPOSOLICITACAO, :NRVINCULOM, :DTSOLICITACAO, :DSOBSSOLICITACAO, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO, :DTULTATU, :CDOPERULTATU, :NRORGULTATU)";

        return $this->connection->prepare($sql)
            ->execute(array(
                "NRORG"             => $this->hcmParameters->getNrorgTrabalho(),
                "NRSOLICITACAO"     => $nrsolicitacao,
                "NRTIPOSOLICITACAO" => $nrtiposolicitacao,
                "NRVINCULOM"        => $nrvinculom,
                "DTSOLICITACAO"     => DateUtil::getDataAtual(true)->format('d/m/Y'),
                "DSOBSSOLICITACAO"  => $dsobservacao,
                "DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGINCLUSAO"     => $this->hcmParameters->getNrorgTrabalho(),
                "CDOPERINCLUSAO"    => $this->hcmParameters->getCdoperador(),
                "DTULTATU"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGULTATU"     => $this->hcmParameters->getNrorgTrabalho(),
                "CDOPERULTATU"    => $this->hcmParameters->getCdoperador()
            )
        );
    }
    
    public function atualizaAlteOcupa($nrorg,$nrvinculom, $dtfimocupacao,$dtatual,$cdoperador) {
    
        $sql = "UPDATE GPE_ALTEOCUPACAO 
        SET DTFIMOCUPACAO = :P_DTFIMOCUPACAO,
        DTULTATU = :P_DTATUAL,
        NRORGULTATU = :P_NRORG,
        CDOPERULTATU = :P_CDOPERADOR
        WHERE DTINIOCUPACAO = (
            SELECT MAX(DTINIOCUPACAO) 
            FROM GPE_ALTEOCUPACAO 
            WHERE NRORG = :P_NRORG 
            AND NRVINCULOM = :P_NRVINCULOM 
            AND DTFIMOCUPACAO IS NULL
            )
        AND NRORG = :P_NRORG 
        AND NRVINCULOM = :P_NRVINCULOM 
        AND DTINIOCUPACAO >= :P_DTFIMOCUPACAO
        AND DTFIMOCUPACAO IS NULL";

        return $this->connection->prepare($sql)
            ->execute(array(
                "P_NRORG"             => $nrorg,
                "P_DTFIMOCUPACAO"     => $dtfimocupacao,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_DTATUAL"           => $dtatual,
                "P_CDOPERADOR"        => $cdoperador
            )
        );
    }
    
    public function atualizaAlteEscala($nrorg,$nrvinculom, $dtfimescala,$dtatual,$cdoperador) {
                      

        $sql = "UPDATE GPE_ALTEESCALA 
        SET DTFIMESCALA = :P_DTFIMESCALA,
        DTULTATU = :P_DTATUAL,
        NRORGULTATU = :P_NRORG,
        CDOPERULTATU = :P_CDOPERADOR
        WHERE DTINIESCALA = (
            SELECT MAX(DTINIESCALA) 
            FROM GPE_ALTEESCALA 
            WHERE NRORG = :P_NRORG 
            AND NRVINCULOM = :P_NRVINCULOM 
            AND DTFIMESCALA IS NULL
            )
        AND NRORG = :P_NRORG 
        AND NRVINCULOM = :P_NRVINCULOM 
        AND DTINIESCALA >= :P_DTFIMESCALA
        AND DTFIMESCALA IS NULL";

        return $this->connection->prepare($sql)
            ->execute(array(
                "P_NRORG"             => $nrorg,
                "P_DTFIMESCALA"     => $dtfimescala,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_DTATUAL"           => $dtatual,
                "P_CDOPERADOR"        => $cdoperador
            )
        );
    }
    

    public function insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, $dschaveitemsoli, $dslabelitemsoli, $dsvaloritemsoli) {

        $query = 'INSERT INTO GPE_ITEMSOLICITACAO
                      (NRORG, NRITEMSOLICITACAO, NRSOLICITACAO, DSCHAVEITEMSOLI, DSLABELITEMSOLI, DSVALORITEMSOLI, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                    VALUES
                      (:NRORG, :NRITEMSOLICITACAO, :NRSOLICITACAO, :DSCHAVEITEMSOLI, :DSLABELITEMSOLI, :DSVALORITEMSOLI, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO)';

        return $this->connection->prepare($query)
            ->execute(array(
                "NRORG"             => $this->hcmParameters->getNrorgTrabalho(),
                "NRITEMSOLICITACAO" => $nritemsolicitacao,
                "NRSOLICITACAO"     => $nrsolicitacao,
                "DSCHAVEITEMSOLI"   => $dschaveitemsoli,
                "DSLABELITEMSOLI"   => $dslabelitemsoli,
                "DSVALORITEMSOLI"   => $dsvaloritemsoli,
                "DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGINCLUSAO"     => $this->hcmParameters->getNrorgTrabalho(),
                "CDOPERINCLUSAO"    => $this->hcmParameters->getCdoperador(),
            )
        );
    }

    public function retornaItensSolicitacao($nrorg, $nrsolicitacao)
    {
        $sql = <<<SQL
SELECT *
  FROM GPE_ITEMSOLICITACAO
 WHERE NRORG = :P_NRORG
   AND NRSOLICITACAO = :P_NRSOLICITACAO
SQL;

        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRSOLICITACAO' => $nrsolicitacao
        );

        return $this->connection->fetchAll($sql, $params);
    }

    public function verificaAcessoOperVinculo($nrorg, $nrvinculom, $cdoperador, $nrtiposolicitacao)
    {
        $sql = <<<SQL
SELECT SUM(ACESSO) ACESSO FROM (
/*VINCULOS PERTECENTES A ESTRUTURA GERENCIAL NA QUAL O OPERADOR É RESPONSÁVEL INFORMAL*/
SELECT
    COUNT(*) ACESSO
  FROM GPE_VINCULOM
  JOIN GPE_VINCULOH ON GPE_VINCULOH.NRORG = GPE_VINCULOM.NRORG
                   AND GPE_VINCULOH.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                   AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                       WHERE H1.NRORG = GPE_VINCULOH.NRORG
                                                         AND H1.NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                                                         AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN ESTRUTURAH ON ESTRUTURAH.NRORG = GPE_VINCULOH.NRORG
                 AND ESTRUTURAH.NRESTRUTURAM = GPE_VINCULOH.NRESTRUTGEREN
                 AND ESTRUTURAH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                   WHERE H1.NRORG = ESTRUTURAH.NRORG
                                                     AND H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN OPERADOR ON OPERADOR.NRORG = ESTRUTURAH.NRORG
               AND OPERADOR.NRPARCNEGOCIO = ESTRUTURAH.NRPARCRESPINF
               AND OPERADOR.CDOPERADOR = :P_CDOPERADOR
 WHERE GPE_VINCULOM.NRORG = :P_NRORG
   AND GPE_VINCULOM.NRVINCULOM = :P_NRVINCULOM
UNION
/*OPERADOR POSSUI PERFIL DE ACESSO DE NIVEL 2 DO TIPO DE SOLICITAÇÃO INFORMADO*/
SELECT
    COUNT(*) ACESSO
  FROM OPERADOR
  JOIN PERFACESSPARC ON PERFACESSPARC.NRORG = OPERADOR.NRORG
                    AND PERFACESSPARC.NRPARCNEGOCIO = OPERADOR.NRPARCNEGOCIO
  JOIN GPE_PERFACESTIPOSOLI ON GPE_PERFACESTIPOSOLI.NRORG = OPERADOR.NRORG
                           AND GPE_PERFACESTIPOSOLI.NRTIPOSOLICITACAO = :P_NRTIPOSOLICITACAO
                           AND GPE_PERFACESTIPOSOLI.NRPERFACESAPROVSEC = PERFACESSPARC.NRPERFILACESSO
 WHERE OPERADOR.NRORG = :P_NRORG
   AND OPERADOR.CDOPERADOR = :P_CDOPERADOR
)
SQL;

        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRVINCULOM' => $nrvinculom,
            'P_CDOPERADOR' => $cdoperador,
            'P_DTMESCOMPETENC' => $this->hcmParameters->getDtCompetencia(),
            'P_NRTIPOSOLICITACAO' => $nrtiposolicitacao
        );

        $result = $this->connection->fetchAll($sql, $params);
        return $result[0]['ACESSO'] > 0 ? true : false;
    }
    
    public function atualizaCompetenciaVinculo($nrorg,$nrvinculom,$competencia) {
    
        $sql = " 
        UPDATE GPE_VINCULOH
        SET DTMESCOMPETENC = :P_DTCOMPETENCIA
        WHERE NRORG = :P_NRORG 
        AND NRVINCULOM = :P_NRVINCULOM
        AND DTMESCOMPETENC = :P_DTADMISSAO";
        return $this->connection->prepare($sql)
            ->execute(array(
                "P_NRORG"             => $nrorg,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_DTADMISSAO"           => "01/01/3000",
                "P_DTCOMPETENCIA"        => $competencia
            )
        );
    }
    
    public function deletaHistoricoAdmissao($nrorg,$nrvinculom) {
    
        $sql = " 
        DELETE FROM GPE_VINCULOH
        WHERE NRORG = :P_NRORG 
        AND NRVINCULOM = :P_NRVINCULOM
        AND DTMESCOMPETENC = :P_DTCOMPETENC";
        return $this->connection->prepare($sql)
            ->execute(array(
                "P_NRORG"             => $nrorg,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_DTCOMPETENC"        => "01/01/3000"
            )
        );
    }

    public function validaPerfilParamOrg($cdoperador, $nrorg) {
        try {
            $sql = <<<SQL
                SELECT 
                    1
                FROM 
                    OPERADOR OP
                    JOIN PERFACESSPARC PA
                        ON PA.NRORG = OP.NRORG
                            AND PA.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                    LEFT JOIN GPE_PARAMORG PO
                        ON PO.NRORG = OP.NRORG
                WHERE 
                    OP.NRORG = :P_NRORG
                    AND OP.CDOPERADOR = :P_CDOPERADOR
                    AND PO.NRPERFAPROVASOLI = PA.NRPERFILACESSO
SQL;
            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"       => $nrorg,
                    "P_CDOPERADOR"  => $cdoperador
                )
            );

            $resultQuery = $query->fetchAll();

            if (!empty($resultQuery)) {
                return true;
            } else {
                return false;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaPerfilAprovSolicitacao($cdoperador, $nrorg, $nrsolicitacao) {
        try {
            $sql = <<<SQL
                SELECT 
                    CASE
                        WHEN EXISTS (
                            SELECT 1
                            FROM PERFACESSPARC PERACESS_APROV
                            WHERE PERACESS_APROV.NRORG = SL.NRORG
                                AND INSTR(',' || FASE_SEGUINTE.DSNRPERFILACESSO || ',', ',' || PERACESS_APROV.NRPERFILACESSO ||  ',') > 0
                                AND PERACESS_APROV.NRPARCNEGOCIO = (
                                    SELECT NRPARCNEGOCIO
                                    FROM OPERADOR
                                    WHERE NRORG = PERACESS_APROV.NRORG AND CDOPERADOR = :P_CDOPERADOR
                                )
                        ) AND SL.IDAPROVACAOSEC NOT IN ('A', 'R') THEN 'S'
                        ELSE 'N'
                    END ID_OPER_APROVA_SOLI
                FROM 
                    GPE_SOLICITACAO SL
                    LEFT JOIN GPE_FASETRAMSOLI FT
                        ON FT.NRORG = SL.NRORG 
                            AND FT.IDATIVO = 'S'
                            AND FT.NRSOLICITACAO = SL.NRSOLICITACAO
                            AND FT.NRFASETRAMSOLI = (
                                    SELECT
                                        MAX(FT1.NRFASETRAMSOLI)
                                    FROM
                                        GPE_FASETRAMSOLI FT1
                                    WHERE
                                        FT1.NRORG = FT.NRORG AND
                                        FT1.NRSOLICITACAO = FT.NRSOLICITACAO AND
                                        FT1.IDATIVO = 'S'
                                )
                    LEFT JOIN GPE_FASEAPROVSOLI FASE_ATUAL
                        ON FASE_ATUAL.NRORG = FT.NRORG
                            AND FASE_ATUAL.NRFASEAPROVSOLI = FT.NRFASEAPROVSOLI
                    LEFT JOIN GPE_FASEAPROVSOLI FASE_SEGUINTE
                        ON FASE_SEGUINTE.NRORG = SL.NRORG
                            AND FASE_SEGUINTE.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                            AND FASE_SEGUINTE.NRORDEM = NVL(FASE_ATUAL.NRORDEM, 0) + 1
                    LEFT JOIN OPERADOR OP
                        ON OP.NRORG = :P_NRORG
                            AND OP.CDOPERADOR = :P_CDOPERADOR
                    LEFT JOIN GPE_PARAMORG PO
                        ON PO.NRORG = OP.NRORG
                WHERE 
                    SL.NRORG = :P_NRORG
                    AND SL.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;
            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"           => $nrorg,
                    "P_CDOPERADOR"      => $cdoperador,
                    "P_NRSOLICITACAO"   => $nrsolicitacao
                )
            );

            $resultQuery = $query->fetchAll();

            if (!empty($resultQuery)) {
                return $resultQuery[0]['ID_OPER_APROVA_SOLI'];
            } else {
                return false;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }
} 
