<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
//use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Geral\Repository\VinculoRepository;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;

use HCM\Util\PhpWord\TemplateProcessorHCM;
//use PhpOffice\PhpWord\Shared\ZipArchive;

class EmissaoDocService {

    protected $entityManager;
    protected $hcmParameters;
    protected $vinculoRepository;
    
    protected $mnemonicsDoc      = array(); //Mnemônicos usados pelo documento
    protected $mnemonicsSystem   = array(); //Mnemônicos existentes no sistema
    protected $mnemonicsSession  = array(); //Lista dos mnemônicos de sessão. ESTA LISTA É FIXA! VERIFICAR
    protected $mnemonicsScreen   = array(); //Lista dos mnemônicos de tela. ESTA LISTA É FIXA! VERIFICAR
    protected $mnemonicsUser     = array(); //Lista dos mnemônicos de valores do usuário. Nõa está atrelado a nenhum repositório ou variável de sessão.
    protected $repositoriesInUse = array(); //Apenas os repositórios em uso (orienta-se pelos mnemônciso existestes no documentos e os dados dos mnemônicos do sistema)
    protected $listDocsFiles     = array(); //Lista de arquivos gerados no processo
    
    private $urlMidiaServer;
    
    
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, VinculoRepository $vinculoRepository) {
        $this->entityManager     = $entityManager;
        $this->hcmParameters     = $hcmParameters;
        $this->vinculoRepository = $vinculoRepository;
    }
    
    
    /**
     * FUNÇÃO PRINCIPAL
     */
    public function emitirDocPadrao($nrorg, $urlMidiaServer, $dtmescompetencia, $dtmescompetenciafim, $dspath, $nmdocumento,
        $nrvinculom_list, $nrlotevinc_list, $nrestrutlegal_list, $nrestrutgeren_list, $nrestrutsind, $dtiniadmissao, $dtfimadmissao, $dtinirescisao,
        $dtfimrescisao, $parametrosAdicionais, $flagPDF) {
        
        error_reporting(E_ERROR); //todo por causa do phpword. gerar warnings que quebrar o retorno. rever isto
        $this->urlMidiaServer = $urlMidiaServer;
        
        //Monta a lista de mnemônicos de sessão e seus valores.
        $this->getMnemonicSessionData();
        
        //Monta a lista de mnemônicos de tela e seus valores.
        $this->setMnemonicScreenData($parametrosAdicionais);

        //Adquiri a lista dos mnemônicos existêntes no sistema e de usuário
        $this->getMnemonicsSystemAndUser();
        
        //Adquiri o documento
        $pathDocName = $this->getDocModelo($dspath, $nmdocumento, $nrorg);
        
        
        //todo colocar estes blocos em função. usar constante para a mensagem abaixo
        //Adquiri lista de todos mnemônicos usados no documento //todo passar para uma função?
        $templateProcessor = new TemplateProcessorHCM($pathDocName); //XML do documento
        $this->mnemonicsDoc = $templateProcessor->getMnemonics();
        if(count($this->mnemonicsDoc) == 0) //Verifica se o documento possui pelo menos um mnemônico
            throw new \Exception("HCM10010_GER_EMISSAODOC_NO_MNEMONIC");
        
        //Mnemonicos quebrados
        $mnemonicsError = $templateProcessor->getMnemonicsError();
        if(count($mnemonicsError) > 0) {
            $msg = 'Existem mnemônicos quebrados no documento modelo:<br><br>● '.implode('<br>● ', $mnemonicsError);
            throw new \Exception($msg);
        }
        //var_dump($quebrados); die;
        
        unset($templateProcessor); //Destroi a instância
        
        
        
        //Monta uma lista dos repositories usados pelo documento e dos mnemônicos do usuário com seus respectivos valores
        $this->getRepositoriesInUse();
        
        //Monta uma lista de vínculos de acordo com o formulário da tela
        $vinculos = $this->vinculoRepository->retornaVinculosAtivosDoc($nrorg, $dtmescompetencia, $nrestrutlegal_list, $nrestrutgeren_list, $nrestrutsind, $nrvinculom_list, $nrlotevinc_list, $dtiniadmissao, $dtfimadmissao, $dtinirescisao, $dtfimrescisao);

        if(count($vinculos) == 0) //Verifica se os filtros do formulário trouzeram pelo um ou mais vínculos
            throw new \Exception("HCM10010_GER_EMISSAODOC_NO_RESULT");
        
        //LOOP - Percorre todos os vínculos retornados pela seleção do formulário
        $this->listDocsFiles = array();
        foreach($vinculos as $vinculo) {
            
            //Usa o array de mnemônicos do documento (mnmonicsDoc) para popular seus valores
            $this->getMnemonicData($vinculo, $dtmescompetencia);

            //Gera um arquivo novo Doc com os mnemônicos substituídos e armazena a lista
            $this->listDocsFiles[] = $this->gerarNovoDoc($pathDocName, $nrorg, $nmdocumento, $vinculo['NMVINCULOM'], $vinculo['NRVINCULOM'], $flagPDF);
        }

        //Compacta os arquivos caso seja gerados dois ou mais docs
        if(count($this->listDocsFiles) > 1)
            return $this->makeZipFromDocs($nmdocumento);
        else
            return $this->listDocsFiles; //todo usar [0]
    }
    
    
    /**
     * Adquiri os dados de sessão dos mnemônicos de sessão
     */
    private function getMnemonicSessionData() {
        $this->mnemonicsSession['[CDOPERADOR]']           = $this->hcmParameters->getCdoperador();
        $this->mnemonicsSession['[NMOPERADOR]']           = $this->hcmParameters->getNmoperador();
        $this->mnemonicsSession['[DATAEMISSAO]']          = $this->hcmParameters->getDataAtual(true);
        $this->mnemonicsSession['[DTMESCOMPETENC]']       = $this->hcmParameters->getDtCompetencia();
        $this->mnemonicsSession['[ULTIMODIACOMPETENCIA]'] = $this->hcmParameters->getUltimoDiaMes();
        $this->mnemonicsSession['[OCORRENCIA]']           = (string)$this->hcmParameters->getOcorrenciaCalculo();
        $this->mnemonicsSession['[MODALIDADECALCULO]']    = (string)$this->hcmParameters->getNrModalidadeCalculo();
        $this->mnemonicsSession['[MODALIDADESALARIAL]']   = (string)$this->hcmParameters->getNrModalidadeSalario();
        $this->mnemonicsSession['[AMBIENTE]']             = $this->hcmParameters->getIdambientesal();
    }
    
    /**
     * Adquiri os dados de campos de tela (parâmetros adicionais) dos mnemônicos de tela
     * @param array $params Contém todos os campos de parâmetros adicionais da tela de emissão
     */
    private function setMnemonicScreenData($params) {
        $this->mnemonicsScreen['[TELAEMISSAO]']         = $params['ADDTEMISSAO'];
        $this->mnemonicsScreen['[TELAULTDIATRAB]']      = $params['ADDTULTIMO'];
        $this->mnemonicsScreen['[TELATERLICENCA]']      = $params['ADDTTERLICENCA'];
        $this->mnemonicsScreen['[TELARETORNORH]']       = $params['ADDTRETORNO'];
        $this->mnemonicsScreen['[TELAINILICENCAMATER]'] = $params['ADDTINILICENCA'];
        $this->mnemonicsScreen['[TELADTPAGTO]']         = $params['ADDTPAGTO'];
        $this->mnemonicsScreen['[TELADTEXAME]']         = $params['ADDTEXAME'];
        $this->mnemonicsScreen['[TELAHREXAME]']         = $params['ADHREXAME'];
        $this->mnemonicsScreen['[TELAHRCOMPARECI]']     = $params['ADHRCOMPARECI'];
    }


    /**
     * Consulta e monta na memória o repositório dos mnemônicos do sistema e de usuários.
     * Mnemônicos de usuários possuem o campo DSVALORMNEMONICO preenchido e o campo DSREPOSITORIO vazio.
     * O campo DSREPOSITORIO possui prioridade sobre o campo DSVALORMNEMONICO.
     */
    private function getMnemonicsSystemAndUser() {
        
        //Limpa/configura o mapeamento dos mnemônicos
        $this->mnemonicsSystem = array();
        $this->mnemonicsUser   = array();
        
        //Mnemônicos da org padrão e org do cliente
        $objMnemonico = $this->entityManager->getRepository(Repositories::GPE_MNEMONICO)->findBy(
            array('nrorg' => array($this->hcmParameters->getNrorgPadrao(),
                                   $this->hcmParameters->getNrorgTrabalho() ) ) );
        
        foreach($objMnemonico as $obj) {
            
            $cdmnemonico      = $obj->getCdmnemonico();
            $dsrepositorio    = $obj->getDsrepositorio();
            $dsatributo       = $obj->getDsatributo();
            $dsparamaux       = $obj->getDsparamaux();
            $dsvalormnemonico = $obj->getDsvalormnemonico();
            
            //Monta lista dos mnemônicos do systema
            if($dsrepositorio != null) {
                $this->mnemonicsSystem[$cdmnemonico] =
                    array(
                        'dsrepositorio' => $dsrepositorio,
                        'dsatributo'    => $dsatributo,
                        'dsparamaux'    => $dsparamaux
                    );
            }
            
            //Monta lista dos mnemônicos do usuário
            if($dsvalormnemonico != null && $dsrepositorio == null) {
                $this->mnemonicsUser[$cdmnemonico] = $dsvalormnemonico;
            }
            
            //todo montar os repositories in use aqui????
        }
        
        unset($objMnemonico, $obj, $cdmnemonico, $dsrepositori, $dsatributo, $dsparamau, $dsvalormnemonico);
    }
    
    
    /**
     * Baixa o arquivo do file server para o temporário e retorna seu nome completo com o caminho.
     * Retorna mensagem de erro se necessário. Se não carregar este retorna falso.
     */
    public function getDocModelo($path, $fileName, $nrorg) {
        
        if($path == null)
            throw new \Exception("HCM10010_GER_EMISSAODOC_NO_DSPTAH");
        
        //Local do destino do arquivo baixado.
        $filePath = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/';

        $this->verifyDisponibilityMidiaServer();
        
        //Conteúdo do arquivo do file server
        $fileContents = file_get_contents($this->urlMidiaServer.$path);
        if(!$fileContents) //Verifica se existe arquivo carrgeado
            throw new \Exception("HCM10010_GER_EMISSAODOC_GETCONTENTS_FAIL");

        //Salva no servidor o arquivo
        if(!file_exists($filePath)) //Verifica se a pasta da organização existe...
            mkdir($filePath, 0777); //...e a cria se necessário
        $filePath .= $fileName; //Adiciona o nome ao caminho
        if(!file_put_contents($filePath, $fileContents))  //Salva e verifica o sucesso da operação
            throw new \Exception("HCM10010_GER_EMISSAODOC_PUTCONTENTS_FAIL");
        
        return $filePath;
    }

    private function verifyDisponibilityMidiaServer() {
        $url = $this->urlMidiaServer;

        $ch = curl_init($url);

        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_exec($ch);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        curl_close($ch);

        if ($httpCode != 200 && $httpCode != 403) {
            throw new \Exception("HCM10010_GER_EMISSAODOC_MIDIASERVER_DOWN");
        }
    }
    
    
    /**
     * Monta uma lista dos repositories/mnemônicos usados pelo documento.
     * O índice é o nome do repositorio. Armazena dados para a chamada deste repositorio.
     * Também faz a aquisição dos mnemônicos do usuário com seus respectivos valores (caso existam).
     */
    private function getRepositoriesInUse() {
        
        $this->repositoriesInUse = array();
        
        //Percorre todos o mnemônicos usados pelo documento
        foreach($this->mnemonicsDoc as $mnemonic => $v) {
            
            //Apenas mnemônicos que existam no sistema
            if(isset($this->mnemonicsSystem[$mnemonic])) {
                
                $dsrepositorio = $this->mnemonicsSystem[$mnemonic]['dsrepositorio'];
                $dsatributo    = $this->mnemonicsSystem[$mnemonic]['dsatributo'];
                $dsparamaux    = $this->mnemonicsSystem[$mnemonic]['dsparamaux']; //todo entidade
                
                //Salta os mnemônicos que não possuem repositórios
                if($dsrepositorio == null) {
                    continue;
                }
                
                //Cria uma entrada de indice <nome do repositório>
                if(!isset($this->repositoriesInUse[$dsrepositorio])) {
                    $this->repositoriesInUse[$dsrepositorio] = array();
                }
                
                //Cria um entrada de indice <nome do repositório> / <nome mnemônico>
                $this->repositoriesInUse[$dsrepositorio][$mnemonic] =
                    array(
                        'dsatributo' => $dsatributo,
                        'dsparamaux' => $dsparamaux
                    );
            } /*else { //todo lançar mensagens de mnemônicos não cadastrados? }*/
        }
    }
    
    /**
     * Função para formatar um dado de acordo com uma máscara fornecida.
     * @param string $dado O dado a ser formatado.
     * @param string $mascara A máscara a ser aplicada ao dado.
     * @return string O dado formatado de acordo com a máscara.
     * Exemplo de uso:
     * $cnpj = "12345678000195";
     * $mascara = "##.###.###/####-##";
     * echo formatarDado($cnpj, $mascara); // Saída: 12.345.678/0001-95
     */
    private function formatarDado($dado, $mascara) {
        //$dado = str_replace(" ","",$dado);
        $numCaracteres = substr_count($mascara, '#');               // Conta o número de caracteres '#' na máscara
        
        $dado = preg_replace('/[^0-9]/', '', $dado);                // remove todos os caracteres que não são números
        $dado = str_pad($dado, $numCaracteres, '0', STR_PAD_LEFT);  // Preenche o dado com zeros à esquerda de acordo com o número de caracteres '#' na máscara

        //$dado = str_pad($dado, 14, '0', STR_PAD_LEFT);  // preenche com zeros à esquerda se necessário
        
        for($i=0; $i < strlen($dado); $i++) {
            $mascara[strpos($mascara, "#")] = $dado[$i];
        }

        return $mascara;
    }

    /**
     * Função para formatar um dado de acordo com o nome da linha fornecida.
     * @param string $rowName O nome da linha.
     * @param string $value O valor a ser formatado.
     * @return string O valor formatado de acordo com o nome da linha.
     */
    function formatarDeAcordoComRowName($rowName, $value) {
        if ($rowName == "CDCNPJESTRUT") {
            return $this->formatarDado($value, "##.###.###/####-##");
        } elseif ($rowName == "NRCPFPESSOA") {
            return $this->formatarDado($value, "###.###.###-##");
        } else {
            return $value;
        }
    }

    /**
     * Popula o array de Mnemônicos do Documento com os valores correspondnetes
     * Usa o array de repositorios em uso para acionar os repositórios correspondnetes.
     * Os valores conseguidos dos repositórios são armazenados no array de mnemônicos dos documentos.
     */
    private function getMnemonicData($params, $dtmescompetencia) {
        
        //Parâmetros padrão para as entidades
        $paramsPadrao['nrorg']          = $this->hcmParameters->getNrorgTrabalho();
        $paramsPadrao['nrorgpadrao']    = $this->hcmParameters->getNrorgPadrao();
        $paramsPadrao['dtmescompetenc'] = $dtmescompetencia;
        
        //Concatenando parâmetros padrão e parâmetros do vínculo
        $params = array_merge($paramsPadrao, $params);

        //Adquiri os dados dos repositorios usados no documento de acordo com o vínculo em questão
        foreach($this->repositoriesInUse as $repository => $repositoryContent) {
            //todo parâmetro auxiliar //$params['DSPARAMAUX']
            
            //Dados do repositório
            $data = $this->entityManager->getRepository(Repositories::GPE_MNEMONICO)->$repository($params);

            //Percorre o dado do indice <nome do repositório>
            foreach($repositoryContent as $mnemonic => $mnemonicContent) {
                
                //Aquisiação do nome do campo corrspondente ao mnemônico e o dado (valor)
                $rowName = $mnemonicContent['dsatributo'];
                $value   = $data[$rowName];
                
                //todo formatação do dado (valor) do campo
                if(substr($rowName, 0, 2) == 'DT') { //TEMPORÁRIO!!!!
                    $value = substr($value, 0, 10);
                }

                $value = $this->formatarDeAcordoComRowName($rowName, $value);
                //var_dump($value);
                
                //Usa o array dos mnemônicos do documento para preencher os dados
                $this->verificaCaractereEspecialNoMnemonico($value);
                $this->mnemonicsDoc[$mnemonic] = $value;
            }
        }
        
        //Adiciona os mnemônicos de sessão, de tela e de usuário aos mnemônicos do documento
        foreach($this->mnemonicsDoc as $mnemonic => $value) {
            //Sessão
            if(array_key_exists($mnemonic, $this->mnemonicsSession))
                $this->mnemonicsDoc[$mnemonic] = $this->mnemonicsSession[$mnemonic];

            //Tela
            if(array_key_exists($mnemonic, $this->mnemonicsScreen))
                $this->mnemonicsDoc[$mnemonic] = $this->mnemonicsScreen[$mnemonic];
            
            //Usuário
            if(array_key_exists($mnemonic, $this->mnemonicsUser))
                $this->mnemonicsDoc[$mnemonic] = $this->mnemonicsUser[$mnemonic];
        }

    }
    
    
    private function verificaCaractereEspecialNoMnemonico(&$value){
        $caractereEComercial = "/&/";
        $replacementEComercial = '&amp;';
        
        //Foi necessario separar o replace do '&' dos outros, pois o replace do '&' estava alterando os outros.
        $OutrosCaracteresEspeciais = ["/\"/", "/'/", "/</", "/>/"];
        $OutrosReplacements = ['&quot;', '&apos;', '&lt;', '&gt;'];
        
        $value = preg_replace($caractereEComercial, $replacementEComercial, $value) . PHP_EOL;
        $value = preg_replace($OutrosCaracteresEspeciais, $OutrosReplacements, $value);
    }
    
    
    /**
     * Gera um novo documento .doc ou .docx com as substituições dos menumônicos.
     * Retorna este documento em string.
     */
    private function gerarNovoDoc($pathDocName, $nrorg, $nmdocumento, $nmvinculom, $nrvinculom, $flagPDF) {        
        //XML do documento - Instância volátil, apenas dentro desta função
        $templateProcessor = new TemplateProcessorHCM($pathDocName);
        
        //Percorre todos o mnemônicos em uso pelo docuemnto
        foreach($this->mnemonicsDoc as $mnemonic => $valueData) {
            $templateProcessor->setValue($mnemonic, $valueData);
        }
        
        //Define local e nome do novo documento
        $targetFileName = explode('.', $nmdocumento); //Separa o nome da extensão
        if(count($targetFileName) != 2) //Verifica se o nome possui apenas um ponto '.'
            throw new \Exception("HCM10010_GER_EMISSAODOC_NOT_ONE_DOT");
        $targetFileName = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/'.$targetFileName[0].' - '.$nrvinculom.' - '.$nmvinculom.'.'.$targetFileName[1];
        
        //Salva o templete com o novo nome
        $templateProcessor->saveAs($targetFileName);

        if ($flagPDF == 'S') {
            $fileConverter = new FileConverter();
        
            $fileConverter->docxtToPdf($targetFileName);

            unlink($targetFileName);

            $targetFileName = str_replace('.docx', '.pdf', $targetFileName);
        }

        unset($templateProcessor); //Destroi a instância
        
        //todo um isFile e uma throw??
        
        return $targetFileName;
    }
    
    
    /**
     * Compacta os arquivos contidos na lista 'listDocsFiles' e retorna o path do novo arquivo zip.
     * Exclui os docs gerados depois de zipado. Se houver um zip com o mesmo nome este será apagado.
     */
    private function makeZipFromDocs($nmdocumento) {
        
        //Converte o nome do documento para um nome .zip
        $nmdocumento = explode('.', $nmdocumento);
        $nmdocumento[1] = 'zip';
        $nmdocumento = implode('.', $nmdocumento);
        
        //Local do arquivo compactado
        $filePath = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/'.$nmdocumento;
        
        //Criando um arquivo zip com mesmo nome do documento modelo
        $zipObj = new \ZipArchive();
        $zipObj->open($filePath, \ZipArchive::CREATE | \ZipArchive::OVERWRITE);
        
        //Adicionando os arquivos de documento no zip
        foreach($this->listDocsFiles as $file) {
            $name = end(explode('/', $file)); //Nome do documento gerado
            $zipObj->addFile($file, $name);
            //unlink($file); //todo ver como fazer isso... se for possível....
        }
        
        $zipObj->close();
        
        unset($nmdocumento, $zipObj, $file, $name);
        
        return $filePath; //Retorno do arquivo ZIP
    }
    
    
    
    /**
     * Ajusta o nome do novo docx gerado pelo TemplateProcessor (PhpWord)
     * TODO retirar essa função
     */
    /*public function renameTemplate($template, $fileName, $nrorg) {
        
        //Local do destino do arquivo temporário template
        $filePath = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/'.$fileName;
        unlink($filePath); //Apaga temporário anterior
        rename($template, $filePath);
        
        return $filePath; //Novo nome e caminho do template (docx)
    }*/

}

class FileConverter
{
  public function docxtToPdf($path)
  {
    $tmp = substr($path, 0, strrpos($path, "/"));

    $command = "export HOME=" .$tmp. " && libreoffice --headless --convert-to pdf --outdir " .$tmp. " \"" .$path. "\" 2>&1";
    
    $result = shell_exec($command);

    $libreOfficeNotFound = strpos($result, 'libreoffice: not found') !== false;

    if ($libreOfficeNotFound) {
        throw new \Exception("HCM10010_GER_EMISSAODOC_NOT_FOUND_LIBREOFFICE");
    }
  }
}