<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\VinculoRepository;
use HCM\Geral\Repository\ESocialRepository;
use HCM\Geral\Logic\QuadroVagas;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Folha\Entities\VabLinhavinc;
use HCM\Util\Messages;
use Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use Teknisa\Model\Gestaopessoas\Entities\GpeCondfispes;
use Teknisa\Model\Gestaopessoas\Entities\FpaTpdemissao;
use HCM\Util\Util;
use HCM\FPA\Services\ExportacaoExcelService;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\FPA;

class VinculoServices {

    protected $gpe_Vinculoh;
    /**
     * @var vinculoRepository
     */
    protected $vinculoRepository;
    protected $eSocialRepository;
    protected $exportacaoExcelService;
    protected $httpKernel;

    const NRSITUFUNCM_NORMAL = 1;
    const NRSITUFUNCM_PRE_REINTEGRACAO = 100140;

    public function __construct(EntityManager $entityManager, VinculoRepository $vinculoRepository, ESocialRepository $eSocialRepository, HCMParameters $hcmParameters, ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel) {
        $this->entityManager = $entityManager;
        $this->vinculoRepository = $vinculoRepository;
        $this->eSocialRepository = $eSocialRepository;
        $this->hcmParameters = $hcmParameters;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel = $httpKernel;
        
        $this->gpe_Vinculoh = Repositories::GPE_VINCULOH;
        $this->gpe_Vinculom = Repositories::GPE_VINCULOM;
    }

    /**
     * @return int
     */
     
    const QRYcancelaAdmissao = "SELECT(SELECT NMTIPOMOVIMENT FROM FPA_TIPOMOVIMENT WHERE NRTIPOMOVIMENT = CF.NRTIPOMOVIMENT) || ' - ' || TO_CHAR(CF.DTOCORRENCIA, 'MM/YYYY') || ' - ' || (SELECT NMTPMODALIDCAL FROM FPA_TPMODALIDCAL WHERE NRTPMODALIDCAL = CF.NRTPMODALIDCAL ) || ' - ' || CF.NROCORRECAL SAIDA FROM FPA_CALCULOFOLHA CF JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = CF.NRORG AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA AND ICF.NRVINCULOM = :P_NRVINCULOM WHERE CF.NRORG = :P_NRORG AND CF.NRTIPOMOVIMENT IN ( 1, 2, 4, 10, 17, 19, 21) GROUP BY CF.DTOCORRENCIA, CF.NRTIPOMOVIMENT, CF.NRTPMODALIDCAL, CF.NROCORRECAL ORDER BY CF.DTOCORRENCIA, CF.NRTIPOMOVIMENT, CF.NRTPMODALIDCAL, CF.NROCORRECAL";
    
    public function getNrVinculoM($nrorg) {
        try{
            $result = $this->vinculoRepository->getNrVinculoM($nrorg);
            $id = $result[0]['MAX_ID'];
            return !is_null($id) ? ++$id : 1;
        }
        catch (Exception $e) {
            throw new exception ($e->getMessage());
        }
    }

     public function getNrVinculoH($nrorg) {
        try{
            $result = $this->vinculoRepository->getNrVinculoH($nrorg);
            $id = $result[0]['MAX_ID'];
            return !is_null($id) ? ++$id : 1;
        }
        catch (Exception $e) {
            throw new exception ($e->getMessage());
        }
    }
    
    public function novoHistorico($nrorg, $nrvinculom, $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm,
                                  $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao,
                                  $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc,
                                  $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin,
                                  $nrestrutsind, $confidencial, $cdmatriculaEsocial = null, $idpagpropfer13 = null, $idAssocSindicato = 'N', $idpartartclt = 'N')
    {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh $vinculoh */
        $vinculoh = new $this->gpe_Vinculoh;

        $vinculoh->setNrorg($nrorg);
        $vinculoh->setNrvinculom($nrvinculom);
        $vinculoh->setDtmescompetenc($dtmescompetenc);

        $vinculoh->setDtbaseferias($dtbaseferias);
        $vinculoh->setIdmultvinc($idmultvinc);
        $vinculoh->setNrsitufuncm($nrsitufuncm);
        $vinculoh->setNrtpmovtransfm($nrtpmovtransfm);
        $vinculoh->setCdcontribindividual($cdcontribindividual);
        $vinculoh->setDtfimcontrdetermin($dtfimcontrdetermin);

        $vinculoh->setNrcargo($nrcargo);
        $vinculoh->setNrfuncao($nrfuncao);
        $vinculoh->setNrescalatrabm($nrescalatrabm);
        $vinculoh->setIdremuneracao($idremuneracao);
        $vinculoh->setIdtppagamento($idtppagamento);
        $vinculoh->setNrtpmodalidsal($nrtpmodalidsal);

        $vinculoh->setNrdependir($nrdependir);
        $vinculoh->setNrdependsfam($nrdependsfam);
        $vinculoh->setConfidencial($confidencial);

        $vinculoh->setNrestrutlegal($nrestrutlegal);
        $vinculoh->setNrestrutgeren($nrestrutgeren);
        $vinculoh->setNrestrutsind($nrestrutsind);
        $vinculoh->setIdcontribuisind(($idcontribuisind) ? : 'NUNCA');
        $vinculoh->setCdmatriculaesocial($cdmatriculaEsocial);
        $vinculoh->setIdpagpropfer13($idpagpropfer13);
        $vinculoh->setIdassocsindicato($idAssocSindicato);
        $vinculoh->setIdpartclt($idpartartclt);
        
        $vinculoh->setDtinclusao(DateUtil::getDataAtual());
        $vinculoh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $vinculoh->setDtultatu(DateUtil::getDataAtual());
        $vinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $vinculoh->setNrvinculoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOH', 1, 12, true));
        
        $this->entityManager->persist($vinculoh);
        $this->entityManager->flush($vinculoh);
        
        return $vinculoh;
    }

    /**
     * Verifica se o vínculo possui histórico mais novo que a competencia atual.
     * Caso não tenha gera novo histórico na competência atual.
     *
     * @param $nrorg
     * @param $nrvinculom
     * @throws \Exception
     */
    public function verificaHistoricoCompetencia($nrorg, $nrvinculom, $competencia = null) {
        if ($competencia == null){
            $competencia = $this->hcmParameters->getDtCompetencia();
    
            $gpevinculoh = $this->vinculoRepository->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $competencia);
            $dthistorico = DateUtil::getDataDeString($gpevinculoh->getDtmescompetenc(), DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $dtcompetenc = DateUtil::getDataDeString($competencia);
            $dtcompetenc->setTime(0,0,0);
            if (DateUtil::comparaDatas($dthistorico, $dtcompetenc)) {
                $this->novoHistorico($nrorg, $nrvinculom, $dtcompetenc, $gpevinculoh->getNrsitufuncm(), $gpevinculoh->getNrcargo(), $gpevinculoh->getNrescalatrabm(),
                    $gpevinculoh->getIdcontribuisind(), $gpevinculoh->getNrdependir(), $gpevinculoh->getNrdependsfam(), $gpevinculoh->getDtbaseferias(), $gpevinculoh->getIdremuneracao(),
                    $gpevinculoh->getIdtppagamento(), $gpevinculoh->getNrestrutlegal(), $gpevinculoh->getNrestrutgeren(), $gpevinculoh->getNrtpmodalidsal(), $gpevinculoh->getIdmultvinc(),
                    $gpevinculoh->getNrfuncao(), $gpevinculoh->getCdcontribindividual(), $gpevinculoh->getNrtpmovtransfm(), $gpevinculoh->getDtfimcontrdetermin(),
                    $gpevinculoh->getNrestrutsind(), $gpevinculoh->getConfidencial(), $gpevinculoh->getCdmatriculaesocial(), $gpevinculoh->getIdpagpropfer13(), $gpevinculoh->getIdassocsindicato());
            }
        } else {
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom, "nrorg" => $nrorg));
            //$gpevinculoh = $this->vinculoRepository->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $competencia);
            $gpevinculoh = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $competencia->format('d/m/Y'));
            $dthistorico = DateUtil::getDataDeString($gpevinculoh->getDtmescompetenc(), DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $dtcompetenc = DateUtil::getDataDeString($competencia);
            $dtcompetenc->setTime(0,0,0);
            if (DateUtil::comparaDatas($dthistorico, $dtcompetenc)) {
                $this->novoHistorico($nrorg, $nrvinculom, $dtcompetenc, $gpevinculoh->getNrsitufuncm(), $gpevinculoh->getNrcargo(), $gpevinculoh->getNrescalatrabm(),
                    $gpevinculoh->getIdcontribuisind(), $gpevinculoh->getNrdependir(), $gpevinculoh->getNrdependsfam(), $gpevinculoh->getDtbaseferias(), $gpevinculoh->getIdremuneracao(),
                    $gpevinculoh->getIdtppagamento(), $gpevinculoh->getNrestrutlegal(), $gpevinculoh->getNrestrutgeren(), $gpevinculoh->getNrtpmodalidsal(), $gpevinculoh->getIdmultvinc(),
                    $gpevinculoh->getNrfuncao(), $gpevinculoh->getCdcontribindividual(), $gpevinculoh->getNrtpmovtransfm(), $gpevinculoh->getDtfimcontrdetermin(),
                    $gpevinculoh->getNrestrutsind(), $gpevinculoh->getConfidencial(), $gpevinculoh->getCdmatriculaesocial(), $gpevinculoh->getIdpagpropfer13(), $gpevinculoh->getIdassocsindicato());
            }
            
        }
    }

    /**
     * Retorna histórico do vínculo na competência informada ou na competência atual
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param null $dtmescompetenc
     * @return mixed
     */
    public function retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc = null) {
        if (!$dtmescompetenc) {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        }
        return $this->vinculoRepository->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc);
    }
    
    public function retornaHistoricoAnteriorUltimoHistorico($nrorg, $nrvinculom, $nrvinculoh, $dtmescompetenc = null) {
        if (!$dtmescompetenc) {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        }
        return $this->vinculoRepository->retornaHistoricoAnteriorUltimoHistorico($nrorg, $nrvinculom, $nrvinculoh, $dtmescompetenc);
    }
    
    /**
     * Retorna todos os históricos a partir da comptência informada
     */
    public function retornaHistoricosPosteriores($nrorg, $nrvinculom, $dtmescompetenc) {
        $where   = "NRORG = :NRORG AND NRVINCULOM = :NRVINCULOM AND DTMESCOMPETENC > :DTMESCOMPETENC";
    	$params  = array(":NRORG" => $nrorg, ":NRVINCULOM" => $nrvinculom, ":DTMESCOMPETENC" => $dtmescompetenc);
    	$orderby = "DTMESCOMPETENC ASC";
    	return \Folha\Entities\GpeVinculoh::getObjectsByWhere($where, $params, $orderby);
    }
    
    /**
     * Atualiza o histórico de vinculo via lista (array) de nrvinculoh.
     */
    public function atualizaHistoricosVinculo($nrorg, $listaNrvinculoh,
        $nrsitufuncm, $nrcargo, $nrfuncao, $nrescalatrabm, $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $cdmatriculaesocial)
    {
    	try {
    	    $this->entityManager->beginTransaction();
    	    
        	$where      = array("nrorg" => $nrorg, "nrvinculoh" => $listaNrvinculoh);
        	$historicos = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findBy($where);
    	    $msg        = array(); //Mensagens das competências alteradas
            
            foreach($historicos as $historico) {
                if($nrsitufuncm  ) $historico->setNrsitufuncm(  $nrsitufuncm  );
        		if($nrcargo      ) $historico->setNrcargo(      $nrcargo      );
        		if($nrfuncao     ) $historico->setNrfuncao(     $nrfuncao     );
        		if($nrescalatrabm) $historico->setNrescalatrabm($nrescalatrabm);
        		if($nrestrutlegal) $historico->setNrestrutlegal($nrestrutlegal);
        		if($nrestrutgeren) $historico->setNrestrutgeren($nrestrutgeren);
        		if($nrestrutsind ) $historico->setNrestrutsind( $nrestrutsind );
        		if($cdmatriculaesocial) $historico->setCdmatriculaesocial($cdmatriculaesocial);
        		
        		$historico->setNrorgultatu( $nrorg                               );
        		$historico->setCdoperultatu($this->hcmParameters->getCdoperador());
        		$historico->setDtultatu(    DateUtil::getDataAtual()             );
                
        		$msg[] = $historico->getDtmescompetenc()->format('d/m/Y');
            }
            $this->entityManager->flush();
            $this->entityManager->commit();
            
            return $msg;
    	} catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }

    // retorna somente vínculo ativos na competência
    public function retornaVinculosAtivosCompetencia($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $nrtipovinculom) {
        return $this->vinculoRepository->retornaVinculosAtivosCompetencia($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $nrtipovinculom);
    }

    // retorna vinculos ativos na competência parametrizado
    public function retornaVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos = false, $ds_inicio = null, $ds_fim = null, $dsOrdem = null, $carregaHistorico = false, $nrEstrutura = null, $verificaConfidencial = true, $nrTipoVinculom = null, $nrVinculoEmpreg = null, $nrEstrutSind = null, $paramCaged = false, $idapenasdesemp = null) {
        $ds_inicio = !is_array($ds_inicio) ? trim(strtoupper($ds_inicio)) : $ds_inicio;
        $ds_fim    = !is_array($ds_fim) ? trim(strtoupper($ds_fim)) : $ds_fim;
        
        // define primeiro dia do mês competência
        $primeiroDiaAno = DateUtil::subtraiIntervalo(substr($this->hcmParameters->getPrimeiroDiaDoAno(), 0, 10), 1, DateUtil::ANO)->format('d/m/Y');
        $mesCompetencia = substr($mesCompetencia, 0, 10);
        $dtUltDiaMes = substr(DateUtil::getUltimoDiaDoMes($mesCompetencia), 0, 10);
        return $this->vinculoRepository->retornaVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos, $primeiroDiaAno, $dtUltDiaMes, $ds_inicio, $ds_fim, $dsOrdem, $carregaHistorico, $nrEstrutura, $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, $nrEstrutSind, $paramCaged, $idapenasdesemp);   

    }

    public function retornaRescinditosCompetencia($nrOrg, $nrOrgPadrao, $mesCompetencia, $nrEstrutura = null, $nrTipoVinculom = null, $nrVinculoEmpreg = null, $carregaHistorico = false, $filters,$todosRescindidos,$ds_inicio = null, $ds_fim = null, $dsOrdem = null, $nrEstrutSind = null, $conditions = null, $movTransfSaida = null, $filtroEntrevistaDeDesligamento = null) {        
        $primeiroDiaAno = $this->hcmParameters->getPrimeiroDiaDoAno();
        $dtUltDiaMes = $this->hcmParameters->getUltimoDiaMes();
        
        return $this->vinculoRepository->retornaRescinditosCompetencia($nrOrg, $nrOrgPadrao, $mesCompetencia, $dtUltDiaMes, $primeiroDiaAno, $carregaHistorico, $nrEstrutura, $nrTipoVinculom, $nrVinculoEmpreg, $filters, $ds_inicio, $ds_fim, $dsOrdem, $nrEstrutSind, $todosRescindidos, $conditions, $movTransfSaida, $filtroEntrevistaDeDesligamento);
    }
    
    public function retornaRescindidosGeral($mesCompetencia, $ds_inicio = null, $ds_fim = null, $dtini = null, $dtfim = null, $dsOrdem = null, $nrEstrutura = null, $carregaHistorico = null, $nrTipoVinculom = null,$nrVinculoEmpreg = null, $nrEstrutSind = null, $conditions) {        
       
        return $this->vinculoRepository->retornaRescindidosGeral($mesCompetencia, $ds_inicio, $ds_fim, $dtini, $dtfim, $dsOrdem, $nrEstrutura, $carregaHistorico, $nrTipoVinculom,$nrVinculoEmpreg, $nrEstrutSind, $conditions);
    }

    public function verificaCalculoMovimentoVinculo($nrvinculom, $nrOrg, $mesCompetencia, $nrtpmodalidcal, $ocorrecal, $nrtipomoviment) {
        return $this->vinculoRepository->verificaCalculoMovimentoVinculo($nrvinculom, $nrOrg, $mesCompetencia, $nrtpmodalidcal, $ocorrecal, $nrtipomoviment);
    }
    
    public function cancelaAdmissao($row) {
        try {
            $this->entityManager->beginTransaction();
            
            //variavel que armazena as mensagens de erro das validações de item de cálculo e de envio dos eventos para o eSocial
            $mensagem = "";



            $calculos = $this->entityManager->getConnection()->prepare(self::QRYcancelaAdmissao);
            $calculos->execute(
                 array (
                        "P_NRORG"                 => $row['NRORG'],
                        "P_NRVINCULOM"            => $row['NRVINCULOM'] 
                )
            );
            $calculos = $calculos->fetchAll();
            $aux = array();
            foreach($calculos as $val) {
                $aux[] = $val['SAIDA'];
            }
            if (count($calculos) > 0) {
                $calculos = implode($aux, "<br>");
                $mensagem .= Messages::evaluateMessage(Messages::HCM00000_GER_VINCULO_POSSUI_ITENSDECALC_ERROR, $calculos);
            }

            $envios = array(); 
            
            if(isset($row['NRTPADMISSAO']) && $row['NRTPADMISSAO'] == 5) {
                $existeEnvioPeriodico = $this->eSocialRepository->existeEnvioPeriodico($row['NRORG'], $row['NRPESSOA'], substr($row['DTBASEFERIAS'],0,10));
            } else {
                $existeEnvioPeriodico = $this->eSocialRepository->existeEnvioPeriodico($row['NRORG'], $row['NRPESSOA'], substr($row['DTADMISSAOVINC'],0,10));
            }
            
            $existeEnvioNaoPeriodico = $this->eSocialRepository->existeEnvioNaoPeriodico($row['NRORG'], $row['NRVINCULOM']);
            
            //Realiza a verificação do envio da admissão para o eSocial -> [06/2023] Verificação da tabela antiga, foi desativada por não haver cancelamento de vínculos antigos
            //if(!$this->eSocialRepository->verififcaSeAdmissaoFoiEnviadaParaOESocial($row['NRORG'], $row['NRVINCULOM'])){

            if($existeEnvioPeriodico || $existeEnvioNaoPeriodico){
                if ((!empty($existeEnvioPeriodico)) && count($existeEnvioPeriodico) > 0) { 
                    $envios = array_merge($envios,$existeEnvioPeriodico);
                }
                if ((!empty($existeEnvioNaoPeriodico)) && count($existeEnvioNaoPeriodico) > 0) { 
                    $envios = array_merge($envios,$existeEnvioNaoPeriodico);
                }

                foreach($envios as $envio) {
                    $retornoEsocialList[] = $envio['NMEVENTO'];
                }

                $retornoEsocial = implode($retornoEsocialList, "<br>");
     
                if(!empty($mensagem)){
                    $mensagem .= "<br /><br />" . Messages::evaluateMessage(Messages::HCM00048_GER_EVENTO_ADMISSAO_ENVIADO_TEXTO_COMPLEMENTAR_AVISO, $retornoEsocial);
                } else {
                    $mensagem .= Messages::evaluateMessage(Messages::HCM00048_GER_EVENTO_ADMISSAO_ENVIADO, $retornoEsocial);
                }
            }
            if(!empty($mensagem)){
                throw new \Exception($mensagem, 10);
            }
            
            $historicos = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findBy(array("nrvinculom" => $row['NRVINCULOM'], "nrorg" => $row['NRORG']));
            foreach ($historicos as $gpevinculoh) {
                $this->entityManager->remove($gpevinculoh);
            }
            $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $row['NRVINCULOM'],"nrorg" => $row['NRORG']));
            $gpevinculom->setNrorgultatu($row['NRORG']);
            $gpevinculom->setDtrescisaovinc($gpevinculom->getDtadmissaovinc());
            $gpevinculom->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gpevinculom->setDtultatu(DateUtil::getDataAtual());
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function novoMestre(
        $nrpessoa, $nrorgTrab, $dtAdmissao, $dtRescisao = null, $matricula = null, $tipoVinculo = null,
        $nrvinculoempreg = null, $nrtpadmissao = null, $nmvinculom = null, $dtPrimeiraAdmissao = null, $dtOpcaoFGTS = null,
        $dtExameMedico = null, $dtAposentadoriaFGTS = null, $nrDiasContratoExp = null, $nrDiasContratoExpPro = null, 
        $dslivroadmissao = null, $dtentregaepi = null, $dtvenccontrexper = null, $nrclausassecurat = null,
        $nrmotivoadmissao = null, $nrvinculomsubst = null
    ){
        try{
            $vinculo = new $this->gpe_Vinculom;
            $vinculo->setNrvinculom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOM', 1, 12, true));
            
            if(empty($matricula)){
                $matricula = $vinculo->getNrvinculom();
            }
        
            $vinculo->setNrpessoa($nrpessoa);
            $vinculo->setNrorg($nrorgTrab);
            $vinculo->setDtadmissaovinc($dtAdmissao);
            $vinculo->setDtrescisaovinc($dtRescisao);
            $vinculo->setCdmatricula($matricula);
            $vinculo->setNrtipovinculom($tipoVinculo);
            $vinculo->setNrvinculoempreg($nrvinculoempreg);
            $vinculo->setNrtpadmissao($nrtpadmissao);
            $vinculo->setNmvinculom($nmvinculom);
            $vinculo->setDtinclusao(DateUtil::getDataAtual());
            $vinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $vinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $vinculo->setDtultatu(DateUtil::getDataAtual());
            $vinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $vinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $vinculo->setDtprimadmissao($dtPrimeiraAdmissao);
            $vinculo->setDtopcaofgts($dtOpcaoFGTS);
            $vinculo->setDtaposentafgts($dtAposentadoriaFGTS);
            $vinculo->setNrdiascontratoexp($nrDiasContratoExp);
            $vinculo->setNrdiascontratoexppro($nrDiasContratoExpPro);
            $vinculo->setDtexamemedico($dtExameMedico);
            $vinculo->setDslivroadmissao($dslivroadmissao);
            $vinculo->setDtentregaepi($dtentregaepi);
            $vinculo->setDtvenccontrexper(DateUtil::getDataDeString($dtvenccontrexper));
            $vinculo->setNrclausassecurat($nrclausassecurat);
            $vinculo->setNrmotivoadmissao($nrmotivoadmissao);
            $vinculo->setNrvinculomsubst($nrvinculomsubst);
            
        
            $this->entityManager->persist($vinculo);
            $this->entityManager->flush($vinculo);
            
            return $vinculo;
        }
        catch (Exception $e){
            $e = new Exception('Erro ao criar um novo vinculo. Nao pode inserir um novo mestre.');
            throw $e;
        }
    }
    
    public function novo($nrPessoa, $dtAdmissaoVinculo, $cdMatricula, $nrTipoVinculom, $nrTipoAdmissao,
                         $nrVinculoEmpregaticio, $dtOpcaoFGTS, $dtAposentadoriaFGTS, $dtPrimeiraAdmissao,$nmVinculom,
                         $nrDiasContratoExp, $nrDiasContratoExpPro, $dtExameMedico, $dtmescompetenc, $nrsitufuncm,
                         $nrcargo, $nrescalatrabm, $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias,
                         $idremuneracao, $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc,
                         $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin, $nrestrutsind,
                         $confidencial = null,$nrorg = null,$dtRescisao = null, $dslivroadmissao = null, $dtentregaepi = null, 
                         $cdmatriculaEsocial = null, $dtvenccontrexper = null, $nrclausassecurat = null, $idpagpropfer13 = null,
                         $nrmotivoadmissao = null, $nrvinculomsubst = null, $idAssocSindicato = null, $idpartartclt = null){
        
        $vinculom = $this->novoMestre($nrPessoa, $nrorg, $dtAdmissaoVinculo, $dtRescisao, $cdMatricula, $nrTipoVinculom,
        $nrVinculoEmpregaticio, $nrTipoAdmissao, $nmVinculom, $dtPrimeiraAdmissao, $dtOpcaoFGTS,
        $dtExameMedico, $dtAposentadoriaFGTS, $nrDiasContratoExp, $nrDiasContratoExpPro, $dslivroadmissao, $dtentregaepi,
        $dtvenccontrexper, $nrclausassecurat, $nrmotivoadmissao, $nrvinculomsubst);
        
        $vinculoh = $this->novoHistorico(
            $nrorg,$vinculom->getNrvinculom(), $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm,
            $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao, $idtppagamento, $nrestrutlegal,
            $nrestrutgeren, $nrtpmodalidsal, $idmultvinc, $nrfuncao, $cdcontribindividual, $nrtpmovtransfm,
            $dtfimcontrdetermin, $nrestrutsind, $confidencial, $cdmatriculaEsocial, $idpagpropfer13, $idAssocSindicato, $idpartartclt
        );
        return array($vinculom, $vinculoh);
        
    }
    
    public function verificaSituaçãoVinculoRescisao($nrorg,$nrpessoa,$dtrescisao){
        
        
        $pessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array('nrorg' => $nrorg,
                    'nrpessoa' => $nrpessoa),(array('dtmescompetenc' => 'DESC')));
    
        if ($pessoah->getNrcondfispes() && ($pessoah->getNrcondfispes() != GpeCondfispes::CONDFISPES_NORMAL) && $dtrescisao) {
            
            $condfispes = $this->entityManager->getRepository(Repositories::GPE_CONDFISPES)->findOneBy(array('nrorg' => $nrorg,
            'nrcondfispes' => $pessoah->getNrcondfispes()));
            
            $dscondfispes = is_object($condfispes) ? $condfispes->getDscondfispes() : 'de deficiente';

            return $dscondfispes;
        }
        return false;
    }
    
    /*O empregado dispensado, sem justa causa, no período de 30 dias que antecede
      a data de sua correção salarial (dissídio), terá direito à indenização.*/
    public function verificaDissidio($nrorg, $nrvinculom, $dtcompetencia, $dtrescisao,$nrtpdemissao){
        
        $vinculoh = $this->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtcompetencia);
        
        if($vinculoh){
            $dtbasesindical = null;
            $nrestruturasind = $vinculoh->getNrestrutsind();
            $estruturasindical = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findOneBy(array('nrorg' => $nrorg,
                        'nrestruturam' => $nrestruturasind));
            if($estruturasindical) {
                $dtbasesindical = $estruturasindical->getDtbasesindical();
            }
            if($dtbasesindical && $dtrescisao) {
                $dtrescisao = Util::datadeString(substr($dtrescisao, 0, 10), true);
                $intervalo = $dtrescisao->diff($dtbasesindical);
                $qtdias =  (int)$intervalo->format('%R%a');
                $intervalo1 = $dtbasesindical->diff($dtrescisao);
                $qtdias1 =  (int)$intervalo1->format('%R%a');
                if ($qtdias <= 30
                    && $qtdias1 <= 30 && $nrtpdemissao == FpaTpdemissao::SEM_JUSTACAUSA) {
                    return true;
                }
            } 
            return false;
        } else {
            return false;
        }
    }
    
    public function verificaExistenciaLancamentoFuturos($row){
        try{
            $msg = null;
            $msg = $this->procuraLancamentosFuturosAdataDeRescisao($row); 
            
            if(!empty($msg)){
                return $msg;
            }
        }catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function procuraLancamentosFuturosAdataDeRescisao($row){
        try{
            $msg = null;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtRescisao = !empty($row['DTRESCISAOVINC']) ? DateUtil::truncateData(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($row['DTRESCISAOVINC'])))->format('d/m/Y') : null;
            $nrvinculom =  !empty($row['NRVINCULOM']) ? $row['NRVINCULOM'] : null;
            
            $lancamentos = $this->vinculoRepository->getLancamentosFuturosVinculo($nrorg, $nrvinculom, $dtRescisao);
            
            if(!empty($lancamentos)){
                $itens = '';
                foreach($lancamentos as $lancamento){
                    $nmtipomoviment = $lancamento['NMTIPOMOVIMENT'];
                    $dtOcorrencia = substr($lancamento['DTOCORRENCIA'], 3, 8);
                    $nmModalidadeCal = $lancamento['NMTPMODALIDCAL'];
                    $nrOcorrenciaCal = $lancamento['NROCORRECAL'];
                    $itens .= "$nmtipomoviment - $dtOcorrencia - $nmModalidadeCal - $nrOcorrenciaCal <br>";
                }
                return $msg = "O vínculo possui lançamentos futuros a data de rescisão, gostaria de prosseguir com a rescisão? <br><br> $itens";
                
            }
            return $msg;
        }catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function verificaExistenciaVinculoNovo($row){
        try{
            $msg = null;
            $msg = $this->procuraVinculosNovos($row); 
            
            if(!empty($msg)){
                return $msg;
            }
        }catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function procuraVinculosNovos($row){
        try{
            $msg = null;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtadmissaovinc = !empty($row['DTADMISSAOVINC']) ? DateUtil::getDataDeString($row['DTADMISSAOVINC'])->format('d/m/Y') : null;
            $nrpessoa =  !empty($row['NRPESSOA']) ? $row['NRPESSOA'] : null;
            $cdmatricula = $row['CDMATRICULA'] ?? null;
            
            $vinculos = $this->vinculoRepository->getVinculosReintegrados($nrorg, $nrpessoa, $dtadmissaovinc, $cdmatricula);
            
            if(!empty($vinculos)){
                $itens = '';
                foreach($vinculos as $vinculo){
                    $nrvinculom = $vinculo['NRVINCULOM'];
                    $dtAdmissao = DateUtil::getDataDeString($vinculo['DTADMISSAOVINC'])->format('d/m/Y');
                
                    $itens .= "Nr. Vínculo: $nrvinculom - Dt. Admissão: $dtAdmissao <br>";
                }
                return $msg = "Existe pelo menos um novo vínculo possivelmente reintegrado, gostaria de prosseguir com a criação? <br><br> $itens";
            }
            return $msg;
        }catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function verificaMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, $dtmescompetenc, $nrestruturam, $cdmatriculaesocialante){
        if($cdmatriculaesocialante != -1){    
            $retorno = array();
            if(isset($nrvinculom)){
                $envio2200 = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosPorNomeEvento($nrorg, 'S-2200', $nrestruturam, 1, 'S', $nrvinculom, null, null, 'N');
                $envio2300 = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosPorNomeEvento($nrorg, 'S-2300', $nrestruturam, 1, 'S', $nrvinculom, null, null, 'N');
            }else{
                $envio2200 = false;
                $envio2300 = false;
            }
            $matriculaRepetida = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true), $nrestruturam);
            if(!empty($envio2200)  || !empty($envio2300) || !empty($matriculaRepetida)){
                $retorno['message'] = 'Matrícula do ESocial não pode ser alterada! Verificar a existência de envios (S-2200, S-2300) ou outro vínculo com o mesmo código.';
                $retorno['type'] = 'ALERT';
                return $retorno;
            }else{
                $parametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(array('nrestruturam' => $nrestruturam, 'nrorg' => $nrorg));
                if($parametro && $parametro->getIdutilizamatric() == 'S' && !empty($cdmatriculaesocial)){    
                    $qtdeDigitos = $parametro->getNrqtdedigmatric();
                    $prefixo = $parametro->getDsprefixomatric();
                    $prefixMatriculaNova = $prefixo ? str_split($cdmatriculaesocial, strlen($prefixo)): null;
                    $prefixMatriculaNova = is_array($prefixMatriculaNova) ? $prefixMatriculaNova[0] : $prefixMatriculaNova;
                    
                    if($prefixo == $prefixMatriculaNova && (strlen($cdmatriculaesocial) == (strlen($prefixo)+$qtdeDigitos))){
                        $retorno['type'] = 'SUCESS';
                        return $retorno;
                    }else{
            
                        $retorno['message'] = 'Matrícula do ESocial incompatível com o padrão da estrutura. (prefixo:'.$prefixo.' | qtde. dígitos: '.$qtdeDigitos.')';
                        $retorno['type'] = 'ERROR';
                        return $retorno;
                    }
                } else {
                    $retorno['type'] = 'SUCESS';
                    return $retorno;
                }    
            }
        }else{
            $retorno['type'] = 'SUCESS';
            return $retorno;
        }
    }
    
    public function setSeguroVinculo($nrvinculom, $idsegdesempvinc){
         try {
            $this->entityManager->beginTransaction();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtcompetencia = $this->hcmParameters->getDtCompetencia();
            $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $nrorg));
	        $gpevinculom->setIdsegdesempvinc($idsegdesempvinc);
	        
            $this->entityManager->flush();
            $this->entityManager->commit();
            
            return $gpevinculom;
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    public function getFichaFinanceira($parameters, $conditions){	 
        return $this->vinculoRepository->getFichaFinanceira($parameters, $conditions);	 
    }
    
    public function getVinculosLiquidoParcial($nrorg, $dtmescompetenc, $nrtpmodalidcal, $nrtipomoviment, $nrestrutlegal, $nrestrutgeren, $incluir13, $incluirDemitidos) {
        return $this->vinculoRepository->getVinculosLiquidoParcial($nrorg, $dtmescompetenc, $nrtpmodalidcal, $nrtipomoviment, $nrestrutlegal, $nrestrutgeren, $incluir13, $incluirDemitidos);
    }
    
    public function getVinculosMatriculaEsocial($parameters, $conditions){
        return $this->vinculoRepository->getVinculosMatriculaEsocial($parameters, $conditions);
    }
    
    /**
     * Função responsável por retornar array de vínculos com  historico agregado.
     */
    public function retornaOnlyObjectVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos = false, $ds_inicio = null, $ds_fim = null, $dsOrdem = null, $carregaHistorico = false, $nrEstrutura = null,
                                               $verificaConfidencial = true, $nrTipoVinculom = null, $nrVinculoEmpreg = null, $nrEstrutSind = null) {

        if(!empty($ds_inicio) && is_array($ds_inicio)){
            $ds_inicio = implode(',', $ds_inicio);
        }
        if(!empty($ds_fim) && is_array($ds_fim)){
            $ds_fim = implode(',', $ds_fim);
        }
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim    = trim(strtoupper($ds_fim));

        // define primeiro dia do mês competência
        $primeiroDiaAno = DateUtil::subtraiIntervalo($this->hcmParameters->getPrimeiroDiaDoAno(), 1, DateUtil::ANO)->format('d/m/Y');
        $dtUltDiaMes = $this->hcmParameters->getUltimoDiaMes();
        return $this->vinculoRepository->retornaOnlyObjectVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos, $primeiroDiaAno, $dtUltDiaMes, $ds_inicio, $ds_fim, $dsOrdem, $carregaHistorico, $nrEstrutura,
                                               $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, $nrEstrutSind);   

    }
    
    public function getVinculoNr($parameters) {
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $parameters["P_DTMESCOMPETENC"];
        $nrvinculom     = $parameters["P_NRVINCULOM"];
        
        $query = $this->entityManager->getConnection()->prepare("
            SELECT VM.NMVINCULOM,
              PM.NRPARCNEGOCIO,
              PH.NRPESSOA,
              PH.NMPESSOA,
              EN.NRCEPENDERECO,
              EN.CDTIPOENDERECO,
              PA.NMPAIS,
              EN.SGESTADO,
              MU.NMMUNICIPIO,
              EN.CDBAIRRO,
              EN.CDLOGRADOURO,
              EN.NMBAIRROENDERECO,
              EN.DSCOMPLEENDERECO,
              EN.DSENDERECO,
              EN.NRIMOVELENDERECO,
              EN.DSREFERENCIAENDE,
              GR.NMGRAUINSTR,
              BC.CDBANCO,
              BC.NMBANCO,
              AG.NMAGENCIA,
              AG.CDAGENCIA,
              TPC.NRTPCONTCORR,
              TPC.DSTPCONTCORR,
              CT.CDCONTCORR,
              PH.NRCERTCASAPES,
              PH.DTCASAPESSOA,
              PH.DTHABCARTPES,
              PH.DSCATEGHABCART,
              PH.DTVALHABCARTPES,
              PH.NRCARTHABPES,
              VM.DTADMISSAOVINC,
              PH.NRCPFPESSOA,
              GPE_ALTEESCALA.DTINIESCALA,
              GPE_ESCALATRABH.NMESCALATRABH,
              GPE_TURNO.DSTURNO,
              FC.NMFORMACOMU,
              CP.CDFORMACOMU,
              CP.CDPREFIXCOMUPARC,
              CP.CDCOMUNICAPARC,
              GAS.DTALTESALARIO,
              GAS.NRTPMODALIDSAL,
              GAS.VRSALARIO,
              FPA_TPMODALIDSAL.NMTPMODALIDSAL,
              GAS.IDTPSALARIO,
              GPE_TIPOSALARIO.DSTIPOSALARIO,
              GPE_ALTEOCUPACAO.NROCUPACAOM,
              OH.NMOCUPACAOH,
              GPE_ALTEOCUPACAO.DTINIOCUPACAO,
              GPE_ALTEOCUPACAO.NRMOTIVOOCUPA,
              GPE_MOTIVOOCUPA.NMMOTIVOOCUPA,
              GPE_ALTEOCUPACAO.NRTIPOOCUPACAO,
              GPE_TIPOOCUPACAO.NMTIPOOCUPACAO,
              VH.*
            FROM GPE_VINCULOM VM

            JOIN GPE_VINCULOH VH
                ON VH.NRORG           = VM.NRORG
                AND VH.NRVINCULOM     = VM.NRVINCULOM
                AND VH.DTMESCOMPETENC =
                  (SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_VINCULOH
                  WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                  AND NRORG             = VH.NRORG
                  AND NRVINCULOM        = VH.NRVINCULOM
                  )

            LEFT JOIN GPE_PESSOAH PH
                ON PH.NRPESSOA        = VM.NRPESSOA
                AND PH.NRORG          = VM.NRORG
                AND PH.DTMESCOMPETENC =
                  (SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_PESSOAH
                  WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                  AND NRORG             = PH.NRORG
                  AND NRPESSOA          = PH.NRPESSOA
                  )

            LEFT JOIN GPE_PESSOA PM
                ON PH.NRPESSOA = PM.NRPESSOA
                AND PH.NRORG   = PM.NRORG

            LEFT JOIN ENDERECOPARC EN
                ON PM.NRPARCNEGOCIO = EN.NRPARCNEGOCIO

            LEFT JOIN PAIS PA
                ON EN.CDPAIS = PA.CDPAIS
    
            LEFT JOIN MUNICIPIO MU
                ON EN.CDMUNICIPIO = MU.CDMUNICIPIO
        
            LEFT JOIN GPE_GRAUINSTR GR
                ON GR.NRGRAUINSTR = PH.NRGRAUINSTR

            LEFT JOIN CONTCORRPARC CT
                ON CT.NRPARCNEGOCIO = PM.NRPARCNEGOCIO
    
            LEFT JOIN BANCO BC
                ON BC.CDBANCO = CT.CDBANCO
        
            LEFT JOIN AGENCIA AG
                ON AG.CDAGENCIA = CT.CDAGENCIA

            LEFT JOIN TPCONTCORR TPC
                ON TPC.NRTPCONTCORR      = CT.NRTPCONTCORR
                
                
            /* ESCALA TRABALHO*/
            LEFT JOIN 
                GPE_ALTEESCALA 
                    ON GPE_ALTEESCALA.NRESCALATRABM = VH.NRESCALATRABM
                    AND GPE_ALTEESCALA.NRORG = VH.NRORG
                    AND GPE_ALTEESCALA.NRVINCULOM = VH.NRVINCULOM
                    AND GPE_ALTEESCALA.DTINIESCALA = (
                                                            SELECT
                                                                MAX(AE.DTINIESCALA) 
                                                        FROM
                                                            GPE_ALTEESCALA AE 
                                                        WHERE
                                                            AE.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM 
                                                            AND AE.NRVINCULOM = GPE_ALTEESCALA.NRVINCULOM
                                                            AND AE.NRORG = GPE_ALTEESCALA.NRORG 
                                                            AND AE.DTINIESCALA <=:P_DTMESCOMPETENC 
                                                        )    
            LEFT JOIN
                GPE_ESCALATRABM 
                    ON GPE_ESCALATRABM.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM 
                    AND (
                        GPE_ESCALATRABM.NRORG = GPE_ALTEESCALA.NRORG 
                        OR GPE_ESCALATRABM.NRORG =:P_NRORG_PADRAO 
                    ) 
            LEFT JOIN
                GPE_ESCALATRABH 
                    ON GPE_ESCALATRABH.NRESCALATRABM = GPE_ESCALATRABM.NRESCALATRABM 
                    AND (
                        GPE_ESCALATRABH.NRORG = GPE_ESCALATRABM.NRORG 
                        OR GPE_ESCALATRABH.NRORG =:P_NRORG_PADRAO 
                    ) 
                    AND GPE_ESCALATRABH.DTMESCOMPETENC = (
                                                                SELECT
                                                                        MAX(EH.DTMESCOMPETENC) 
                                                                FROM
                                                                    GPE_ESCALATRABH EH 
                                                                WHERE
                                                                    EH.NRESCALATRABM = GPE_ESCALATRABH.NRESCALATRABM 
                                                                    AND EH.NRORG = GPE_ESCALATRABH.NRORG 
                                                                    AND EH.DTMESCOMPETENC <=:P_DTMESCOMPETENC 
                                                          ) 
            LEFT JOIN
                GPE_TURNO 
                    ON GPE_TURNO.NRTURNO = GPE_ALTEESCALA.NRTURNO 
                    AND GPE_TURNO.NRORG = GPE_ALTEESCALA.NRORG 
                    AND GPE_TURNO.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM             
            
            /*FORMA COMUNICAÇÃO*/
            LEFT JOIN COMUNICAPARC CP 
                ON CP.NRORG = VH.NRORG
                AND CP.NRPARCNEGOCIO = PM.NRPARCNEGOCIO
                AND CP.CDFORMACOMU = '01'
        
            LEFT JOIN FORMACOMU FC 
                ON CP.CDFORMACOMU = FC.CDFORMACOMU 
                AND(
                    FC.NRORG = CP.NRORG 
                    OR FC.NRORG = :P_NRORG_PADRAO
                ) 
                
            /*ALTERAÇÃO SALARIO*/    
            LEFT JOIN GPE_ALTESALARIO GAS
                ON GAS.NRORG = VM.NRORG
                AND GAS.NRVINCULOM = VM.NRVINCULOM
                AND GAS.DTALTESALARIO = (
                                            SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                            WHERE GAS.NRORG = GPE_ALTESALARIO.NRORG
                                            AND GAS.NRVINCULOM = GPE_ALTESALARIO.NRVINCULOM
                                        )
            
            LEFT JOIN
                FPA_TPMODALIDSAL 
                    ON FPA_TPMODALIDSAL.NRTPMODALIDSAL = GAS.NRTPMODALIDSAL 
                    AND (
                        FPA_TPMODALIDSAL.NRORG = GAS.NRORG 
                        OR FPA_TPMODALIDSAL.NRORG = :P_NRORG_PADRAO
                    ) 
            LEFT JOIN
                GPE_TIPOSALARIO 
                    ON GPE_TIPOSALARIO.CDTIPOSALARIO = GAS.IDTPSALARIO 
                    AND (
                        GPE_TIPOSALARIO.NRORG = GAS.NRORG 
                        OR GPE_TIPOSALARIO.NRORG = :P_NRORG_PADRAO
                    )                     


            /*OCUPAÇÃO*/
            LEFT JOIN GPE_ALTEOCUPACAO 
                ON GPE_ALTEOCUPACAO.NRVINCULOM = VM.NRVINCULOM
                AND GPE_ALTEOCUPACAO.NRORG = VM.NRORG
            JOIN
                GPE_OCUPACAOH OH 
                    ON OH.NROCUPACAOM = GPE_ALTEOCUPACAO.NROCUPACAOM 
                    AND OH.NRORG = GPE_ALTEOCUPACAO.NRORG 
                    AND OH.DTMESCOMPETENC = (
                                                SELECT
                                                    MAX(OH1.DTMESCOMPETENC) 
                                                FROM
                                                    GPE_OCUPACAOH OH1 
                                                WHERE
                                                    OH1.NROCUPACAOM = OH.NROCUPACAOM 
                                                    AND OH1.NRORG = OH.NRORG 
                                                    AND OH1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                            )
            LEFT JOIN
                GPE_MOTIVOOCUPA 
                    ON GPE_MOTIVOOCUPA.NRMOTIVOOCUPA = GPE_ALTEOCUPACAO.NRMOTIVOOCUPA 
                    AND (
                        GPE_MOTIVOOCUPA.NRORG = GPE_ALTEOCUPACAO.NRORG 
                        OR GPE_MOTIVOOCUPA.NRORG = :P_NRORG_PADRAO
                    )

            LEFT JOIN
                GPE_TIPOOCUPACAO 
                    ON GPE_TIPOOCUPACAO.NRTIPOOCUPACAO = GPE_ALTEOCUPACAO.NRTIPOOCUPACAO 
                    AND (
                        GPE_TIPOOCUPACAO.NRORG = GPE_ALTEOCUPACAO.NRORG 
                        OR GPE_TIPOOCUPACAO.NRORG = :P_NRORG_PADRAO
                    )                     


            WHERE VM.NRORG           = :P_NRORG
                AND ( VM.DTRESCISAOVINC >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                OR VM.DTRESCISAOVINC    IS NULL )
                AND (VM.NRVINCULOM       = :P_NRVINCULOM)
        ");

        $query->execute(
            array(
                "P_NRORG"                 => $nrorg,
                "P_NRORG_PADRAO"          => $this->hcmParameters->getNrorgPadrao(),
                "P_DTMESCOMPETENC"        => $dtmescompetenc,
                "P_NRVINCULOM"            => $nrvinculom
            )
        );

        $vinculo = $query->fetchAll();
        return $vinculo;
    }
    
    public function alteraBeneficiários($nrvinculom, $dtRescisaoVinc, $dtRescisaoRow) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $modificaBenef = false;
        
        if($dtRescisaoVinc != $dtRescisaoRow) {
            $modificaBenef = true;
        } else {
            $modificaBenef = false;
        }
        
        if($modificaBenef) {
           $this->vinculoRepository->alteraBeneficiários($nrvinculom, $dtRescisaoRow, $nrorg);
        }
    }
    
    public function updateConvocacaoTrabalhoIntermitente ($parameters){
        return $this->vinculoRepository->updateConvocacaoTrabalhoIntermitente($parameters);
    }
    
    public function existeMesmoPeriodoConvocacaoTrabalhoIntermitente ($parameters){
        $result = $this->vinculoRepository->existeMesmoPeriodoConvocacaoTrabalhoIntermitente($parameters);
        if( count($result) > 0 ){
            if( array_key_exists('NRCONVOCACAO', $parameters) ){
                if( $result[0]['NRCONVOCACAO'] == $parameters['NRCONVOCACAO'] ){
                    return false;
                }
            }
            return true;
        }
    }
    
    public function updateComposicaoValeTransporte ($parameters, $flag = null){
        $dataInicio = implode( '', array_reverse(explode('/', substr($parameters['DTINICONVOCACAO'], 0, 10))) );
        $dataFim = implode( '', array_reverse(explode('/', substr($parameters['DTFIMCONVOCACAO'], 0, 10))) );
        $vinculoh = $this->retornaUltimoHistoricoCompetencia($parameters['NRORG'], $parameters['NRVINCULOM'], $parameters['DTINICONVOCACAO']);
        if( substr($vinculoh->getDtmescompetenc()->format('d/m/Y'), 3,7) == substr($parameters['DTINICONVOCACAO'], 3,7) ){
            $totalDiasComposicaoVT = $vinculoh->getNrdiasapurvales();
        } else {
            $totalDiasComposicaoVT = 0;
        } 
       
        $periodoAntigo = $parameters['PERIODOANTIGO'];
        $periodoNovo = $dataFim - $dataInicio + 1;
        
        if( $flag == "update" ){
            $parameters['NRDIASAPURVALES'] = $totalDiasComposicaoVT - $periodoAntigo + $periodoNovo;
        } else if( $flag == "delete" ){
            if($totalDiasComposicaoVT > 0){
               $parameters['NRDIASAPURVALES'] = $totalDiasComposicaoVT - $periodoNovo;
               
               if($parameters['NRDIASAPURVALES'] == 0){
                   $parameters['NRDIASAPURVALES'] = null;
               }
            } else {
                $parameters['NRDIASAPURVALES'] = null;
            }
        }

        if( substr($vinculoh->getDtmescompetenc()->format('d/m/Y'), 3,7) == substr($parameters['DTINICONVOCACAO'], 3,7) ){
            $vinculoh->setNrdiasapurvales($parameters['NRDIASAPURVALES']);
            $vinculoh->setNrorgultatu($parameters['NRORG']);
        	$vinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
        	$vinculoh->setDtultatu(DateUtil::getDataAtual());
        	$this->entityManager->persist($vinculoh);
            $this->entityManager->flush($vinculoh);
        } else {
            $dtmescompetenc = str_split(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($parameters['DTINICONVOCACAO']))->__toString(), 10)[0];
            $dtmescompetenc = DateUtil::getDataDeString(substr($dtmescompetenc, 0, 10));
            $dtmescompetenc = DateUtil::truncateData($dtmescompetenc);
            $alteOcupacao = $this->entityManager->getRepository(Repositories::GPE_ALTEOCUPACAO)->findOneBy(array('nrvinculom' => $parameters['NRVINCULOM'], 'nrorg' => $parameters['NRORG']));
            
            $this->novoHistorico($parameters['NRORG'], $parameters['NRVINCULOM'], $dtmescompetenc, $vinculoh->getNrsitufuncm(), $alteOcupacao->getNrocupacaom(), $vinculoh->getNrescalatrabm(),
                                  $vinculoh->getIdcontribuisind(), $vinculoh->getNrdependir(), $vinculoh->getNrdependsfam(), $vinculoh->getDtbaseferias(), $vinculoh->getIdremuneracao(),
                                  $vinculoh->getIdtppagamento(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrtpmodalidsal(), $vinculoh->getIdmultvinc(),
                                  $vinculoh->getNrfuncao(), $vinculoh->getCdcontribindividual(), $vinculoh->getNrtpmovtransfm(), $vinculoh->getDtfimcontrdetermin(),
                                  $vinculoh->getNrestrutsind(), $vinculoh->getConfidencial());

            $vinculoh = $this->retornaUltimoHistoricoCompetencia($parameters['NRORG'], $parameters['NRVINCULOM'], $parameters['DTINICONVOCACAO']);
            $vinculoh->setNrdiasapurvales($parameters['NRDIASAPURVALES']);
            $this->entityManager->persist($vinculoh);
            $this->entityManager->flush($vinculoh);
        }
        return 0;
    }
    
    public function setCadastroAutomaticoConvocacaoTrabalhoIntermitente ($parameters){
        $this->vinculoRepository->setCadastroAutomaticoConvocacaoTrabalhoIntermitente($parameters);
        $this->updateComposicaoValeTransporte($parameters, "update");
        return 0;
    }
    
    public function getSituacaoVinculoEsocial($parameters, $conditions){	 
        return $this->vinculoRepository->getSituacaoVinculoEsocial($parameters, $conditions);	 
    }
    
    public function updateDtBaseFeriasNovoVinculo($nrvinculom, $nrorg, $dtefeitofinanceiro, $cdmatricula, $dtadmissao) {
        $vinculoAtual = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom, "nrorg" => $nrorg));
        $nrpessoa = $vinculoAtual->getNrpessoa();
        
        $novoVinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrpessoa" => $nrpessoa, "cdmatricula" => $cdmatricula, "nrorg" => $nrorg, "dtadmissaovinc" => DateUtil::getDataDeString(substr($dtadmissao, 0, 10), DateUtil::FORMATO_BRASILEIRO, true), "dtrescisaovinc" => null));
        $ultimoHistorico = $this->retornaUltimoHistoricoCompetencia($nrorg, $novoVinculo->getNrvinculom(), DateUtil::getDataAtual());
        if(!empty($ultimoHistorico)) {
            $ultimoHistorico->setDtbaseferias(DateUtil::getDataDeString(substr($dtefeitofinanceiro, 0, 10), DateUtil::FORMATO_BRASILEIRO, true));
            $ultimoHistorico->setDtinclusao(DateUtil::getDataAtual());
            $ultimoHistorico->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $ultimoHistorico->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $ultimoHistorico->setDtultatu(DateUtil::getDataAtual());
            $ultimoHistorico->setCdoperultatu($this->hcmParameters->getCdoperador());
            $ultimoHistorico->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($ultimoHistorico);
            $this->entityManager->flush();
        }
        
    }
    
    public function existeVinculoReintegrado($nrvinculom, $nrorg, $dtadmissao, $cdmatricula) {
        $vinculoAtual = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom, "nrorg" => $nrorg));
        $nrpessoa = $vinculoAtual->getNrpessoa();
        
        $vinculoReintegrado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrpessoa" => $nrpessoa, "cdmatricula" => $cdmatricula, "nrorg" => $nrorg, "dtadmissaovinc" => DateUtil::getDataDeString(substr($dtadmissao, 0, 10), DateUtil::FORMATO_BRASILEIRO, true), "dtrescisaovinc" => null));
        
        return !empty($vinculoReintegrado);
    }
    
    public function setReplicarVinculo($nrvinculom, $nrorg, $dtrescisao, $dtadmissao, $dtefeitofinanceiro, $dtretorno) { 
        try{
            $dtefeitofinanceiro = $dtefeitofinanceiro ? DateUtil::getDataDeString($dtefeitofinanceiro, $formato = DateUtil::FORMATO_BRASILEIRO, true) : null;
            $dtretorno = $dtretorno ? DateUtil::getDataDeString($dtretorno, $formato = DateUtil::FORMATO_BRASILEIRO, true) : null;

            $this->entityManager->getConnection()->beginTransaction();
            
            $nrVinculomNovoVinculo = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_VINCULOM', 1, 12, true);
            
            $this->replicarVinculoHistorico($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtefeitofinanceiro);
            $this->replicarSituacaoFuncional($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao, $dtefeitofinanceiro, $dtretorno, $dtadmissao);
            $this->replicarSalario($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarDependentes($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarEscala($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarMovimentacao($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarOcupacao($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarEvento($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarValeTransporte($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarEstabilidade($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarBeneficio($nrvinculom, $nrorg, $nrVinculomNovoVinculo, $dtrescisao);
            $this->replicarVinculoMestre($nrvinculom, $nrorg, $dtadmissao, $nrVinculomNovoVinculo, $dtefeitofinanceiro, $dtretorno);
            
            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
            
            return $nrVinculomNovoVinculo;
        } catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
    }
    
    public function replicarVinculoMestre($nrvinculom, $nrorg, $dtadmissao, $nrvinculomNovo, $dtefeitofinanceiro, $dtretorno) {
        $Gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $nrorg));
        $novoGpevinculom = clone $Gpevinculom;
        $novoGpevinculom->setNrvinculom($nrvinculomNovo);
		$novoGpevinculom->setDtadmissaovinc(DateUtil::getDataDeString(substr($dtadmissao, 0, 10), DateUtil::FORMATO_BRASILEIRO, true));
		$novoGpevinculom->setNrtpadmissao(5);
        $novoGpevinculom->setNrtpavipre(null);
		$novoGpevinculom->setNrtpdemissao(null);
		$novoGpevinculom->setNrmotivoresc(null);
		$novoGpevinculom->setDtavisoprevio(null);
		$novoGpevinculom->setDtrescisaovinc(null);
		$novoGpevinculom->setIdtpreintegracao(null);
		$novoGpevinculom->setNrprocjud(null);
		$novoGpevinculom->setNrleianistia(null);
		$novoGpevinculom->setIdpagtojuizo(null);
		$novoGpevinculom->setDtretorno($dtretorno);
		$novoGpevinculom->setDtefeitofinanceiro($dtefeitofinanceiro);
		$novoGpevinculom->setObsreintegracao(null);
		$novoGpevinculom->setDtinclusao(DateUtil::getDataAtual());
        $novoGpevinculom->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $novoGpevinculom->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
		$novoGpevinculom->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
        $novoGpevinculom->setCdoperultatu($this->hcmParameters->getCdoperador());
        $novoGpevinculom->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
		$this->entityManager->persist($novoGpevinculom);
		
    }
    
    public function replicarVinculoHistorico($nrvinculom, $nrorg, $nrvinculomNovo, $dtefeitofinanceiro) {
        $historicosVinculos = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($historicosVinculos as $historicoVinculo) {
	        $novoCdmatriculaesocial = $historicoVinculo->getCdmatriculaesocial() ?? $nrvinculom;
	        
	        $novoHistoricoVinculo = clone $historicoVinculo;
	        $novoHistoricoVinculo->setNrvinculom($nrvinculomNovo);
	        $novoHistoricoVinculo->setCdmatriculaesocial($novoCdmatriculaesocial);
		    $novoHistoricoVinculo->setDtbaseferias(DateUtil::getDataDeString(substr($dtefeitofinanceiro, 0, 10), DateUtil::FORMATO_BRASILEIRO, true));
	        $novoHistoricoVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novoHistoricoVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novoHistoricoVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novoHistoricoVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novoHistoricoVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novoHistoricoVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novoHistoricoVinculo);
            
	    }
    }
    
    public function replicarFeriasVinculo($nrvinculom, $nrorg, $nrvinculomNovo) {
        $todasFeriasVinculo = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($todasFeriasVinculo as $feriasVinculo) {
	        $nrFeriasAntigo = $feriasVinculo->getNrferias();
	        
	        $novaFeriasVinculo = clone $feriasVinculo;
	        $novaFeriasVinculo->setNrvinculom($nrvinculomNovo);
	        $novaFeriasVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novaFeriasVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novaFeriasVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novaFeriasVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novaFeriasVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novaFeriasVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novaFeriasVinculo);
            
            
            $gozoFeriasVinculo = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->findBy(array('nrorg' => $nrorg, 'nrferias' => $nrFeriasAntigo));
             if(!empty($gozoFeriasVinculo)) {
                $novoGozoFeriasVinculo = clone $gozoFeriasVinculo;
                $novoGozoFeriasVinculo->setNrferias($novaFeriasVinculo->getNrferias());
                $novoGozoFeriasVinculo->setDtinclusao(DateUtil::getDataAtual());
                $novoGozoFeriasVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $novoGozoFeriasVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $novoGozoFeriasVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
                $novoGozoFeriasVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
                $novoGozoFeriasVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $this->entityManager->persist($novoGozoFeriasVinculo);
                
             }
	    }
    }
    
    public function replicarSituacaoFuncional($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao, $dtefeitofinanceiro, $dtretorno, $dtadmissao) {

        $this->addSituacaoFuncional($nrorg, $nrvinculomNovo, $dtefeitofinanceiro, $dtretorno, $dtadmissao, self::NRSITUFUNCM_PRE_REINTEGRACAO); 
        $this->addSituacaoFuncional($nrorg, $nrvinculomNovo, $dtefeitofinanceiro, $dtretorno, null, self::NRSITUFUNCM_NORMAL);
		
        /*$situacoesFuncionaisVinculo = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaItem($nrvinculom, $nrorg);
        krsort($situacoesFuncionaisVinculo, SORT_NUMERIC);
        
        $dtfimsitufunc = null;
	    foreach($situacoesFuncionaisVinculo as $situFuncVinculo) {
	        $novaSituFuncVinculo = clone $situFuncVinculo;
	        
	        if($novaSituFuncVinculo->getDtfimsitufunc() == null && $novaSituFuncVinculo->getNrsitufuncm() == 13) {

                if($dtefeitofinanceiro) {
                    $dtfimsitufunc = clone $dtefeitofinanceiro;
                    $dtfimsitufunc->modify('-1 day');
                }
                else if($dtretorno) {
                    $dtfimsitufunc = clone $dtretorno;
                    $dtfimsitufunc->modify('-1 day');
                }
                $novaSituFuncVinculo->setDtfimsitufunc($dtfimsitufunc);
	        }
	        
	        $novaSituFuncVinculo->setNrvinculom($nrvinculomNovo);
	        $novaSituFuncVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novaSituFuncVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novaSituFuncVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novaSituFuncVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novaSituFuncVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novaSituFuncVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novaSituFuncVinculo);
	    }*/
    }

    private function addSituacaoFuncional($nrorg, $nrvinculomNovo, $dtefeitofinanceiro, $dtretorno, $dtadmissao, $nrsitufuncm) {
        //Cria a nova situação funcional de reintegração.

        if($dtadmissao) {
            $dtinisitufunc = DateUtil::getDataDeString(substr($dtadmissao, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            
            $dtfimsitufunc = clone ($dtefeitofinanceiro ? $dtefeitofinanceiro : $dtretorno);
            $dtfimsitufunc->modify('-1 day');
        } else {
            $dtinisitufunc = $dtefeitofinanceiro ? $dtefeitofinanceiro : $dtretorno;
            $dtfimsitufunc = null;
        }

        $gpealtesitufunc = Repositories::GPE_ALTESITUFUNC;
        $novaSituFuncVinculo = new $gpealtesitufunc;
        
        $novaSituFuncVinculo->setNrorg($nrorg);
        $novaSituFuncVinculo->setNrvinculom($nrvinculomNovo);
        $novaSituFuncVinculo->setNrsitufuncm($nrsitufuncm);
        $novaSituFuncVinculo->setDtinisitufunc($dtinisitufunc);
        $novaSituFuncVinculo->setDtfimsitufunc($dtfimsitufunc);

        $novaSituFuncVinculo->setDtinclusao(DateUtil::getDataAtual());
        $novaSituFuncVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $novaSituFuncVinculo->setNrorginclusao($nrorg);
        $novaSituFuncVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
        $novaSituFuncVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
        $novaSituFuncVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $novaSituFuncVinculo->setNraltesitufunc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true));

        $this->entityManager->persist($novaSituFuncVinculo);
    }
    
    public function replicarSalario($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $salariosVinculo = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($salariosVinculo as $salarioVinculo) {
	        $novoSalarioVinculo = clone $salarioVinculo;
	        $novoSalarioVinculo->setNrvinculom($nrvinculomNovo);
	        $novoSalarioVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novoSalarioVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novoSalarioVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novoSalarioVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novoSalarioVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novoSalarioVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novoSalarioVinculo);
            
	    }
    }
    
    public function replicarDependentes($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $dependentesVinculo = $this->entityManager->getRepository(Repositories::FPA_DEPVINCULO)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($dependentesVinculo as $dependenteVinculo) {
	        $novoDependenteVinculo = clone $dependenteVinculo;
	        $novoDependenteVinculo->setNrvinculom($nrvinculomNovo);
	        $novoDependenteVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novoDependenteVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novoDependenteVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novoDependenteVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novoDependenteVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novoDependenteVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novoDependenteVinculo);
            
	    }
    }
    
    public function replicarEscala($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $escalasVinculo = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($escalasVinculo as $escalaVinculo) {
	        $novaEscalaVinculo = clone $escalaVinculo;
	        
	        if($novaEscalaVinculo->getDtfimescala()!= null && DateUtil::comparaDatas($novaEscalaVinculo->getDtfimescala(), $dtrescisao, 3)) {
	            $novaEscalaVinculo->setDtfimescala(null);
	        }
	        
	        $novaEscalaVinculo->setNrvinculom($nrvinculomNovo);
	        $novaEscalaVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novaEscalaVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novaEscalaVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novaEscalaVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novaEscalaVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novaEscalaVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novaEscalaVinculo);
            
	    }
    }
    
    public function replicarMovimentacao($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $movimentacoesVinculo = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->retornaMovimentacoesporVinculoExcetoDemitido($nrvinculom, $nrorg);
		    
	    foreach($movimentacoesVinculo as $movimentacaoVinculo) {
	        $novaMovimentacaoVinculo = clone $movimentacaoVinculo;
	        $novaMovimentacaoVinculo->setNrvinculom($nrvinculomNovo);
	        $novaMovimentacaoVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novaMovimentacaoVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novaMovimentacaoVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novaMovimentacaoVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novaMovimentacaoVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novaMovimentacaoVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novaMovimentacaoVinculo);
            
	    }
    }
    
    public function replicarOcupacao($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $ocupacoesVinculo = $this->entityManager->getRepository(Repositories::GPE_ALTEOCUPACAO)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($ocupacoesVinculo as $ocupacaoVinculo) {
	        $novaOcupacaoVinculo = clone $ocupacaoVinculo;
	        
	        if($novaOcupacaoVinculo->getDtfimocupacao() != null && DateUtil::comparaDatas($novaOcupacaoVinculo->getDtfimocupacao(), $dtrescisao, 3)) {
	            $novaOcupacaoVinculo->setDtfimocupacao(null);
	        }
	        
	        $novaOcupacaoVinculo->setNrvinculom($nrvinculomNovo);
	        $novaOcupacaoVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novaOcupacaoVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novaOcupacaoVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novaOcupacaoVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novaOcupacaoVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novaOcupacaoVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novaOcupacaoVinculo);
            
	    }
    }
    
    public function replicarEvento($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $eventosVinculo = $this->entityManager->getRepository(Repositories::FPA_VINCEVENTO)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($eventosVinculo as $eventoVinculo) {
	        $novoEventoVinculo = clone $eventoVinculo;
	        $novoEventoVinculo->setNrvinculom($nrvinculomNovo);
	        $novoEventoVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novoEventoVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novoEventoVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novoEventoVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novoEventoVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novoEventoVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novoEventoVinculo);
            
	    }
    }
    
    public function replicarValeTransporte($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $valesVinculo = $this->entityManager->getRepository(Repositories::VAB_LINHAVINC)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($valesVinculo as $valeVinculo) {
	        $novoValeVinculo = clone $valeVinculo;
	        
	        if($novoValeVinculo->getDtfimlinhavinc() != null && DateUtil::comparaDatas($novoValeVinculo->getDtfimlinhavinc(), $dtrescisao, 3)) {
	            $novoValeVinculo->setDtfimlinhavinc(null);
	        }
	        
	        $novoValeVinculo->setNrvinculom($nrvinculomNovo);
	        $novoValeVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novoValeVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novoValeVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novoValeVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novoValeVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novoValeVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novoValeVinculo);
            
	    }
    }
    
    public function replicarEstabilidade($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $estabilidadesVinculo = $this->entityManager->getRepository(Repositories::GPE_ALTESTABVINC)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($estabilidadesVinculo as $estabilidadeVinculo) {
	        $novaEstabilidadeVinculo = clone $estabilidadeVinculo;
	        
	        if($novaEstabilidadeVinculo->getDtfimestabvinc() != null && DateUtil::comparaDatas($novaEstabilidadeVinculo->getDtfimestabvinc(), $dtrescisao, 3)) {
	            $novaEstabilidadeVinculo->setDtfimestabvinc(null);
	        }
	        
	        $novaEstabilidadeVinculo->setNrvinculom($nrvinculomNovo);
	        $novaEstabilidadeVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novaEstabilidadeVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novaEstabilidadeVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novaEstabilidadeVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novaEstabilidadeVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novaEstabilidadeVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novaEstabilidadeVinculo);
            
	    }
    }
    
    public function replicarBeneficio($nrvinculom, $nrorg, $nrvinculomNovo, $dtrescisao) {
        $beneficiosVinculo = $this->entityManager->getRepository(Repositories::VAB_BENEFICIARIO)->findBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
		    
	    foreach($beneficiosVinculo as $beneficioVinculo) {
	        $novoBeneficioVinculo = clone $beneficioVinculo;
	        $novoBeneficioVinculo->setNrvinculom($nrvinculomNovo);
	        $novoBeneficioVinculo->setDtinclusao(DateUtil::getDataAtual());
            $novoBeneficioVinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $novoBeneficioVinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
	        $novoBeneficioVinculo->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
            $novoBeneficioVinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $novoBeneficioVinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($novoBeneficioVinculo);
            
	    }
    }

    public function validaPlanilhaVinculo($row) {
        //Dados básicos
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = []; //Mensages de erro, se estiver vazio não houve erros
        $impVinc     = []; //Vinculos com dados corretos a importar.
        $dataReturn  = [];
        
        //Colunas de data
        $dateColumns = ['DTADMISSAOVINC','DTRESCISAOVINC','DTMESCOMPETENC','DTCTPSPESSOA' ,'DTINSCPISPASEP'    ,'DTEXRGPESSOA','DTVALHABCARTPES','DTNASCPESSOA',
                        'DTOPCAOFGTS'   ,'DTAPOSENTAFGTS','DTPRIMADMISSAO','DTAVISOPREVIO','DTFIMCONTRDETERMIN','DTBASEFERIAS','DTEXAMEMEDICO'  ,'DTAVISOTRCT'];
        
        //Aquisição dos dados de exportação
        $dados = $this->getDadosPlanilhaVinculos($row['DSPATH'][0], $dateColumns);
        
        if(empty($dados) || (is_array($dados) && count($dados) <= 3))
            throw new \Exception('Não há dados para importar. Verifique se dados informados no arquivo estão corretos.');
        
        foreach($dados as $lin => &$dado) {
            if($lin <= 2)
                continue; //Pula as linhas informativas da planilha 2, 3 e 4. A leitura é feita a partir da linha 5.
        
            $nrRow = $lin + 2;

            foreach($dado as $nmcampo => &$value) {
                $this->trataDadosEspPlanilhaVinc($nmcampo, $value);
            }

            $res = $this->validaPlanilhaDadoVinculo($nrRow,                    $nrorg,                    $nrorgPadrao,
                                                    $dado['DTADMISSAOVINC'],   $dado['DTRESCISAOVINC'],   $dado['NRVINCULOM'],     $dado['NRSITUFUNCM'],       $dado['NRESCALATRABM'],
                                                    $dado['DTMESCOMPETENC'],   $dado['IDCONTRIBUISIND'],  $dado['NMVINCULOM'],     $dado['CDESTACIVIL'],       $dado['NRCPFPESSOA'],
                                                    $dado['NRCTPSPESSOA'],     $dado['NRSERIECTPSPES'],   $dado['SGUFCTPSPES'],    $dado['DTCTPSPESSOA'],      $dado['NRPISPASEPPES'],
                                                    $dado['DTINSCPISPASEP'],   $dado['NRRGPESSOA'],       $dado['CDEXRGPESSOA'],   $dado['SGUFRGPESSOA'],      $dado['DTEXRGPESSOA'],
                                                    $dado['NRTITUELEIPES'],    $dado['NRSECAELEIPES'],    $dado['NRZONAELEIPES'],  $dado['NRCARTHABPES'],      $dado['DTVALHABCARTPES'],
                                                    $dado['DSCATEGHABCART'],   $dado['NRCERTRESEPES'],    $dado['NRCATMILITARPE'], $dado['NRRACAPESSOA'],      $dado['CDPAIS'],
                                                    $dado['SGESTADO'],         $dado['CDMUNICIPIO_NAT'],  $dado['DTNASCPESSOA'],   $dado['IDSEXOPESSOA'],      $dado['IDREMUNERACAO'],
                                                    $dado['CDBANCO'],          $dado['CDAGENCIA'],        $dado['CDCONTCORR'],     $dado['NRTPCONTCORR'],      $dado['NRNACIONALIDADE'],
                                                    $dado['IDTPPAGAMENTO'],    $dado['NRESTRUTLEGAL'],    $dado['NRESTRUTGEREN'],  $dado['CDPAIS_END'],        $dado['SGESTADO_END'],
                                                    $dado['NMMUNICIPIO_END'],  $dado['CDLOGRADOURO_END'], $dado['DSENDERECO'],     $dado['NRIMOVELENDERECO'],  $dado['NMBAIRROENDERECO'],
                                                    $dado['DSCOMPLEENDERECO'], $dado['NRCEPENDERECO'],    $dado['NRTIPOVINCULOM'], $dado['NRTPDEMISSAO'],      $dado['DTOPCAOFGTS'],
                                                    $dado['DTAPOSENTAFGTS'],   $dado['DTPRIMADMISSAO'],   $dado['NROCUPACAOM'],    $dado['IDMULTVINC'],        $dado['CDCONTRIBINDIVIDUAL'],
                                                    $dado['NRMOTIVORESC'],     $dado['DSOBSRESCISAO'],    $dado['DTAVISOPREVIO'],  $dado['NRTPMOVTRANSFM'],    $dado['DTFIMCONTRDETERMIN'],
                                                    $dado['NRVINCULOEMPREG'],  $dado['NRTPADMISSAO'],     $dado['DTBASEFERIAS'],   $dado['NRDEPENDIR'],        $dado['NRDEPENDSFAM'],
                                                    $dado['NRTPMODALIDSAL'],   $dado['NRGRAUINSTR'],      $dado['NRESTRUTSIND'],   $dado['NRDIASCONTRATOEXP'], $dado['NRDIASCONTRATOEXPPRO'],
                                                    $dado['DTEXAMEMEDICO'],    $dado['DTAVISOTRCT'],      $dado['IDGRUPSANGUPES'], $dado['IDFATORRHPES'],      $dado['NRINSCRCONSREG'],
                                                    $dado['NRCONDFISPES'],     $dado['TELPRINCIPAL'],     $dado['DSEMAILOPER'],    $dado['VRSALARIO'],         $dado['NMPESSOAPAI'],
                                                    $dado['NMPESSOAMAE'],      $dado['DSEMAILOPER2'],     $dado,                   $dateColumns);
            
            if(count($res) > 0) {
                foreach($res as $v)
                    $messages[] = $v;
            } else {
                $impVinc[$nrRow] = $dado;
            }
        }
        //var_dump('VinculoServices',$dataReturn);die;
        $dataReturn['messages']     = $messages;
        $dataReturn['impVinculos']  = $impVinc;
        
        return $dataReturn;
    }

    private function getDadosPlanilhaVinculos($path, $dateColumns) {
        //Os ID's (campos) atribuídos aos dados da planilha
        $head = [
            'DTADMISSAOVINC',       //A - Data Admissão
            'DTRESCISAOVINC',       //B - Data Rescisão
            'NRVINCULOM',           //C - Nr. Vínculo
            'NRSITUFUNCM',          //D - Situ. Funcional
            'NRESCALATRABM',        //E - Escala Trabalho
            'DTMESCOMPETENC',       //F - Mês Competência
            'IDCONTRIBUISIND',      //G - Contribuiçao Sind.
            'NMVINCULOM',           //H - Nome
            'CDESTACIVIL',          //I - Estado Civil
            'NRCPFPESSOA',          //J - CPF
            'NRCTPSPESSOA',         //K - CTPS
            'NRSERIECTPSPES',       //L - Série CTPS
            'SGUFCTPSPES',          //M - UF CTPS
            'DTCTPSPESSOA',         //N - Emissao CTPS
            'NRPISPASEPPES',        //O - PIS PASEP
            'DTINSCPISPASEP',       //P - Data de Cadastro do PIS
            'NRRGPESSOA',           //Q - Carteira de Identidade
            'CDEXRGPESSOA',         //R - Orgao emissor carteira de identidade
            'SGUFRGPESSOA',         //S - UF Carteira Identidade
            'DTEXRGPESSOA',         //T - Data de emissão carteira de identidade
            'NRTITUELEIPES',        //U - Título de eleitor
            'NRSECAELEIPES',        //V - Seção de Votacão
            'NRZONAELEIPES',        //W - Zona de Votação
            'NRCARTHABPES',         //X - CNH
            'DTVALHABCARTPES',      //Y - Vencimento CNH
            'DSCATEGHABCART',       //Z - Tipo CNH
            'NRCERTRESEPES',        //AA - Certificado reservista
            'NRCATMILITARPE',       //AB - Categoria Militar
            'NRRACAPESSOA',         //AC - Raça
            'CDPAIS',               //AD - País
            'SGESTADO',             //AE - Estado
            'CDMUNICIPIO_NAT',      //AF - Naturalidade.          GPE_PESSOAH.CDMUNICIPIO
            'DTNASCPESSOA',         //AG - Data de Nascimento
            'IDSEXOPESSOA',         //AH - Sexo
            'IDREMUNERACAO',        //AI - Tipo Remuneração
            'CDBANCO',              //AJ - Banco
            'CDAGENCIA',            //AK - Agência
            'CDCONTCORR',           //AL - Conta Corrente
            'NRTPCONTCORR',         //AM - Tp Conta Corrente
            'NRNACIONALIDADE',      //AN - Nacionalidade
            'IDTPPAGAMENTO',        //AO - Tp. Pagamento
            'NRESTRUTLEGAL',        //AP - Estrutura Trabalho
            'NRESTRUTGEREN',        //AQ - Estrutura Gerencial
            'CDPAIS_END',           //AR - País_end
            'SGESTADO_END',         //AS - Estado_end
            'NMMUNICIPIO_END',      //AT - Municipio_end
            'CDLOGRADOURO_END',     //AU - Logradouro
            'DSENDERECO',           //AV - Endereço
            'NRIMOVELENDERECO',     //AW - Número
            'NMBAIRROENDERECO',     //AX - Nome do Bairro
            'DSCOMPLEENDERECO',     //AY - Complemento
            'NRCEPENDERECO',        //AZ - Cep
            'NRTIPOVINCULOM',       //BA - Tipo Vínculo Mestre
            'NRTPDEMISSAO',         //BB - Tipo de Demissão
            'DTOPCAOFGTS',          //BC - Data FGTS
            'DTAPOSENTAFGTS',       //BD - Dt. Aposentadoria FGTS
            'DTPRIMADMISSAO',       //BE - Data Primeira Admissão
            'NROCUPACAOM',          //BF - Ocupação
            'IDMULTVINC',           //BG - Múltiplos Vínculos
            'CDCONTRIBINDIVIDUAL',  //BH - Nr. Contrib. Individual
            'NRMOTIVORESC',         //BI - Motivo da Rescisão
            'DSOBSRESCISAO',        //BJ - Observação Rescisão
            'DTAVISOPREVIO',        //BK - Data Aviso Prévio
            'NRTPMOVTRANSFM',       //BL - Tp. Mov. Transferência
            'DTFIMCONTRDETERMIN',   //BM - Fim contrato Determinado
            'NRVINCULOEMPREG',      //BN - Vínculo Empregatício
            'NRTPADMISSAO',         //BO - Tipo Admissão
            'DTBASEFERIAS',         //BP - Dt. Base Férias
            'NRDEPENDIR',           //BQ - Dependentes IR.          GPE_VINCULOH
            'NRDEPENDSFAM',         //BR - Dependentes Sal Fam.     GPE_VINCULOH
            'NRTPMODALIDSAL',       //BS - Modalidade Salário
            'NRGRAUINSTR',          //BT - Grau Instrucao
            'NRESTRUTSIND',         //BU - Estrutura Sindical
            'NRDIASCONTRATOEXP',    //BV - Dias Contrato Exp.
            'NRDIASCONTRATOEXPPRO', //BW - Dias Contrato Exp. Prorr
            'DTEXAMEMEDICO',        //BX - Dt. Exame Médico
            'DTAVISOTRCT',          //BY - Dt. Aviso TRCT.      GPE_VINCULOM.DTAVISOTRCT
            'IDGRUPSANGUPES',       //BZ - Tipo Sanguineo
            'IDFATORRHPES',         //CA - Fator RH
            'NRINSCRCONSREG',       //CB - Inscrição Conselho
            'NRCONDFISPES',         //CC - Condição Física
            'TELPRINCIPAL',         //CD - Telefone Principal
            'DSEMAILOPER',          //CE - E-mail Operador
            'VRSALARIO',            //CF - Salário
            'NMPESSOAPAI',          //CG - Nome do Pai
            'NMPESSOAMAE',          //CH - Nome da Mãe
            'CDPREFIXCOMUPARC1',    //CI - DDD Telefone Principal
            'CDPREFIXCOMUPARC2',    //CJ - DDD Telefone Celular
            'TELCELULAR',           //CK - Telefone Celular
            'DSEMAILOPER2'          //CL - E-mail Pessoal
        ];

        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);

        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, ['NRVINCULOM','NRCPFPESSOA'], $dateColumns);
    }

    private function validaPlanilhaDadoVinculo($nrRow,             $nrorg,            $nrorgPadrao,
                                               $dtadmissaovinc,    $dtrescisaovinc,   $nrvinculom,     $nrsitufuncm,       $nrescalatrabm,
                                               $dtmescompetencRow, $idcontribuisind,  $nmvinculom,     $cdestacivil,       $nrcpfpessoa,
                                               $nrctpspessoa,      $nrseriectpspes,   $sgufctpspes,    $dtctpspessoa,      $nrpispaseppes,
                                               $dtinscpispasep,    $nrrgpessoa,       $cdexrgpessoa,   $sgufrgpessoa,      $dtexrgpessoa,
                                               $nrtitueleipes,     $nrsecaeleipes,    $nrzonaeleipes,  $nrcarthabpes,      $dtvalhabcartpes,
                                               $dscateghabcart,    $nrcertresepes,    $nrcatmilitarpe, $nrracapessoa,      $cdpais,
                                               $sgestado,          $cdmunicipio_nat,  $dtnascpessoa,   $idsexopessoa,      $idremuneracao,
                                               $cdbanco,           $cdagencia,        $cdcontcorr,     $nrtpcontcorr,      $nrnacionalidade,
                                               $idtppagamento,     $nrestrutlegal,    $nrestrutgeren,  $cdpais_end,        $sgestado_end,
                                               $nmmunicipio_end,   $cdlogradouro_end, $dsendereco,     $nrimovelendereco,  $nmbairroendereco,
                                               $dscompleendereco,  $nrcependereco,    $nrtipovinculom, $nrtpdemissao,      $dtopcaofgts,
                                               $dtaposentafgts,    $dtprimadmissao,   $nrocupacaom,    $idmultvinc,        $cdcontribindividual,
                                               $nrmotivoresc,      $dsobsrescisao,    $dtavisoprevio,  $nrtpmovtransfm,    $dtfimcontrdetermin,
                                               $nrvinculoempreg,   $nrtpadmissao,     $dtbaseferias,   $nrdependir,        $nrdependsfam,
                                               $nrtpmodalidsal,    $nrgrauinstr,      $nrestrutsind,   $nrdiascontratoexp, $nrdiascontratoexppro,
                                               $dtexamemedico,     $dtavisotrct,      $idgrupsangupes, $idfatorrhpes,      $nrinscrconsreg,
                                               $nrcondfispes,      $telprincipal,     $dsemailoper,    $vrsalario,         $nmpessoapai,
                                               $nmpessoamae,       $dsemailoper2,     &$dado,          $dateColumns) {
        try {
            $nmocupacaoh = null; $return = [];
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $detailMsg = " Erro na linha $nrRow. Vinculo $nrvinculom - $nmvinculom.";
            $return = $this->validaDatas($dado, $dateColumns, $return, $nrRow, $detailMsg);

            if(!empty($return)) {
                return $return;
            }

            if($dtadmissaovinc)
                $dateadmissaovinc = DateUtil::getDataDeString($dtadmissaovinc, DateUtil::FORMATO_BRASILEIRO, true);
            else {
                $return[] = ["msg" => "A data de admissão não pode ser vazia. $detailMsg"];
                return $return;
            }

            if(!$nrvinculom && !$nrcpfpessoa) { 
                $return[] = ["msg" => "O Nr. Vínculo ou CPF deve ser informado. $detailMsg"];
            }
            
            if($nrcpfpessoa) {
                if(strlen($nrcpfpessoa) < 11) { 
                    $return[] = ["msg" => "CPF $nrcpfpessoa possui menos de 11 dígitos. $detailMsg"];
                    return $return;
                }
                $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpfImpVinc($nrorg, $dtmescompetenc, $nrcpfpessoa);
                
                if($pessoa) {
                    $pessoa = $pessoa[0]; // A posição 0 retorna o histórico mais recente.

                    if($pessoa['VINCULOATIVO'] == 'S') {
                        $return[] = ["msg" => 'CPF '.$nrcpfpessoa.' já cadastrado para a pessoa '.$pessoa['NMPESSOA'].' e o vínculo está ativo, logo não pode ser cadastrado. '.$detailMsg]; 
                    } else {
                        //Armazena os dados da pessoa para ser reaproveitado na importação da admissão.
    
                        $dado['NRPESSOA'] = $pessoa['NRPESSOA'];
                        $dado['NRPARCNEGOCIO'] = $pessoa['NRPARCNEGOCIO'];
                        $dado['NMPESSOAMAE'] = null; //Remove os dados de mãe e pai para não inserir dependentes do parceiro.
                        $dado['NMPESSOAPAI'] = null;
                    }
                }
            }
            
            if($dtrescisaovinc) {
                $daterescisaovinc = DateUtil::getDataDeString($dtrescisaovinc, DateUtil::FORMATO_BRASILEIRO, true);

                if(DateUtil::comparaDatas($daterescisaovinc, $dateadmissaovinc)) {
                    $return[] = ["msg" => "A data de rescisão $dtrescisaovinc não pode ser inferior a data de admissão $dtadmissaovinc. $detailMsg"];
                    return $return;
                }
            }

            if(($dtrescisaovinc && !$nrtpdemissao) || (!$dtrescisaovinc && $nrtpdemissao)) {
                $return[] = ["msg" => "A data de rescisão e o Tipo de Demissão devem ser informados juntos. $detailMsg"];
            } else {
                if($nrtpdemissao) {
                    $tpDemissao = $this->entityManager->getRepository(Repositories::FPA_TPDEMISSAO)->findOneBy(["nrtpdemissao" => $nrtpdemissao, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$tpDemissao)
                        $return[] = ["msg" => "Tipo de Demissão $nrtpdemissao não encontrado. $detailMsg"];
                }
            }

            if(!$nrsitufuncm || !$nrescalatrabm) {
                $return[] = ["msg" => "A situação funcional e/ou escala de trabalho não podem ser vazios. $detailMsg"];
            } else {
                if($nrsitufuncm) {
                    $situacaom = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->findOneBy(["nrsitufuncm" => $nrsitufuncm, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$situacaom)
                        $return[] = ["msg" => "Situação funcional $nrsitufuncm não encontrada. $detailMsg"];
                }

                if($nrescalatrabm) {
                    $escalam = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABM)->findOneBy(["nrescalatrabm" => $nrescalatrabm, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$escalam)
                        $return[] = ["msg" => "Escala de trabalho $nrescalatrabm não encontrada. $detailMsg"];
                }
            }

            if(!$dtmescompetencRow) {
                $return[] = ["msg" => "O mês competência não pode ser vazio. $detailMsg"];
                return $return;
            }
            
            if(!$idcontribuisind || !$nmvinculom || !$cdestacivil) {
                $return[] = ["msg" => "A contribuição sindical e/ou o nome e/ou estado civil do vínculo não podem ser vazios. $detailMsg"];
            } else {
                if($idcontribuisind) {
                    $idcontribuisindAceitos = ['NUNCA','DESCONTOU','SEMPRE'];

                    if(!in_array($idcontribuisind, $idcontribuisindAceitos))
                        $return[] = ["msg" => "Contribuição sindical $idcontribuisind inválida. $detailMsg"];
                }

                if($cdestacivil) {
                    $estadoCivil = $this->entityManager->getRepository(Repositories::ESTADOCIVIL)->findOneBy(["cdestacivil" => $cdestacivil, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$estadoCivil)
                        $return[] = ["msg" => "Estado civil $cdestacivil inválido. $detailMsg"];
                }
            }

            /*if(!$nrctpspessoa || !$nrseriectpspes || !$sgufctpspes || !$dtctpspessoa || !$nrpispaseppes || !$dtinscpispasep) {
                $return[] = ["msg" => "Os dados da carteira de trabalho (CTPS) e/ou dados PIS/PASEP não podem ser vazios. $detailMsg"];
            } else*/
            if($nrpispaseppes && !$this->validaPIS($nrpispaseppes)) {
                $return[] = ["msg" => "Número do PIS/PASEP $nrpispaseppes inválido. $detailMsg"];
            }
            
            if(!$nrrgpessoa || !$cdexrgpessoa || !$sgufrgpessoa)
                $return[] = ["msg" => "O número e/ou o orgão emissor e/ou a UF da carteira de identidade não podem ser vazios. $detailMsg"];
                
            if(!$nrracapessoa) {
                $return[] = ["msg" => "A raça não pode ser vazio. $detailMsg"];
            } else {
                $raca = $this->entityManager->getRepository(Repositories::GPE_RACAPESSOA)->findOneBy(["nrracapessoa" => $nrracapessoa, "nrorg" => [$nrorg, $nrorgPadrao]]);

                if(!$raca)
                    $return[] = ["msg" => "Raça $nrracapessoa inválida. $detailMsg"];
            }
            
            if(!$cdpais || !$dtnascpessoa) {
                $return[] = ["msg" => "O país e/ou data de nascimento não podem ser vazios. $detailMsg"];
            } else {
                if($cdpais) {
                    $pais = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(["cdpais" => $cdpais, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$pais)
                        $return[] = ["msg" => "País $cdpais inválido. $detailMsg"];
                }
            }

            if(!$idsexopessoa || !$idremuneracao) {
                $return[] = ["msg" => "O sexo e/ou tipo de remuneração não podem ser vazios. $detailMsg"];
            } else {
                if($idsexopessoa) {
                    $idsexoAceitos = ['M','F'];

                    if(!in_array($idsexopessoa, $idsexoAceitos))
                        $return[] = ["msg" => "Sexo do vínculo $idsexopessoa inválido. $detailMsg"];
                }

                if($idremuneracao) {
                    $idremuneracaoAceitos = ['SAL1','SAL1_RSR','SAL2','SAL2_RSR','ESTAGIO','SOCIO','AUTONOMO','NENHUM'];

                    if(!in_array($idremuneracao, $idremuneracaoAceitos))
                        $return[] = ["msg" => "Tipo de remuneração $idremuneracao inválido. $detailMsg"];
                }
            }

            if(!$nrnacionalidade || !$idtppagamento) {
                $return[] = ["msg" => "A nacionalidade e/ou tipo de pagamento não podem ser vazios. $detailMsg"];
            } else {
                if($nrnacionalidade) {
                    $nacionalidade = $this->entityManager->getRepository(Repositories::GPE_NACIONALIDADE)->findOneBy(["nrnacionalidade" => $nrnacionalidade, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$nacionalidade)
                        $return[] = ["msg" => "Nacionalidade $nrnacionalidade inválida. $detailMsg"];
                }

                if($idtppagamento) {
                    $idtppagamentoAceitos = ['QUINZENAL','DIARIO','MENSAL','MENSAL_SALARIO_HORA','SEMANAL'];

                    if(!in_array($idtppagamento, $idtppagamentoAceitos))
                        $return[] = ["msg" => "Tipo de pagamento $idtppagamento inválido. $detailMsg"];
                }
            }

            if(!$nrtipovinculom) {
                $return[] = ["msg" => "O Tipo Vínculo Mestre não pode ser vazio. $detailMsg"];
                return $return;
            } else {
                $tipovinculo = $this->entityManager->getRepository(Repositories::GPE_TIPOVINCULOH)->getTipoVinculo($nrtipovinculom, $nrorg, $nrorgPadrao, $dtmescompetenc);
                
                if(!$tipovinculo)
                    $return[] = ["msg" => "Tipo Vínculo Mestre $nrtipovinculom inválido. $detailMsg"];
            }

            if(!$nrestrutlegal || !$nrestrutgeren || ($nrtipovinculom == 1 && !$nrestrutsind) ) {
                $return[] = ["msg" => "A estrutura de trabalho e/ou gerencial e/ou sindical não podem ser vazios. $detailMsg"];
            } else {
                if($nrestrutlegal) {
                    $estruturaLegal = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstruturaExistenteComTipo($nrorg, $nrestrutlegal, $dtmescompetenc, $this->hcmParameters->getTipoEstruturaLegalOrg());
                    
                    if(!$estruturaLegal)
                        $return[] = ["msg" => "Estrutura de Trabalho $nrestrutlegal inválida. $detailMsg"];
                }
                if($nrestrutgeren) {
                    $estruturaGeren = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstruturaExistenteComTipo($nrorg, $nrestrutgeren, $dtmescompetenc, $this->hcmParameters->getTipoEstruturaGerencialOrg());
                    
                    if(!$estruturaGeren)
                        $return[] = ["msg" => "Estrutura Gerencial $nrestrutgeren inválida. $detailMsg"];
                }
                if($nrestrutsind) {
                    $estruturaSind = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstruturaExistenteComTipo($nrorg, $nrestrutsind, $dtmescompetenc, 10);
                    
                    if(!$estruturaSind)
                        $return[] = ["msg" => "Estrutura Sindical $nrestrutsind inválida. $detailMsg"];
                }
            }

            if(!$cdpais_end || !$sgestado_end || !$nmmunicipio_end || !$cdlogradouro_end || !$dsendereco || !$nrimovelendereco || !$nmbairroendereco || !$nrcependereco) {
                $return[] = ["msg" => "Os dados de endereço do vínculo não podem ser vazios. $detailMsg"];
            } else {
                if($cdpais_end) {
                    $pais = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(["cdpais" => $cdpais_end, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$pais) {
                        $return[] = ["msg" => "País $cdpais_end do endereço do vínculo está inválido. $detailMsg"];
                    } else if($sgestado_end) {
                        $estado = $this->entityManager->getRepository(Repositories::ESTADO)->findOneBy(["cdpais" => $cdpais_end, "sgestado" => $sgestado_end ,"nrorg" => [$nrorg, $nrorgPadrao]]);

                        if(!$estado) {
                            $return[] = ["msg" => "Estado $sgestado_end do endereço do vínculo está inválido. $detailMsg"]; 
                        } else if($nmmunicipio_end) {
                            $municipio = $this->entityManager->getRepository(Repositories::MUNICIPIO)->getCityNameLike($sgestado_end, $cdpais_end, strtoupper($nmmunicipio_end));

                            if(!$municipio) {
                                $return[] = ["msg" => "Município $nmmunicipio_end do endereço do vínculo está inválido. $detailMsg"]; 
                            } else {
                                $dado['CDMUNICIPIO_END'] = $municipio[0]['CDMUNICIPIO'];
                            }
                        }
                    }
                }
                if($cdlogradouro_end) {
                    $logradouro = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findOneBy(["cdlogradouro" => $cdlogradouro_end, "nrorg" => [$nrorg, $nrorgPadrao]]);
                    
                    if(!$logradouro)
                        $return[] = ["msg" => "Logradouro $cdlogradouro_end inválido. $detailMsg"];
                }
            }

            if(!$nrocupacaom || !$idmultvinc) {
                $return[] = ["msg" => "A Ocupação e/ou Múltiplos Vínculos não podem ser vazios. $detailMsg"];
            } else {
                if($nrocupacaom) {
                    $ocupacao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistorico($nrocupacaom, $nrorg, $dtmescompetenc);

                    if(!$ocupacao) {
                        $return[] = ["msg" => "Ocupação $nrocupacaom inválida. $detailMsg"];
                        return $return;
                    } else {
                        $nmocupacaoh = $ocupacao->getNmocupacaoh();
                    }
                }
                if($idmultvinc) {
                    $idmultvincAceitos = ['UNICO','PRIMARIO','SECUNDARIO'];

                    if(!in_array($idmultvinc, $idmultvincAceitos))
                        $return[] = ["msg" => "Múltiplos Vínculos $idmultvinc inválido. $detailMsg"];
                }
            }
            
            if(!$nrvinculoempreg || !$nrtpadmissao || !$dtbaseferias) {
                $return[] = ["msg" => "O Vínculo Empregatício e/ou Tipo Admissão e/ou Dt. Base Férias não podem ser vazios. $detailMsg"];
            } else {
                if($nrvinculoempreg) {
                    $vinculoEmpreg = $this->entityManager->getRepository(Repositories::GPE_VINCULOEMPREG)->findOneBy(['nrvinculoempreg' => $nrvinculoempreg]);

                    if(!$vinculoEmpreg)
                        $return[] = ["msg" => "Vínculo Empregatício $nrvinculoempreg inválido. $detailMsg"];
                }
                if($nrtpadmissao) {
                    $tipoAdmissao = $this->entityManager->getRepository(Repositories::FPA_TPADMISSAO)->findOneBy(['nrtpadmissao' => $nrtpadmissao, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$tipoAdmissao)
                        $return[] = ["msg" => "Tipo de Admissão $nrtpadmissao inválido. $detailMsg"];
                }
            }

            if(!$nrtpmodalidsal || !$nrgrauinstr) {
                $return[] = ["msg" => "A Modalidade Salário e/ou Grau Instrução não podem ser vazios. $detailMsg"];
            } else {
                if($nrtpmodalidsal) {
                    $tpmodalidsal = $this->entityManager->getRepository(Repositories::FPA_TPMODALIDSAL)->findOneBy(['nrtpmodalidsal'=> $nrtpmodalidsal, "nrorg" => [$nrorg, $nrorgPadrao]]);

                    if(!$tpmodalidsal)
                        $return[] = ["msg" => "Modalidade de Salário $nrtpmodalidsal inválida. $detailMsg"];
                }
                if($nrgrauinstr) {
                    $grauinstr = $this->entityManager->getRepository(Repositories::GPE_GRAUINSTR)->findOneBy(['nrgrauinstr' => $nrgrauinstr, "nrorg" => $nrorgPadrao]);

                    if(!$grauinstr)
                        $return[] = ["msg" => "Grau de Instrução $nrgrauinstr inválido. $detailMsg"];
                }
            }

            if($dsemailoper) {
                $existeOperadorDsemailoper = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(['dsemailoper' => $dsemailoper]);

                if($existeOperadorDsemailoper) {
                    $return[] = ["msg" => "O e-mail operador $dsemailoper já está cadastrado para o operador ".$existeOperadorDsemailoper->getNmoperador().". $detailMsg"];
                }

                if(!filter_var($dsemailoper, FILTER_VALIDATE_EMAIL)) {
                    $return[] = ["msg" => "O e-mail operador $dsemailoper está inválido. $detailMsg"];
                }
            }

            if($dsemailoper2 && !filter_var($dsemailoper2, FILTER_VALIDATE_EMAIL)) {
                $return[] = ["msg" => "O e-mail pessoal $dsemailoper2 está inválido. $detailMsg"];
            }

            if($cdbanco && (!$cdagencia || !$cdcontcorr || !$nrtpcontcorr))
                $return[] = ["msg" => " Foi informado o código do banco então os demais dados: Agência e/ou Conta Corrente e/ou Tp Conta Corrente não podem ser vazios. $detailMsg"];

            if(!$vrsalario || !$nmpessoamae) {
                $return[] = ["msg" => "O Salário e/ou nome da mãe do vínculo não podem ser vazios. $detailMsg"];
                return $return;
            }
            else if($vrsalario && $this->isvalidSalarioPisoTetoAdmissaoImp($nrorg, $vrsalario, $nrocupacaom, $dtmescompetenc) === false) {
                $return[] = ["msg" => "O Salário R$$vrsalario é incompatível com o cargo: $nmocupacaoh. Favor verificar em 'Tabela Salarial de Ocupação'. $detailMsg"];
            }

            if($dtfimcontrdetermin) {
                $datedtfimcontrdetermin = DateUtil::getDataDeString($dtfimcontrdetermin, DateUtil::FORMATO_BRASILEIRO, true);

                if(DateUtil::comparaDatas($datedtfimcontrdetermin, $dateadmissaovinc) || DateUtil::comparaDatas($datedtfimcontrdetermin, $dateadmissaovinc, DateUtil::IGUAL)) {
                    $return[] = ["msg" => "Data de Fim de Contrato $dtfimcontrdetermin não deve ser menor ou igual a data de Admissão $dtadmissaovinc. $detailMsg"];
                }
            }
           
        } catch (\Exception $e) {
            throw $e;
        }
        return $return;
    }
    
    public function getVerificacaoQuadroVagasImportacao($nrorg, $nrestrutgeren, $nrocupacaom, $nrescalatrabm, $nrestrutlegal, $nrestrutsind): array
    { 
        $dadosRetornados = $this->vinculoRepository->getVerificacaoQuadroVagasImportacao($nrorg, $nrestrutgeren, $nrocupacaom, $nrescalatrabm, $nrestrutlegal, $nrestrutsind);
      
        $movimentacoes = [
            'GERENCIAL' => [
                'TIPOESTRUTURA' => $dadosRetornados['NRTIPOESTRUTURA_GEREN'],
                'NRESTRUTURA' => $nrestrutgeren,
                'NMESTRUTURA' => $dadosRetornados['NMRAZSOCESTRUT_GEREN'],
                'NROCUPACAO' => $nrocupacaom,
                'NMOCUPACAO' => $dadosRetornados['NMOCUPACAOH'],
                'NRESCALATRABM' => $nrescalatrabm,
                'NMESCALATRABH' => $dadosRetornados['NMESCALATRABH']
            ],
            'LEGAL' => [
                'TIPOESTRUTURA' => $dadosRetornados['NRTIPOESTRUTURA_LEGAL'],
                'NRESTRUTURA' => $nrestrutlegal,
                'NMESTRUTURA' => $dadosRetornados['NMRAZSOCESTRUT_LEGAL'],
                'NROCUPACAO' => $nrocupacaom,
                'NMOCUPACAO' => $dadosRetornados['NMOCUPACAOH'],
                'NRESCALATRABM' => $nrescalatrabm,
                'NMESCALATRABH' => $dadosRetornados['NMESCALATRABH']
            ],
            'SINDICAL' => [
                'TIPOESTRUTURA' => $dadosRetornados['NRTIPOESTRUTURA_SINDICAL'] ?? null,
                'NRESTRUTURA' => $nrestrutsind ?? null,
                'NMESTRUTURA' => $dadosRetornados['NMRAZSOCESTRUT_SINDICAL'] ?? null,
                'NROCUPACAO' => $nrocupacaom,
                'NMOCUPACAO' => $dadosRetornados['NMOCUPACAOH'],
                'NRESCALATRABM' => $nrescalatrabm,
                'NMESCALATRABH' => $dadosRetornados['NMESCALATRABH']
            ],
            
        ];
        
        $verificadorQuadroVagas = new QuadroVagas\Verificador;
        
        return $verificadorQuadroVagas->getVerificacao($movimentacoes);
    }

    private function validaDatas($dado, $dateColumns, $return, $nrRow, $detailMsg){
        $labelDateColumns = ['Data Admissão','Data Rescisão'         ,'Mês Competência'       ,'Emissao CTPS'     ,'Data de Cadastro do PIS' ,'Data de emissão carteira de identidade','Vencimento CNH'  ,'Data de Nascimento',
                             'Data FGTS'    ,'Dt. Aposentadoria FGTS','Data Primeira Admissão','Data Aviso Prévio','Fim contrato Determinado','Dt. Base Férias'                       ,'Dt. Exame Médico','Dt. Aviso TRCT'];

        foreach($dado as $fieldDado => $valueDado) {
            $test = null;

            foreach($dateColumns as $key => $dateColumn) {
                if($valueDado && ($fieldDado == $dateColumn)) {
                    $test = DateUtil::getDataDeString($valueDado, DateUtil::FORMATO_BRASILEIRO, true);

                    if(!$test) {
                        $return[] = ["msg" => "O campo ".$labelDateColumns[$key]." está inválido. $detailMsg"];
                    }
                }
            }
        }
        return $return;
    }

    private function validaPIS($pis) {
        $multiplicadorBase = "3298765432";
        $total = 0;
        $resto = 0;
        $multiplicando = 0;
        $multiplicador = 0;
        $digito = 99;
        
        // Retira a mascara
        $numeroPIS = preg_replace('/[^0-9]/', '', $pis);
        
        if (strlen($numeroPIS) !== 11 || 
            $numeroPIS === "00000000000" || 
            $numeroPIS === "11111111111" || 
            $numeroPIS === "22222222222" || 
            $numeroPIS === "33333333333" || 
            $numeroPIS === "44444444444" || 
            $numeroPIS === "55555555555" || 
            $numeroPIS === "66666666666" || 
            $numeroPIS === "77777777777" || 
            $numeroPIS === "88888888888" || 
            $numeroPIS === "99999999999") {
            return false;
        } else {
            for ($i = 0; $i < 10; $i++) {
                $multiplicando = intval( substr($numeroPIS, $i, 1));
                $multiplicador = intval( substr($multiplicadorBase, $i, 1));
                $multiplicacao = $multiplicando * $multiplicador;
                $total += $multiplicacao;
            }
        
            $resto = 11 - $total % 11;
            $resto = $resto === 10 || $resto === 11 ? 0 : $resto;
        
            $digito = intval(substr($numeroPIS, 10));
            return $resto === $digito;
        }
    }

    private function isvalidSalarioPisoTetoAdmissaoImp($nrorg, $vrsalario, $nrocupacaom, $dtmescompetenc) {
        /*Função replicada para a rotina de importação da planilha de vínculos.*/

        $dtUltDiaComp = DateUtil::getUltimoDiaDoMes($dtmescompetenc)->format('d/m/Y');
        $pisoTetoSalarial = $this->vinculoRepository->getpisoTetoSalarialImp($nrorg, $nrocupacaom, $dtUltDiaComp);
        
        if($pisoTetoSalarial){
            if($vrsalario >= $pisoTetoSalarial['VRPISOTABSALARIAL'] && $vrsalario <= $pisoTetoSalarial['VRTETOTABSALARIAL']){
                //Caso seja encontrado um cargo cadastrado com os parametros informados na isvalidSalarioPisoTeto e o salario esteja dentro dos limites o retorno é true
                return true;
            }
            //Caso seja encontrado um cargo cadastrado com os parametros informados na isvalidSalarioPisoTeto porém o salario está fora dos limites o retorno é false
            return false;
        }
        //Caso não seja encontrado nenhum cargo cadastrado com os parametros informados na isvalidSalarioPisoTeto o retorno é nulo
        return null;
    }

    private function trataDadosEspPlanilhaVinc($nmcampo, &$value) { 
        $arrCamposMask = ['NRCPFPESSOA', 'NRPISPASEPPES'];
        $arrCamposEsp  = ['NMMUNICIPIO_END', 'CDLOGRADOURO_END', 'DSENDERECO', 'NRIMOVELENDERECO', 'NMBAIRROENDERECO', 'DSCOMPLEENDERECO'];

        if(empty($value)) {
            $value = null;
        } else if(gettype($value) == 'string') {
            $value = trim($value);
        }

        if(in_array($nmcampo, $arrCamposMask)) {
            $value = str_replace(['.','/','-'], '', $value);
        } else if(in_array($nmcampo, $arrCamposEsp)) {
            $value = Util::removeCaracteresEspeciais($value);
        }
    }
    
}