<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class OcorrenciaFrequenciaAdmissaoController extends Crud {
    
    
    protected $dataSourceNameGet = 'getOcorrenciaFrequenciaAdmissao';
    protected $dataSourceName = 'pto_ocorrenciaFrequenciaAdmissao';
    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }
    
    public function getOcorrenciaFrequenciaAdmissao(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            
            foreach($rows as &$row){
                $row['NRESTRUTSIND']    = isset($row['NRESTRUTSIND'])   ? explode(",", $row['NRESTRUTSIND'])    : null;
                $row['NRESTRUTLEGAL']   = isset($row['NRESTRUTLEGAL'])  ? explode(",", $row['NRESTRUTLEGAL'])   : null;
                $row['NRESTRUTGEREN']   = isset($row['NRESTRUTGEREN'])  ? explode(",", $row['NRESTRUTGEREN'])   : null;
                $row['NROCUPACAOM']     = isset($row['NROCUPACAOM'])    ? explode(",", $row['NROCUPACAOM'])     : null;
                $row['NRESCALATRABM']   = isset($row['NRESCALATRABM'])  ? explode(",", $row['NRESCALATRABM'])   : null;
                
                $row['NMESTRUTSIND']    = isset($row['NMESTRUTSIND'])   ? explode(",", $row['NMESTRUTSIND'])    : null;
                $row['NMESTRUTLEGAL']   = isset($row['NMESTRUTLEGAL'])  ? explode(",", $row['NMESTRUTLEGAL'])   : null;
                $row['NMESTRUTGEREN']   = isset($row['NMESTRUTGEREN'])  ? explode(",", $row['NMESTRUTGEREN'])   : null;
                $row['NMOCUPACAOH']     = isset($row['NMOCUPACAOH'])    ? explode(",", $row['NMOCUPACAOH'])     : null;
                $row['NMESCALATRABH']   = isset($row['NMESCALATRABH'])  ? explode(",", $row['NMESCALATRABH'])   : null;
            }
            
            $response->addDataSet(new DataSet($filterData->getDataSourceName(), $rows));
            
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    public function setOcorrenciaFrequenciaAdmissao(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NROCORRENCIADMISSAO', $row)) {
                    if ($row['NROCORRENCIADMISSAO'] === '') {
                        $row['NROCORRENCIADMISSAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'PTO_OCORRENCIADMISSAO', 1, 12, true); 
                    }
                } else {
                    $row['NROCORRENCIADMISSAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'PTO_OCORRENCIADMISSAO', 1, 12, true); 
                }
                if(array_key_exists('IDDIASSEMANA', $row)){
                    $row['IDDIASSEMANA'] = implode(',', $row['IDDIASSEMANA']);
                }
                if(is_array($row['NRESTRUTSIND'])){
                    $row['NRESTRUTSIND'] = implode(',', $row['NRESTRUTSIND']);
                }
                if(is_array($row['NRESTRUTLEGAL'])){
                   $row['NRESTRUTLEGAL'] = implode(',', $row['NRESTRUTLEGAL']);
                }  
                if(is_array($row['NRESTRUTGEREN'])){
                   $row['NRESTRUTGEREN'] = implode(',', $row['NRESTRUTGEREN']);
                } 
                if(is_array($row['NROCUPACAOM'])){
                   $row['NROCUPACAOM'] = implode(',', $row['NROCUPACAOM']);
                } 
                if(is_array($row['NRESCALATRABM'])){
                   $row['NRESCALATRABM'] = implode(',', $row['NRESCALATRABM']);
                }
                if(empty($row['NRHORDIAM'])){
                    $row['NRHORDIAM'] = null;
                }
            }
            parent::save($request, $response); 
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function deleteOcorrenciaFrequenciaAdmissao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}