<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 29/06/2016
 * Time: 17:42
 */

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class FatorriscoController extends Crud {

    protected $dataSourceName = 'sst_fatorrisco';
    
    protected $dataSourceName_RiscoEpc = 'sst_riscoEpc';
    
    protected $dataSourceName_RiscoEpi_Get = 'sst_riscoEpi_get';
    protected $dataSourceName_RiscoEpi_Set = 'sst_riscoEpi_set';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getFatorrisco (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName('sst_fatorrisco_get');
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
     
    }

    public function setFatorrisco (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFATORRISCO', $row)) {
                    if ($row['NRFATORRISCO'] === '') {
                        $row['NRFATORRISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FATORRISCO', 1, 12, true);
                    }
                } else {
                    $row['NRFATORRISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FATORRISCO', 1, 12, true);
                }
            }
            
            $row['IDEPCEFICAZ'] = isset($row['IDEPCEFICAZ'])?$row['IDEPCEFICAZ']:"N";
            $row['IDEPIEFICAZ'] = isset($row['IDEPIEFICAZ'])?$row['IDEPIEFICAZ']:"N";
            $row['NRINSTRUMENTO'] = $row['NRINSTRUMENTO'] ?? null;
            $row['NRMARCA'] = $row['NRMARCA'] ?? null;
            $row['DSUNIDADE'] = $row['DSUNIDADE'] ?? null;
            $row['DSSUGESTOES'] = $row['DSSUGESTOES'] ?? null;
            $row['DSMETODOLOGIA'] = $row['DSMETODOLOGIA'] ?? null;
            $row['DSRISCOS'] = $row['DSRISCOS'] ?? null;
            
            if (!array_key_exists('NRFATRISCAMB', $row)){
                 $row['NRFATRISCAMB'] = null;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFatorrisco(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function getRiscoEpc (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_RiscoEpc;
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRiscoEpc (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_RiscoEpc;
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRRISCOEPC', $row)) {
                    if ($row['NRRISCOEPC'] === '') {
                        $row['NRRISCOEPC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_RISCOEPC', 1, 12, true);
                    }
                } else {
                    $row['NRRISCOEPC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_RISCOEPC', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRiscoEpc (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_RiscoEpc;
        parent::delete($request, $response);
    }

    public function getRiscoEpi (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_RiscoEpi_Get;
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRiscoEpi (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_RiscoEpi_Set;
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRRISCOEPI', $row)) {
                    if ($row['NRRISCOEPI'] === '') {
                        $row['NRRISCOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_RISCOEPI', 1, 12, true);
                    }
                } else {
                    $row['NRRISCOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_RISCOEPI', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRiscoEpi (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_RiscoEpi_Set;
        parent::delete($request, $response);
    }

    public function getUnidadeMedida(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "unidmedida_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 