<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\HistoricoEpiService;
use HCM\SST\Repository\PerfilepiRepository;

class HistoricoEpiController extends Crud {

    protected $dataSourceName = 'sst_historicoepi_get';

    protected $parameterBagColumns = ['P_NRORG','P_NRORG_PADRAO','P_DTMESCOMPETENCIA','P_NRTIPOESTRUTGEREN','P_NRTIPOESTRUTOMADOR','NRVINCULOM_LIST'];
    protected $toConvertInArray = array('NRVINCULOM_LIST');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, HistoricoEpiService $historicoEpiService, PerfilepiRepository $perfilepiRepository){
        
        $this->historicoEpiService = $historicoEpiService;
        $this->hcmParameters       = $hcmParameters;
        $this->parameterBag        = $parameterBag;
        $this->perfilepiRepository = $perfilepiRepository;
        
        parent::__construct($dataSourceManager);
    }

    public function getHistoricoEpi(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);

        $filterData->addCondition('P_DTMESCOMPETENCIA','=',$this->hcmParameters->getDtCompetencia());
        $filterData->addCondition('P_NRTIPOESTRUTGEREN', '=', $this->hcmParameters->getTipoEstruturaGerencialOrg());
        $filterData->addCondition('P_NRTIPOESTRUTOMADOR', '=', $this->hcmParameters->getTipoEstruturaTomadorOrg());

        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setHistoricoEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_historicoepi";

            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRHISTORICOEPI', $row)) {
                    if ($row['NRHISTORICOEPI'] === '') {
                        $row['NRHISTORICOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                    }
                } else {
                    $row['NRHISTORICOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                }
                $row['NRORG']      = intval($row['NRORG']);
                $row['NRVINCULOM'] = intval($row['NRVINCULOM']);
                $this->historicoEpiService->validaCamposVazios($row);
            }

            //Verifica se é um registro novo de lista de vínculos e EPI's
            $row = $request->getDataSet()->getRows()[0];
            $lista_vinculo = isset( $row['NRVINCULOM_LIST'] ) ? $row['NRVINCULOM_LIST'] : null;
            $lista_epi     = isset( $row['NREPI_LIST'] )      ? $row['NREPI_LIST']      : null;

            //Campos que foram removidos da tela
            $lista_dtvalid = isset( $row['DTVALIDADEEPI'] )   ? $row['DTVALIDADEEPI']   : null;
            $lista_fornece = isset( $row['NMPRINCIPALPARC'] ) ? $row['NMPRINCIPALPARC'] : null;
            $lista_modelo  = isset( $row['DSMODELOEPI'] )     ? $row['DSMODELOEPI']     : null;
            $lista_ca      = isset( $row['NRCTRLAPROVEPI'] )  ? $row['NRCTRLAPROVEPI']  : null;

            if( isset($lista_vinculo) ) {
                foreach( $request->getDataSet()->getRows() as &$row ) {
                    foreach( $lista_vinculo as $nrvinculom ) {
                        foreach( $lista_epi as $key => $nrepi ) {
                            $row['NRVINCULOM'] = intval( $nrvinculom );
                            $row['NREPI']      = intval( $nrepi );
                            //Campos que foram removidos da tela
                            $row['DTVALIDADEEPI']   = $lista_dtvalid[ $key ];
                            $row['NMPRINCIPALPARC'] = $lista_fornece[ $key ];
                            $row['DSMODELOEPI']     = $lista_modelo [ $key ];
                            $row['NRCTRLAPROVEPI']  = $lista_ca     [ $key ];
                            //Save nativo
                            parent::save( $request, $response );
                            //Novo código para os próximos registros
                            $row['NRHISTORICOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                        }
                    }
                }
            } else {
                parent::save( $request, $response );
            }

            /*$ehLista = $request->getDataSet()->getRows();
            $ehLista = isset($ehLista[0]['NRVINCULOM_LIST']);
            if( $ehLista ) {
                foreach( $request->getDataSet()->getRows() as &$row ) {
                    foreach( $row['NRVINCULOM_LIST'] as $nrvinculom ) {
                        $row['NRVINCULOM'] = intval( $nrvinculom );
                        foreach( $row['NREPI_LIST'] as $nrepi ) {
                            $row['NREPI'] = intval( $nrepi );
                            parent::save( $request, $response );
                        }
                    }
                }
            } else {
                parent::save( $request, $response );
            }*/
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setCadastroAutomaticoHistoricoEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_historicoepi";
            $rows = array();
            
            foreach ($request->getDataSet()->getRows() as $row) {
                $currentRow = $row;
                
                // Auditoria e __is_new
                $currentRow['__is_new'] = true;
                $currentRow['DTINCLUSAO'] = $this->hcmParameters->getDataAtual();
                $currentRow['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                $currentRow['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                $currentRow['DTULTATU'] = $this->hcmParameters->getDataAtual();
                $currentRow['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                $currentRow['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();

                // Evita a modificação do dado na row
                $nrvinculos = $row['NRVINCULOM'];
                $nmvinculos = $row['NMVINCULOM'];
                $nrEPIs = isset($row['NREPI']) ? $row['NREPI'] : null;
                $nmEPIs = isset($row['NMEPI']) ? $row['NMEPI'] : null;
                $qntFornecida = isset($row['QTFORNECIDAEPI']) ? $row['QTFORNECIDAEPI'] : null;
                $nrCertAprov = isset($row['NRCTRLAPROVEPI']) ? $row['NRCTRLAPROVEPI'] : null;
                $dsModeloEpi = isset($row['DSMODELOEPI']) ? $row['DSMODELOEPI'] : null;
                $fornecedor = isset($row['NRPARCNEGOCIO']) ? $row['NRPARCNEGOCIO'] : null;
                $dtValidadeEpi = isset($row['DTVALIDADEEPI']) ? $row['DTVALIDADEEPI'] : null;
                $perfilEpiList  = is_array($row['NRPERFILEPI']) && count($row['NRPERFILEPI']) > 0 ? $row['NRPERFILEPI'] : null; 
                foreach($nrvinculos as $key => $nrvinculom) {
                    if(!is_null($perfilEpiList)) {
                        foreach ($perfilEpiList as $perfilEpi) {
                            $epiList = $this->perfilepiRepository->buscaEpi($currentRow['NRORG'], $perfilEpi);
                            foreach ($epiList as $epi) {
                                $currentRow['NRVINCULOM'        ] = $nrvinculom;
                                $currentRow['NMVINCULOM'        ] = $nmvinculos[$key];
                                $currentRow['NREPI'             ] = $epi['NREPI'];
                                $currentRow['NMEPI'             ] = $epi['NMEPI'];
                                $currentRow['NRCTRLAPROVEPI'    ] = $epi['NRCTRLAPROVEPI'];
                                $currentRow['DSMODELOEPI'       ] = $epi['DSMODELOEPI'];
                                $currentRow['NRPARCNEGOCIO'     ] = $epi['NRPARCNEGOCIO'];
                                $currentRow['DTVALIDADEEPI'     ] = $epi['DTVALIDADEEPI'];
                                $currentRow['NRHISTORICOEPI'    ] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                                $currentRow['NRORG'             ] = intval($row['NRORG']);
                                $currentRow['NRVINCULOM'        ] = intval($currentRow['NRVINCULOM']);
                                $currentRow['IDFICHAEPIASSINADA'] = 'N';
                                $this->historicoEpiService->validaCamposVazios($currentRow);
                                
                                $row = $currentRow;
        
                                parent::save($request, $response);
                            }
                        }
                    } else {
                        foreach($nrEPIs as $keyEPI => $epi) {
                            $nrCertAprovValue = isset($nrCertAprov[$keyEPI]) ? $nrCertAprov[$keyEPI] : null;
                            $dsModeloEpiValue = isset($dsModeloEpi[$keyEPI]) ? $dsModeloEpi[$keyEPI] : null;
                            $dtValidadeEpiValue = isset($dtValidadeEpi[$keyEPI]) ? $dtValidadeEpi[$keyEPI] : null;
                            $currentRow['NRVINCULOM'] = $nrvinculom;
                            $currentRow['NMVINCULOM'] = $nmvinculos[$key];
                            $currentRow['NREPI'] = $epi;
                            $currentRow['NMEPI'] = $nmEPIs[$keyEPI];
                            $currentRow['NRCTRLAPROVEPI'] = $nrCertAprovValue;
                            $currentRow['DSMODELOEPI'] = $dsModeloEpiValue; 
                            $currentRow['NRPARCNEGOCIO'] = $fornecedor[$keyEPI];
                            $currentRow['DTVALIDADEEPI'] =  $dtValidadeEpiValue;
                            $currentRow['QTFORNECIDAEPI'] = $qntFornecida[$keyEPI];
                            $currentRow['NRHISTORICOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                            $currentRow['NRORG']      = intval($row['NRORG']);
                            $currentRow['NRVINCULOM'] = intval($currentRow['NRVINCULOM']);
                            $currentRow['IDFICHAEPIASSINADA'] = 'N';
                            $this->historicoEpiService->validaCamposVazios($currentRow);
                            
                            $row = $currentRow;
    
                            parent::save($request, $response);
                        }
                    }   
                }
            }

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setAssinarAutomaticamente(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $row['NRORG']        = $this->hcmParameters->getNrorgTrabalho();
        $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();

        $this->historicoEpiService->setAssinarAutomaticamenteService( $row );
    }

    public function deleteHistoricoEpi(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function setInlineHistoricoEpi(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            error_reporting(E_ERROR | E_PARSE);
            foreach ($request->getDataSet()->getRows() as &$rows) {
                
                $this->historicoEpiService->setInlineHistoricoEpi($rows);
                
                $message = new \Zeedhi\Framework\DTO\Response\Message('Registro salvo com sucesso', DTO\Response\Message::TYPE_MESSAGE);
                $response->addMessage($message);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function convertArrayToString( $condition, $toConvertInArray ) {
        if ( in_array( $condition['columnName'], $toConvertInArray ) ) {
            if ( is_array( $condition['value'] ) ) {
                $condition['value'] = implode( ',', $condition['value'] );
                $condition['operator'] = '=';
            }
        }
        return $condition;
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $condition = $this->convertArrayToString( $condition, $this->toConvertInArray );
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 