<?php
namespace HCM\SST\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class InstrumentosAvalRiscosRepository {

    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }

    public function setNovaMarca($row, $nomeMarca)
    {   
        $nrMarca = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'SST_MARCA', 1, 12, true);
        $params = array(
                "P_NRORG" => $row['NRORG'],
                "P_NRMARCA" => $nrMarca,
                "P_NMMARCA" => $nomeMarca,
                "P_IDATIVO" => $row['IDATIVO'],
                "P_DTINCLUSAO" => $row['DTINCLUSAO'],
                "P_NRORGINCLUSAO" => $row['NRORGINCLUSAO'],
                "P_CDOPERINCLUSAO" => $row['CDOPERINCLUSAO'],
                "P_NRORGULTATU" => $row['NRORGULTATU'],
                "P_CDOPERULTATU" => $row['CDOPERULTATU']
        );


        $sql = "INSERT INTO SST_MARCA (
           NRORG, 
           NRMARCA,
           NMMARCA,
           IDATIVO,
           DTINCLUSAO, 
           NRORGINCLUSAO, 
           CDOPERINCLUSAO, 
           NRORGULTATU, 
          CDOPERULTATU
        ) VALUES (
           :P_NRORG,
           :P_NRMARCA,
           :P_NMMARCA,
           :P_IDATIVO,
           :P_DTINCLUSAO,
           :P_NRORGINCLUSAO,
           :P_CDOPERINCLUSAO,
           :P_NRORGULTATU,
           :P_CDOPERULTATU
        )";

       
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);
        return  array($nrMarca, $nomeMarca);
    }

    public function existeMarca($row) 
    {
        try {
            $sql = $this->entityManager->getConnection()->prepare( 
                "SELECT *
                FROM SST_MARCA
                WHERE NMMARCA = :P_NMMARCA
                  AND NRORG = :P_NRORG"
                    );
                    
                    $sql->execute(
                        array(
                            'P_NRORG' => $row['NRORG'],
                            'P_NMMARCA' => $row['NMMARCA'],
                        )
                    );
                    
                    return $sql->fetch();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function existeItensFilhos($nrorg, $nrinstrumento)
    {   
        try {
            $sql = $this->entityManager->getConnection()->prepare(
                "SELECT MI.*, M.NMMARCA FROM SST_MARCAINSTRUMENTO MI JOIN SST_MARCA M ON M.NRMARCA = MI.NRMARCA AND M.NRORG = MI.NRORG WHERE(MI.NRORG = :P_NRORG OR MI.NRORG = :P_NRORG_PADRAO) AND MI.NRINSTRUMENTO = :P_NRINSTRUMENTO"
                );
                
                $sql->execute(
                    array(
                        'P_NRORG' => $nrorg,
                        "P_NRINSTRUMENTO" => $nrinstrumento,
                        "P_NRORG_PADRAO" => 0
                    )
                );
              
                return $sql->fetch();
        } catch (\Exception $e) {
            throw $e;
        }
    } 

    public function existeInstrumento($nrorg, $nminstrumento)
    {   
        try {
            $sql = $this->entityManager->getConnection()->prepare(
                "SELECT I.* FROM SST_INSTRUMENTO I WHERE(I.NRORG = :P_NRORG OR I.NRORG = :P_NRORG_PADRAO) AND I.NMINSTRUMENTO = :P_NMINSTRUMENTO"
                );
                
                $sql->execute(
                    array(
                        'P_NRORG' => $nrorg,
                        "P_NMINSTRUMENTO" => $nminstrumento,
                        "P_NRORG_PADRAO" => 0
                    )
                );
              
                return $sql->fetch();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function existeMarcaInstrumento($nrorg, $nrmarca, $nrinstrumento)
    {   
        try {
            $sql = $this->entityManager->getConnection()->prepare(
                "SELECT MI.* FROM SST_MARCAINSTRUMENTO MI WHERE(MI.NRORG = :P_NRORG OR MI.NRORG = :P_NRORG_PADRAO) AND MI.NRINSTRUMENTO = :P_NRINSTRUMENTO AND MI.NRMARCA = :P_NRMARCA"
                );
                
                $sql->execute(
                    array(
                        'P_NRORG' => $nrorg,
                        "P_NRINSTRUMENTO" => $nrinstrumento,
                        "P_NRORG_PADRAO" => 0,
                        "P_NRMARCA" => $nrmarca
                    )
                );
              
                return $sql->fetch();
        } catch (\Exception $e) {
            throw $e;
        }
    }   
} 

