<?php

namespace HCM\VAB\Controllers;

use HCM\Util\Messages;
use HCM\VAB\Services\CalculoBeneficioService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use Folha\Entities\FpaTipomoviment;
use HCM\Util\Repositories;
use HCM\Geral\Controllers\RelatoriosHcmController;
use HCM\Util\DateUtil;
use Teknisa\Model\Gestaopessoas\Repositories\FpaCalculoconsumer;

class CalculoBeneficioController extends Crud {

    protected $dataSourceName = 'vab_formcalcbenef';
    protected $dataSourceName_get = 'vab_formcalcbenef_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERINCLUSAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager,EntityManager $entityManager,  ParameterBag $parameterBag, HCMParameters $hcmParameters, CalculoBeneficioService $calculoBeneficioService, RelatoriosHcmController $relatoriosHcmController ){
        $this->entityManager           = $entityManager;
        $this->parameterBag            = $parameterBag;
        $this->hcmParameters           = $hcmParameters;
        $this->calculoBeneficioService = $calculoBeneficioService;
        $this->relatoriosHcmController = $relatoriosHcmController;
        
        parent::__construct($dataSourceManager);
    }

    public function getFormcalcbenef(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();

        foreach($rows as $key => &$row){
            if($row['DSORDEMCALCBENEF'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIOCALCBENEF']);
                $row['DSINICIOCALCBENEF'] = $row['NRVINCULOM_LIST'];
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = $row['LIST_NMVINCULOM'];
                $row['LIST_NMVINCULOM'] = explode(",", $row['LIST_NMVINCULOM']);
            }    
            
            if($row['DSORDEMCALCBENEF'] == 'LISTA_LOTE'){
                $row['NRLOTEVINC_LIST'] = explode(",", $row['DSINICIOCALCBENEF']);
                $row['DSINICIOCALCBENEF'] = $row['NRLOTEVINC_LIST'];
                foreach($row['NRLOTEVINC_LIST'] as $chave => $value)
                    $row['NRLOTEVINC_LIST'][$chave] = intval($value);
                
                $row['NMLOTEVINC'] = $row['LIST_NMLOTEVINC'];
                $row['LIST_NMLOTEVINC'] = explode(",", $row['LIST_NMLOTEVINC']);
            }
            
            if($row['DSORDEMCALCBENEF'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIOCALCBENEF']);
                $row['DSINICIOCALCBENEF'] = $row['LIST_NRESTRUTLEGAL'];
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['NMESTRUTLEGAL'] = $row['LIST_NMESTRUTLEGAL'];
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['LIST_NMESTRUTLEGAL']);
                
                if($row['DSFIMCALCBENEF']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIMCALCBENEF']);
                    $row['DSFIMCALCBENEF'] = $row['LIST_NRESTRUTGEREN'];
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    $row['NMESTRUTGEREN'] = $row['LIST_NMESTRUTGEREN'];
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['LIST_NMESTRUTGEREN']);
                }
                
                $row['DTINIADMISSAO'] = $row['DSINICIOCALCBENEF1'] ? $row['DSINICIOCALCBENEF1'] : null;
                $row['DTFIMADMISSAO'] = $row['DSFIMCALCBENEF1'] ? $row['DSFIMCALCBENEF1'] : null;
            }   
            
            if($row['DSORDEMCALCBENEF'] === "LISTA_NRESTRUTSIND"){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIOCALCBENEF']);
                $row['DSINICIOCALCBENEF'] = $row['LIST_NRESTRUTLEGAL'];
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['NMESTRUTLEGAL'] = $row['LIST_NMESTRUTLEGAL'];
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['LIST_NMESTRUTLEGAL']);
                
                if($row['DSFIMCALCBENEF']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIMCALCBENEF']);
                    $row['DSFIMCALCBENEF'] = $row['LIST_NRESTRUTSIND'];
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                    
                    $row['NMESTRUTSIND'] = $row['LIST_NMESTRUTSIND'];
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['LIST_NMESTRUTSIND']);
                }
                
                $row['DTINIADMISSAO'] = $row['DSINICIOCALCBENEF1'] ? $row['DSINICIOCALCBENEF1'] : null;
                $row['DTFIMADMISSAO'] = $row['DSFIMCALCBENEF1'] ? $row['DSFIMCALCBENEF1'] : null;
            }   
            
            if($row['DSORDEMCALCBENEF'] === "LISTA_BENEFICIO"){
                if($row['DSINICIOCALCBENEF']) {
                    $row['LIST_NRBENEFICIOM'] = explode(',',$row['DSINICIOCALCBENEF']);
                    $row['DSINICIOCALCBENEF'] = $row['LIST_NRBENEFICIOM'];
                    foreach($row['LIST_NRBENEFICIOM'] as $chave => $value)
                        $row['LIST_NRBENEFICIOM'][$chave] = intval($value);
                    
                    $row['DSBENEFICIO'] = $row['LIST_DSBENEFICIO'];
                    $row['LIST_DSBENEFICIO'] = explode(",", $row['LIST_DSBENEFICIO']);
                }
            }
            
            if($row['DSORDEMCALCBENEF'] == 'LISTA_TOMADOR'){
                $row['LIST_NRESTRUTURAMSUP'] = explode(",", $row['DSINICIOCALCBENEF']);
                $row['DSINICIOCALCBENEF'] = $row['LIST_NRESTRUTURAMSUP'];
                foreach($row['LIST_NRESTRUTURAMSUP'] as $chave => $value)
                    $row['LIST_NRESTRUTURAMSUP'][$chave] = intval($value);
                
                $row['NMESTRUTURAMSUP'] = $row['LIST_NMESTRUTURAMSUP'];
                $row['LIST_NMESTRUTURAMSUP'] = explode(",", $row['LIST_NMESTRUTURAMSUP']);
                
                if($row['DSFIMCALCBENEF']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIMCALCBENEF']);
                    $row['DSFIMCALCBENEF'] = $row['LIST_NRESTRUTGEREN'];
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    $row['NMESTRUTGEREN'] = $row['LIST_NMESTRUTGEREN'];
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['LIST_NMESTRUTGEREN']);
                }
                
                $row['DTINIADMISSAO'] = $row['DSINICIOCALCBENEF1'] ? $row['DSINICIOCALCBENEF1'] : null;
                $row['DTFIMADMISSAO'] = $row['DSFIMCALCBENEF1'] ? $row['DSFIMCALCBENEF1'] : null;
            }
                
        }
        $response->addDataSet(new DataSet($this->dataSourceName_get, $rows));
    }
    
    
    public function GeraRelatBeneficio(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $rows           = $request->getDataSet()->getRows();
            $nrorg          = $this->hcmParameters->getNrorgTrabalho();
            $cdoperador     = $this->hcmParameters->getCdoperador();
            $nrocorrecal    = $this->hcmParameters->getOcorrenciaCalculo();
            $nrtpmodalidcal = $this->hcmParameters->getNrModalidadeCalculo();
            $competencia    = $this->hcmParameters->getDtCompetencia();
            foreach($rows as $row) {
                $filtro = array(
                    'P_NRFORMCALCBENEF' => $row['NRFORMCALCBENEF'],
                    'P_INICIO'          => $row['DTINICALCBENEF'],
                    'P_FINAL'           => $row['DTFIMCALCBENEF'],
                    'P_NRORG'           => $nrorg,
                    'P_MODALIDCAL'      => $nrtpmodalidcal,
                    'P_OCORRECAL'       => $nrocorrecal,
                    'P_COMPETENCIA'     => $competencia,
                    
                );
               $caminhoReport[] =  $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'demonstrativo_beneficio', $request, $response);
               $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('report', $caminhoReport));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setFormcalcbenef(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['DSINICIOCALCBENEF1'] = null;
                $row['DSFIMCALCBENEF1'] = null;
                
                if($row['IDCALCCREDINDEV'] && $row['IDCALCCREDINDEV'] == 'S'){
                    $dtiniciocredindev = DateUtil::getDataDeString($row['DTINICIOCREDINDEV'], DateUtil::FORMATO_BRASILEIRO, true);
                    $dtfinalcredindev = DateUtil::getDataDeString($row['DTFINALCREDINDEV'], DateUtil::FORMATO_BRASILEIRO, true);
                    
                    $dtinicio = DateUtil::getDataDeString($row['DTINICALCBENEF'], DateUtil::FORMATO_BRASILEIRO, true);
                    $dtfinal = DateUtil::getDataDeString($row['DTFIMCALCBENEF'], DateUtil::FORMATO_BRASILEIRO, true);
                    
                    $primeiroDiaMes = DateUtil::getPrimeiroDiaDoMes($dtinicio);
                    $ultimoDiaMes = DateUtil::getUltimoDiaDoMes($dtfinal);
                    
                    $primeiroDiaMesCredIndev = DateUtil::getPrimeiroDiaDoMes($dtiniciocredindev);
                    $ultimoDiaMesCredIndev = DateUtil::getUltimoDiaDoMes($dtiniciocredindev);
                    
                    $mesCredIndevFinal = $dtfinalcredindev->format('m');
                    $mesCredIndevInicio = $primeiroDiaMesCredIndev->format('m');
                    $anoCredIndevFinal = $dtfinalcredindev->format('Y');
                    $anoCredIndevInicio = $primeiroDiaMesCredIndev->format('Y');
                    
                    //Validação para analisar mes e ano, inicio e fim do credito indevido devem estar no mesmo periodo de mês
                    if($mesCredIndevFinal != $mesCredIndevInicio || $anoCredIndevFinal != $anoCredIndevInicio){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Início e fim dos campos de crédito indevido devem estar dentro do mesmo mês e ano informado nos campos.', DTO\Response\Message::TYPE_MESSAGE));
                        return;
                    }
                    
                    
                    if(($primeiroDiaMesCredIndev > $ultimoDiaMes) || ($primeiroDiaMesCredIndev == $primeiroDiaMes)){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('As datas Início e Fim do crédito indevido não podem ser maiores ou iguais que o período de cálculo do formulário do benefício (modifique para uma data retroativa).', DTO\Response\Message::TYPE_MESSAGE));
                        return;
                    }
                }
                
                if (!array_key_exists('NRTPBENEFICIO', $row)) {
                    $row['NRTPBENEFICIO'] = null;
                }
                if (!array_key_exists('NRESTRUTURAM', $row)) {
                    $row['NRESTRUTURAM'] = null;
                }
                if (array_key_exists('NRFORMCALCBENEF', $row)) {
                    if ($row['NRFORMCALCBENEF'] === '') {
                        $row['NRFORMCALCBENEF'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_FORMCALCBENEF', 1, 12, true);
                    }
                } else {
                    $row['NRFORMCALCBENEF'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_FORMCALCBENEF', 1, 12, true);
                }
                
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEMCALCBENEF'] === "LISTA_VINCULO"){
                    if(is_array($row['NRVINCULOM_LIST'])){
                        $row['DSINICIOCALCBENEF'] = implode(',',$row['NRVINCULOM_LIST']);
                    }
                }
                
                if($row['DSORDEMCALCBENEF'] === "LISTA_LOTE"){
                    if(is_array($row['NRLOTEVINC_LIST'])){
                        $row['DSINICIOCALCBENEF'] = implode(',',$row['NRLOTEVINC_LIST']);
                    }
                }
                
                if($row['DSORDEMCALCBENEF'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['LIST_NRESTRUTLEGAL'])){
                        $row['DSINICIOCALCBENEF'] = implode(',',$row['LIST_NRESTRUTLEGAL']);
                    }
                    if(isset($row['LIST_NRESTRUTGEREN']) && is_array($row['LIST_NRESTRUTGEREN']) && count($row['LIST_NRESTRUTGEREN']) > 0 && is_array($row['LIST_NRESTRUTGEREN'])){
                        $row['DSFIMCALCBENEF'] = implode(',',$row['LIST_NRESTRUTGEREN']);
                    }else{
                        if(is_array($row['DSFIMCALCBENEF'])){
                            $row['DSFIMCALCBENEF'] = null;
                        }
                    }
                    
                    $row['DSINICIOCALCBENEF1'] = $row['DTINIADMISSAO'] ? $row['DTINIADMISSAO'] : null;
                    $row['DSFIMCALCBENEF1'] = $row['DTFIMADMISSAO'] ? $row['DTFIMADMISSAO'] : null;
                    
                }
                
                if($row['DSORDEMCALCBENEF'] === "LISTA_NRESTRUTSIND"){
                    if(is_array($row['LIST_NRESTRUTLEGAL'])){
                        $row['DSINICIOCALCBENEF'] = implode(',',$row['LIST_NRESTRUTLEGAL']);
                    }
                    if(count($row['LIST_NRESTRUTSIND']) > 0 && is_array($row['LIST_NRESTRUTSIND'])){
                        $row['DSFIMCALCBENEF'] = implode(',',$row['LIST_NRESTRUTSIND']);
                    }else{
                        if(is_array($row['LIST_NRESTRUTSIND'])){
                            $row['DSFIMCALCBENEF'] = null;
                        }
                    }
                    
                    $row['DSINICIOCALCBENEF1'] = $row['DTINIADMISSAO'] ? $row['DTINIADMISSAO'] : null;
                    $row['DSFIMCALCBENEF1'] = $row['DTFIMADMISSAO'] ? $row['DTFIMADMISSAO'] : null;
                }
                
                if($row['DSORDEMCALCBENEF'] === "LISTA_BENEFICIO"){
                    if(count($row['LIST_NRBENEFICIOM']) > 0 && is_array($row['LIST_NRBENEFICIOM'])){
                        $row['DSINICIOCALCBENEF'] = implode(',',$row['LIST_NRBENEFICIOM']);
                    }else{
                        if(is_array($row['LIST_NRBENEFICIOM'])){
                            $row['DSINICIOCALCBENEF'] = null;
                        }
                    }
                }
                
                if($row['DSORDEMCALCBENEF'] === "LISTA_TOMADOR"){
                    if(is_array($row['LIST_NRESTRUTURAMSUP'])){
                        $row['DSINICIOCALCBENEF'] = implode(',',$row['LIST_NRESTRUTURAMSUP']);
                    }
                    if(isset($row['LIST_NRESTRUTGEREN']) && is_array($row['LIST_NRESTRUTGEREN']) && count($row['LIST_NRESTRUTGEREN']) > 0 && is_array($row['LIST_NRESTRUTGEREN'])){
                        $row['DSFIMCALCBENEF'] = implode(',',$row['LIST_NRESTRUTGEREN']);
                    }else{
                        if(is_array($row['DSFIMCALCBENEF'])){
                            $row['DSFIMCALCBENEF'] = null;
                        }
                    }
                    
                    $row['DSINICIOCALCBENEF1'] = $row['DTINIADMISSAO'] ? $row['DTINIADMISSAO'] : null;
                    $row['DSFIMCALCBENEF1'] = $row['DTFIMADMISSAO'] ? $row['DTFIMADMISSAO'] : null;
                    
                }
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*
    * Em resumo, esta função calcula a quantidade de vínculos com base em $qntVinculos, dando prioridade a um valor baseado em uma fração do número de vínculos, 
    * mas limitando esse valor a 1 se for menor que 1 ou a uma divisão inteira se for maior que um limite definido ($maxRequisicoes).
    */
    public function calculaQntVinculosArray($qntVinculos){
        $maxRequisicoes = 90;
        
        if($qntVinculos * 0.1 > $maxRequisicoes){
            return intval($qntVinculos / $maxRequisicoes);
        }
        
        $qntDivisao = ($qntVinculos * 0.08) < 1 ? 1 : ($qntVinculos * 0.08);
        return $qntDivisao;
    }
    
    public function buscarVinculosBeneficio(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            error_reporting(E_ERROR | E_PARSE);
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $foiCalculado = false;
                $dsordem = $row['DSORDEMCALCBENEF'];
                $dsinicio = $row['DSINICIOCALCBENEF'];
                $dsfim = $row['DSFIMCALCBENEF'];
                $dsinicio1 = $row['DSINICIOCALCBENEF1'];
                $dsfim1 = $row['DSFIMCALCBENEF1'];
                $dtinicio = isset($row['DTINICALCBENEF']) && !empty($row['DTINICALCBENEF']) ? substr($row['DTINICALCBENEF'], 0, 10) : null;
                $dtfinal = isset($row['DTFIMCALCBENEF']) && !empty($row['DTFIMCALCBENEF']) ? substr($row['DTFIMCALCBENEF'], 0, 10) : null;
                $dtpagamento = substr($row['DTPGTOCALCBENEF'], 0, 10);
                $nrtpbeneficio = $row['NRTPBENEFICIO'];
                $nrestruturam = $row['NRESTRUTURAM'];
                $tpperiodo = $row['TPPERIODOBENEF'];
                $dtmescompetenc = substr($this->hcmParameters->getDtCompetencia(), 0, 10);
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrcalculobenef = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
                $qtdiasbenef = $row['QTDIASBENEF'];
                $cdoperador = $this->hcmParameters->getCdoperador();
                $ocorrenciaCalc = $this->hcmParameters->getOcorrenciaCalculo();
                $route = 'calcularBeneficio';
                $row['DTCOMPETENC'] = $dtmescompetenc;
                $row['OCORRCALC'] = $ocorrenciaCalc;
                $row['CDOPERADOR'] = $cdoperador;
                $row['TIPOESTRUTURALEGALORG'] = $this->hcmParameters->getTipoEstruturaLegalOrg();
                
                if(!$row['NAORETORNAVINCULOS']){
                    $vinculos = $this->calculoBeneficioService->buscarVinculosBeneficio($tpperiodo, $dsordem, $dsinicio, $dsfim, $dtinicio, $dtfinal, $dtpagamento, $nrtpbeneficio, $nrestruturam, $dtmescompetenc, $nrorg, $row, $response, $qtdiasbenef, $dsinicio1, $dsfim1);
                    
                    $arrayNrVinculoVazio = [];
                    foreach($vinculos as $vinculo){
                        array_push($arrayNrVinculoVazio, $vinculo->getNrvinculom());
                    }
                    
                    $divisorDeRequisicoes = $this->calculaQntVinculosArray(count($arrayNrVinculoVazio));
                    
                    if(count($arrayNrVinculoVazio) < $divisorDeRequisicoes){
                        $divisorDeRequisicoes = count($arrayNrVinculoVazio);
                    }
                    
                    $array_divididos = array_chunk($arrayNrVinculoVazio, $divisorDeRequisicoes);
                }
                
                $contador = 0;
                $qntReq = count($array_divididos);
                $row['QNTTOTAL'] = count($vinculos);
                
                if(!empty($array_divididos)){
                    //Insere calculo Consumer.
                    \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();
                    $nrcalculoconsumer = $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->insereCalculoConsumer($nrorg, count($vinculos), $this->hcmParameters, $dsordem, implode(',',$dsinicio), implode(',',$dsfim));
                    $calcConsumer = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_CALCULOCONSUMER);
                    
                    // retorna registro do calculoconsumer da fila e seta a data hora de inicio
                    $fpaCalculoConsumer = $calcConsumer->findOneBy(array('nrcalculoconsumer' => $nrcalculoconsumer, 'nrorg' => $nrorg));
                    
                    if ($fpaCalculoConsumer->getIdsituacao() == FpaCalculoconsumer::IDSITUACAO_PENDENTE)
                        $fpaCalculoConsumer->setIdsituacao(FpaCalculoconsumer::IDSITUACAO_ANDAMENTO);
                    if (is_null($fpaCalculoConsumer->getDthrinicio()))
                        $fpaCalculoConsumer->setDthrinicio(DateUtil::getDataAtual());
                    
                    $fpaCalculoConsumer->setNrtotprocessvinc();
                    
                    $row['NRCALCULOCONSUMER'] = $nrcalculoconsumer;
                    $row['NRCALCULOBENEF'] = $nrcalculobenef;
                    
                    $fpaCalculoConsumer->setNrcalculofolha($nrcalculobenef);
                    //$fpaCalculoConsumer->setIderro(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA));
                    $fpaCalculoConsumer->setDscalculoparams('Cálculo do tipo 21 - Calculo de Beneficio realizado pela rotina paralela do HCMSERVICES ');
                    
                    \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                    \Setforms2\Shared\Registry::getEntityManager()->commit();
                    
                    foreach($array_divididos as $array_dividido){
                        $row['VINCULOS'] = $array_dividido;
                        $contador++;
                         
                        $params = [
                            'requestType' => 'Row',
                            'row' => $row,
                        ];
                        
                        #var_dump(json_encode($params));die;
                        try {
                            $this->calculoBeneficioService->enviaCalculoParaHcmServices($route, $params, $nrorg, 'POST');
                        } catch(\Exception $e) {
                            
                        }
                    }
                    
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vinculosDivididos', $array_divididos));
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
        
    }

    public function calcular(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            error_reporting(E_ERROR | E_PARSE);
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $foiCalculado = false;
                $dsordem = $row['DSORDEMCALCBENEF'];
                $dsinicio = $row['DSINICIOCALCBENEF'];
                $dsfim = $row['DSFIMCALCBENEF'];
                $dsinicio1 = $row['DSINICIOCALCBENEF1'];
                $dsfim1 = $row['DSFIMCALCBENEF1'];
                $dtinicio = isset($row['DTINICALCBENEF']) && !empty($row['DTINICALCBENEF']) ? substr($row['DTINICALCBENEF'], 0, 10) : null;
                $dtfinal = isset($row['DTFIMCALCBENEF']) && !empty($row['DTFIMCALCBENEF']) ? substr($row['DTFIMCALCBENEF'], 0, 10) : null;
                $dtpagamento = substr($row['DTPGTOCALCBENEF'], 0, 10);
                $nrtpbeneficio = $row['NRTPBENEFICIO'];
                $nrestruturam = $row['NRESTRUTURAM'];
                $tpperiodo = $row['TPPERIODOBENEF'];
                $dtmescompetenc = substr($this->hcmParameters->getDtCompetencia(), 0, 10);
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrcalculobenef = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
                $qtdiasbenef = $row['QTDIASBENEF'];

                $foiCalculado = $this->calculoBeneficioService->calcular($tpperiodo, $dsordem, $dsinicio, $dsfim, $dtinicio, $dtfinal, $dtpagamento, $nrtpbeneficio, $nrestruturam, $dtmescompetenc, $nrorg, $row, $response, $qtdiasbenef, $dsinicio1, $dsfim1);
                
                /*$message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->getMsgFiltrada(
                     $nrcalculobenef, 
                     'CALCULOBENEFICIO', 
                     $this->hcmParameters->getNrorgTrabalho(),
                     $dtmescompetenc,
                     $dsordem,
                     $dsinicio,
                     $dsfim
                );*/
                
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(
                    [
                        'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                        'dsorigmsg'      => 'CALCULOBENEFICIO',
                        'nrcalculofolha' => $nrcalculobenef, 
                        'cdoperinclusao' => $this->hcmParameters->getCdoperador(),
                    ]
                );

                if(count($message) > 0){
                    $row = array();
                    
                    foreach ($message as $mensagem) {
                        $row[] = [
                                  "NRVINCULOM"  => $mensagem->getNrvinculom(), 
                                  "NRMSGCALC"   => $mensagem->getNrmsgcalc(), 
                                  "DSDESCRICAO" => $mensagem->getDescricao(), 
                                  "IDTIPOMSG"   => $mensagem->getIdtipomsg() === 'A' ? 'Alerta' : 'Erro',
                                  "DTINCLUSAO"  => $mensagem->getDtinclusao()->format('d/m/Y'),
                                ];
                    }
                    
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('calculoBeneficio', $row));                    
                }
                
                if($foiCalculado)
                    $response->addNotification(new DTO\Response\Notification('calcular'), DTO\Response\Notification::TYPE_SUCCESS);
                else
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02015_VAB_NENHUM_BENEFICIO_CALCULADO, DTO\Response\Message::TYPE_ERROR));
                

                
                /*if($tpperiodo === 'DIARIO')
                    $foiCalculado = $this->calculoBeneficioService->calcular_Diario($dsordem, $dsinicio, $dsfim, $dtinicio, $dtfinal, $dtpagamento, $nrtpbeneficio, $nrestruturam, $dtmescompetenc, $nrorg);
                else{
                    $foiCalculado = $this->calculoBeneficioService->calcular_Mensal($dsordem, $dsinicio, $dsfim, $dtpagamento, $nrtpbeneficio, $nrestruturam, $dtmescompetenc, $nrorg);
                    
                    $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->getMsgFiltrada(
                         $nrcalculobenef, 
                         'CALCULOBENEFICIO', 
                         $this->hcmParameters->getNrorgTrabalho(),
                         $dtmescompetenc,
                         $dsordem,
                         $dsinicio,
                         $dsfim
                    );
                    
                    $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('dsorigmsg' => 'CALCULOBENEFICIO','nrcalculofolha' => $nrcalculobenef, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));

                    if(count($message) > 0){
                        $row = array();
                        foreach ($message as $mensagem) {
                            $row[] = [
                                      "NRVINCULOM"  => $mensagem->getNrvinculom(), 
                                      "NRMSGCALC"   => $mensagem->getNrmsgcalc(), 
                                      "DSDESCRICAO" => $mensagem->getDescricao(), 
                                      "IDTIPOMSG"   => $mensagem->getIdtipomsg() === 'A' ? 'Alerta' : 'Erro',
                                      "DTINCLUSAO"  => $mensagem->getDtinclusao()->format('d/m/Y'),
                                    ];
                        }
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('calculoBeneficioMensal', $row));                    
                    }
                }    
                if($foiCalculado)
                    $response->addNotification(new DTO\Response\Notification('calcular'), DTO\Response\Notification::TYPE_SUCCESS);
                else
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02015_VAB_NENHUM_BENEFICIO_CALCULADO, DTO\Response\Message::TYPE_ERROR));
                */    
                // throw new \Exception(Messages::HCM02015_VAB_NENHUM_BENEFICIO_CALCULADO, Messages::ERROR_MESSAGE_CODE);    
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function retroceder(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            foreach ($request->getDataSet()->getRows() as &$row) {
                $foiRetrocedido = false;
                $dsinicio1 = isset($row['DSINICIOCALCBENEF1']) ? $row['DSINICIOCALCBENEF1'] : null;
                $dsfim1 = isset($row['DSFIMCALCBENEF1']) ? $row['DSFIMCALCBENEF1'] : null;
                $dsordem = $row['DSORDEMCALCBENEF'];
                $dsinicio = $row['DSINICIOCALCBENEF'];
                $dsfim = $row['DSFIMCALCBENEF'];
                $tpperiodo = $row['TPPERIODOBENEF'];
                $nrestruturam = isset($row['NRESTRUTURAM']) ? $row['NRESTRUTURAM'] : null;
                $nrtpbeneficio = isset($row['NRTPBENEFICIO']) ? $row['NRTPBENEFICIO'] : null;
                $dtmescompetenc = substr($this->hcmParameters->getDtCompetencia(), 0, 10);
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrcalculobenef = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
                
                // Por ser retrocesso deve-se gravar o log da operação
                $this->calculoBeneficioService->persistirLogRetrocesso($row);
                
                $foiRetrocedido = $this->calculoBeneficioService->retroceder($tpperiodo, $dsordem, $dsinicio, $dsfim, $dtmescompetenc, $nrorg, $nrestruturam, $nrtpbeneficio, $row, $dsinicio1, $dsfim1);
                
                if(is_array($foiRetrocedido) && array_key_exists('CONFIRMACAO', $foiRetrocedido)){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('O retrocesso será realizado para  '.strval($foiRetrocedido['QTDVINCULO']).' vínculos. Deseja continuar?', 'C'));
                    return;
                }    
                
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(
                    [
                        'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                        'dsorigmsg'      => 'RETROCESSOBENEFICIO',
                        'nrcalculofolha' => $nrcalculobenef, 
                        'cdoperinclusao' => $this->hcmParameters->getCdoperador(),
                    ]
                );

                if(count($message) > 0){
                    $row = array();
                    foreach ($message as $mensagem) {
                        $row[] = [
                                  "NRVINCULOM"  => $mensagem->getNrvinculom(), 
                                  "NRMSGCALC"   => $mensagem->getNrmsgcalc(), 
                                  "DSDESCRICAO" => $mensagem->getDescricao(), 
                                  "IDTIPOMSG"   => $mensagem->getIdtipomsg() === 'A' ? 'Alerta' : 'Erro',
                                  "DTINCLUSAO"  => $mensagem->getDtinclusao()->format('d/m/Y'),
                                ];
                    }
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retrocessoCalculo', $row));                    
                }                    

                if($foiRetrocedido)
                    $response->addNotification(new DTO\Response\Notification('retroceder'), DTO\Response\Notification::TYPE_SUCCESS);
                else
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02015_VAB_NENHUM_CALCULO_RETROCEDIDO, DTO\Response\Message::TYPE_ERROR));


                /* ===antigamente===if($tpperiodo === 'DIARIO')
                    $foiRetrocedido = $this->calculoBeneficioService->retroceder_Diario($dsordem, $dsinicio, $dsfim, $dtmescompetenc, $nrorg);
                else{
                    $foiRetrocedido = $this->calculoBeneficioService->retroceder_Mensal($dsordem, $dsinicio, $dsfim, $dtmescompetenc, $nrorg, $nrestruturam, $nrtpbeneficio);

                    $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('dsorigmsg' => 'CALCULOBENEFICIO','nrcalculofolha' => $nrcalculobenef, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));

                    if(count($message) > 0){
                        $row = array();
                        foreach ($message as $mensagem) {
                            $row[] = [
                                      "NRVINCULOM"  => $mensagem->getNrvinculom(), 
                                      "NRMSGCALC"   => $mensagem->getNrmsgcalc(), 
                                      "DSDESCRICAO" => $mensagem->getDescricao(), 
                                      "IDTIPOMSG"   => $mensagem->getIdtipomsg() === 'A' ? 'Alerta' : 'Erro',
                                      "DTINCLUSAO"  => $mensagem->getDtinclusao()->format('d/m/Y'),
                                    ];
                        }
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retrocessoCalculoMensal', $row));                    
                    }                    
                }    
                
                if($foiRetrocedido)
                    $response->addNotification(new DTO\Response\Notification('retroceder'), DTO\Response\Notification::TYPE_SUCCESS);
                else
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02015_VAB_NENHUM_CALCULO_RETROCEDIDO, DTO\Response\Message::TYPE_ERROR));
                */


            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}