<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 01/03/2016
 * Time: 11:14
 */

namespace HCM\VAB\Services;

use HCM\Util\HCMParameters;
use Folha\Logic\ValesBeneficios\CalculoValeTransporte\Calculo;
use Folha\Entities\FpaTipomoviment;
use HCM\FPA\Controllers\LogRetrocessoCalculoController;
use HCM\VAB\Repository\CalculoValeTransporteRepository;
use HCM\Geral\Services\HcmServices;

class CalculoValeTransporteService {

    protected $hcmParameters;
    protected $hcmServices;
    protected $logRetrocessoCalculoController;
    protected $calculoValeTransporteRepository;

    public function __construct(HCMParameters $hcmParameters, LogRetrocessoCalculoController $logRetrocessoCalculoController, CalculoValeTransporteRepository $calculoValeTransporteRepository, HcmServices $hcmServices) {
        $this->hcmParameters = $hcmParameters;
        $this->logRetrocessoCalculoController = $logRetrocessoCalculoController;
        $this->calculoValeTransporteRepository = $calculoValeTransporteRepository;
        $this->hcmServices   = $hcmServices;
    }
    
    public function buscaVinculos_Mensal(&$response, $idMemoria, $dsordem, $dsiniciovale, $dsfimvale, $dtiniciovale, $dtfinalvale, $dtpagamento, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row) {
        try {
            $vinculos = $this->calculoValeTransporteRepository->buscaVinculos_Mensal($dsiniciovale, $dsfimvale, $dtmescompetenc, $dsordem, $dtiniciovale, $dtfinalvale, $row);
            return $vinculos;
        } catch (\Exception $e) {
            throw $e; 
        }
    }
    
    public function enviaCalculoParaHcmServices($route, $params, $nrOrg, $post){
        try {
            $result = $this->hcmServices->sendAsync($route, $params, $nrOrg, $post, false);
        } catch (\Exception $e) {
            throw $e; 
        }
    }

    public function calcular(&$response, $idMemoria, $dsordem, $dsiniciovale, $dsfimvale, $dtiniciovale, $dtfinalvale, $dtpagamento, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row) {
        try {
            $info = new Calculo($response, $idMemoria, $dsiniciovale, $dsfimvale, $dtiniciovale, $dtfinalvale, $dtpagamento, $nrorg, $dtmescompetenc, $dsordem, $nrtipolinhavinc, $this->hcmParameters, $row);
            $info->calculo();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaQtdRetrocessoVale($dsordem, $dsiniciovale, $dsfimvale, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            return $retrocesso = Calculo::verificaQtdRetrocesso($nrcalculofolha, $dtmescompetenc, $nrorg, $dsiniciovale, $dsfimvale, $dsordem, $nrtipolinhavinc, $row);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retroceder($dsordem, $dsiniciovale, $dsfimvale, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            Calculo::retroceder($nrcalculofolha, $dtmescompetenc, $nrorg, $dsiniciovale, $dsfimvale, $dsordem, $nrtipolinhavinc, $row);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function persistirLogRetrocesso($row){
        $row['DSORDEM'] = $row['DSORDEMVALE'];
        $row['DSINICIO'] = $row['DSINICIOVALE'];
        $row['DSFIM'] = $row['DSFIMVALE'];
        $this->logRetrocessoCalculoController->persistirLogRetrocesso('CALC_VT', $row);
    }

}