// FILE: assets/js/util/ZhuAclConfig.js
/* global Configuration */
function ZhuAclConfig() {
	var self = this;
	this.RULE_ALLOW = 'A';
	this.RULE_READONLY = 'R';
	this.RULE_INVISIBLE = 'I';

	this.exportRulesModal = 'exportRulesModal';
	this.menuElementsListWidgetName = 'elementsListWidget';
	this.containerElementsListWidgetName = 'containerElementsModal';
	this.containerElementsTreeWidgetName = 'containerElementsTree';

	this.containerType = 'container';

	this.getDefaultDataSourceFilter = function(groupIds, projectId, organizationId){
		var dataFilter = [];
		var groupFilter = {
			"name": "GROUP_ID",
			"operator": "=",
			"value": groupIds
		};
		var projectFilter = {
			"name": "PROJECT_ID",
			"operator": "=",
			"value": projectId
		};

		if(organizationId !== undefined){
			var organizationFilter = {
				"name": "ORGANIZATION_ID",
				"operator": "=",
				"value": organizationId
			};
			dataFilter.push(organizationFilter);
		}

		dataFilter.push(groupFilter);
		dataFilter.push(projectFilter);
		return dataFilter;
	};

	this.getAclMap = function(type){
		var aclMap = {
			"container": {
				"aclTypeCod": "C",
				"childs": ["widgets"]
			},
			"widgets": {
				"aclTypeCod": "W",
				"childs": ["widgets", "fields", "actions"]
			},
			"fields": {
				"aclTypeCod": "F",
				"childs": []
			},
			"actions": {
				"aclTypeCod": "A",
				"childs": []
			}
		};
		return aclMap[type] || {};
	};

	this.getRuleType = function(isVisible, isEditable){
		var ruleType;
		if(!isVisible){
			ruleType = self.RULE_INVISIBLE;
		} else if(!isEditable){
			ruleType = self.RULE_READONLY;
		} else {
			ruleType = self.RULE_ALLOW;
		}
		return ruleType;
	};

	this.getRuleConfiguration = function(ruleType){
		var ruleConfig = {
			isVisible: ruleType != self.RULE_INVISIBLE,
			isEditable: ruleType == self.RULE_ALLOW
		};
		return ruleConfig;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ZhuAclConfig', ZhuAclConfig);
});

// FILE: assets/js/util/ZhuAclMessages.js
var ZhuAclMessages = {
	CANCEL: 'Cancelar',
	YES: 'Sim',
	NO: 'Não',
	VISIBLE: 'visíveis',
	EDITABLE: 'editáveis',
	INVISIBLE: 'invisíveis'
};
(function(){
	var mappedMessages = [
		"ACCESS_DENIED",
		"RULE_NOT_SAVED",
		"SUC_RULE_INSERTED",
		"ERR_RULE_INSERTED",
		"SUC_RULES_EXPORTED",
		"ERR_RULES_EXPORTED",
		"CANNOT_LOAD_RULES",
		"CONFIRMATION_MESSAGE",
		"CONFIRM_CHILDS_QUESTION",
		"CONFIRM_EXPORT",
		"NSHOW_WITHOUT_ID",
		"NSHOW_WITHOUT_CONTAINER_CONFIG",
		"NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG"
	];
	mappedMessages.forEach(function addMappedMessages(code){
		ZhuAclMessages[code] = 'zhuacl#' + code;
	});
})();

// FILE: assets/js/services/ZhuAccessButtonService.js
/* global _Notification, Util, ZeedhiServices */
function ZhuAccessButtonService(){
	this.insertButtonInProject = function(project){
		var notification = {
			"label": "Access Control",
			"name": "accessControl",
			"icon": "block",
			"isVisible": false,
			"tooltip": "Access Control",
			"isAclNotification": true,
			"events": [
				{
					"name": "NotificationEvent",
					"code": "ZhuAclController.openAclModalWidget();"
				}
			]
		};
		try {
			notification = Util.extend(new _Notification(), notification);
			project.notifications = project.notifications || [];
			project.notifications.push(notification);
		} catch (e){
			console.error('O Controle de acesso à elementos só funciona em versões acima da 1.40.x do Zeedhi Framework');
		}
	};

	this.disableButtonInProject = function(project){
		if(project.notifications){
			project.notifications = project.notifications.filter(function(notification){
				return !notification.isAclNotification;
			});
		}
	};

	this.enableButtonInProject = function(project){
		if(project.notifications){
			project.notifications.forEach(function(notification){
				if(notification.isAclNotification){
					notification.isVisible = true;
				}
			});
		}
	};
}

ZeedhiServices.service("ZhuAccessButtonService", function(ApplicationContext){
	var instance = new ZhuAccessButtonService();
	ApplicationContext.registerInstance("ZhuAccessButtonService", instance);
	return instance;
});


// FILE: assets/js/services/ZhuAclService.js
/* global Util, Configuration, ZhuAclMessages*/
function ZhuAclService(requestDataSourceEngine, restEngine, ZhuAclConfig){

	this.getZhuAclRules = function(projectId, groupIds){
		return requestDataSourceEngine.getDataSource('zhuacl#/acl_zhu_acl_rules', ZhuAclConfig.getDefaultDataSourceFilter(groupIds, projectId), 1, undefined, undefined, undefined, 120000).then(function(response){
			var dataset = response.dataset || {};
			var rules = dataset.zhu_access_control_rule_login;
			if(Util.isArray(rules)){
				var objToReturn = {
					aclRules: rules,
					defaultRule: dataset.zhu_access_control_default_rule.defaultRule,
				};
				return objToReturn;
			} else {
				throw {code: ZhuAclMessages.CANNOT_LOAD_RULES};
			}
		});
	};

	this.getWidgetMetadata = function(widgetName){
		var widgetUrl = 'zhuacl#json/'+ widgetName +'.json';
		return restEngine.requestMetaData(widgetUrl).then(function(metadata){
			return metadata.data;
		});
	};

	this.exportRules = function(profileFrom, profilesTo){
		var rows = [{
				changed: true,
				profileFrom: profileFrom,
				profilesTo: profilesTo
		}];
		return requestDataSourceEngine.setDataSource('zhuacl#/acl_zhu_acl_element_exportRules', rows);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclService', ZhuAclService);
});

// FILE: assets/js/controllers/ZhuAclController.js
/* global openWindow, Util, Configuration, ZhuAclMessages */
function ZhuAclController(ScreenService, templateManager, metaDataFactory, ZHPromise, ZhuAclService, ZhuAccessButtonService, ZhuAclConfig, MessageService, ApplicationContext, eventAggregator){
	var self = this;
	var user;
	var validProperties = ['widgets', 'fields', 'actions'];
	var loadedContainers = {};

	function User(userId, projectId, organizationId, defaultRule){
		var currentGroupId;
		var userDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;

		this.setCurrentGroupId = function(id){
			currentGroupId = id;
		};

		this.getDefaultRule = function(){
			return userDefaultRule;
		};

		this.getCurrentGroupId = function(){
			return currentGroupId;
		};

		this.getProjectId = function(){
			return projectId;
		};

		this.getUserId = function(){
			return userId;
		};

		this.getOrganizationId = function(){
			return organizationId;
		};
	}

	this.getDefaultRule = function(){
		return user.getDefaultRule();
	};

	this.getCurrentGroupId = function(){
		return user.getCurrentGroupId();
	};

	this.getProjectId = function(){
		return user.getProjectId();
	};

	var prepareLabelToName = function(label){
		var name = label.toLowerCase();
		name = name.replace(/\ /g, "_");
		return name.replace(/[^\w\s]/gi, '');
	};

	var translateLabel = function(label){
		return ScreenService.i18n(label);
	};

	this.getRulesFromDatasource = function(rulesData){
		var userId = user.getUserId();
		var rules = {};
		rulesData.forEach(function(row){
			var obj = {
				ruleRow: row,
				userGroups: {}
			};
			obj.userGroups[userId] = {
				rule: row.RULE
			};
			rules[row.ELEMENT_UNIQUE_ID] = obj;
		});
		return rules;
	};

	this.getElementRule = function(rules, itemId){
		var itemRule = {};
		if(rules[itemId]){
			itemRule = rules[itemId].ruleRow;
		}
		return itemRule;
	};

	this.getPlannedItemToInsertInDataSet = function(rules, item, itemType, parentId){
		var uniqueId = item.id;
		var itemRule = self.getElementRule(rules, uniqueId);
		var itemToInsert = {
			"UNIQUE_ID": uniqueId,
			"NAME": item.name || prepareLabelToName(item.label),
			"LABEL": translateLabel(item.label || item.name),
			"TYPE": itemType,
			"RULE": itemRule.RULE || this.getDefaultRule(),
			"RULE_ID": itemRule.ID,
			"PARENT_ID": parentId,
			"USER_ID": user.getUserId(),
			"PROJECT_ID": user.getProjectId(),
			"ORGANIZATION_ID": user.getOrganizationId()
		};

		itemToInsert.isVisible = item.aclIsVisible ? item.aclIsVisible : checkChildrenVisible(item, true);
		itemToInsert.isEditable = !item.aclReadOnly && item.aclIsVisible ? true : checkChildrenVisible(item);

		return itemToInsert;
	};

	var checkChildrenVisible = function(component, isVisibleHandle) {
		var control = false;
		var childrenTypes = ['widgets', 'fields', 'notifications'];
		if(!(component instanceof Container)){
			childrenTypes.push('actions');
		}
		childrenTypes.forEach(function(type){
			if(component[type]) {
				component[type].forEach(function(children){
					if (children && children.aclIsVisible && isVisibleHandle) {
						control = true;
					}
					if (children && children.aclIsVisible && !children.aclReadOnly) {
						control = true;
					}
				});
			}
		});
		return control;
	};

	var insertContainerElementsListWidget = function(container){
		return getFactoredWidget(container, ZhuAclConfig.containerElementsListWidgetName, true).then(function(factoredWidget){
			container.actions = container.actions || [];
			return factoredWidget;
		});
	};

	this.openAclModalWidget = function(){
		insertContainerElementsListWidget(templateManager.container).then(function(widget){
			ScreenService.openPopup(widget);
		});
	};

	var redirectToMainWindow = function(){
		var dashboardMenu = templateManager.project.getMenu('dashboard');
		var openLinkByMenu = ApplicationContext.LibUtilities && ApplicationContext.LibUtilities.openLinkByMenu;
		if(dashboardMenu && openLinkByMenu){
			openLinkByMenu(dashboardMenu);
		} else {
			openWindow(templateManager.project.mainWindow);
		}
	};

	var verifyContainerRule = function(rule){
		var containerIsRestrict = rule === ZhuAclConfig.RULE_INVISIBLE;
		if(containerIsRestrict){
			var actions = [{
				label: ScreenService.i18n("Ok"),
				code: redirectToMainWindow
			}];
			var dialogConfig = {
				type: 'ALERT',
				icon: 'exclamation',
				actions: actions
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.ACCESS_DENIED, {}, dialogConfig);
		}
		return !containerIsRestrict;
	};

	var mergeUndefinedRules = function(groupIds, itemRule, parentRules){
		parentRules = parentRules || {};

		var definedRule = itemRule.rule;
		var groupRules = parentRules.groupRules || {};

		if(!definedRule){
			definedRule = parentRules.rule || templateManager.project.aclDefaultRule;
		} else {
			var undefinedGroups = itemRule.undefinedGroups || [];
			groupIds.forEach(function(groupId){
				var parentRule = groupRules[groupId] || templateManager.project.aclDefaultRule;
				var isUndefinedGroup = undefinedGroups.indexOf(groupId) !== -1;
				var groupRule = isUndefinedGroup ? parentRule : definedRule;
				groupRules[groupId] = groupRule;

				if(definedRule !== ZhuAclConfig.RULE_ALLOW && groupRule !== ZhuAclConfig.RULE_INVISIBLE){
					definedRule = groupRule;
				}
			});
			itemRule.rule = definedRule;
		}

		return {rule: definedRule, groupRules: groupRules};
	};

	var getUserRule = function(rules, userId, elementId){
		return Util.getNestedProperty(rules, [elementId, 'userGroups', userId]);
	};

	var initObject = function(obj, objKeys, lastKeyValue){
		var objKeysLength = objKeys.length;
		objKeys.reduce(function(prev, next, index){
			if(index == objKeysLength - 1){
				prev[next] = lastKeyValue;
			} else {
				prev[next] = prev[next] || {};
			}
			return prev[next];
		}, obj);
	};

	var updateContainerAcl = function(rules, groupIds, userId){
		var applyRule = function(item, parentRules){
			var id = item.id;
			var userRule = getUserRule(rules, userId, id) || {};
			var mergedItem = mergeUndefinedRules(groupIds, userRule, parentRules);
			var itemRule = mergedItem.rule;

			item.aclIsVisible = itemRule != ZhuAclConfig.RULE_INVISIBLE;
			item.aclReadOnly = itemRule != ZhuAclConfig.RULE_ALLOW;

			return mergedItem;
		};

		var updateItemAcl = function(item, type, parentRules){
			var mergedItem = applyRule(item, parentRules);

			ZhuAclConfig.getAclMap(type).childs.forEach(function(type){
				if(Util.isArray(item[type])){
					item[type].forEach(function(child){
						updateItemAcl(child, type, mergedItem);
					});
				}
			});
		};

		var changeNotificationVisibility = function(isVisible){
			templateManager.project.notifications.some(function(notification){
				var isAclNotification = notification.name === "accessControl";
				if(isAclNotification){
					notification.isVisible = isVisible;
				}
				return isAclNotification;
			});
		};

		var updateUserRules = function(container){
			var containerId = container.id;
			var parentMenuId = container.parentMenuId;
			var userRule = getUserRule(rules, userId, containerId);
			if(!userRule && parentMenuId) {
				userRule = getUserRule(rules, userId, parentMenuId);
				if(userRule){
					initObject(rules, [containerId, 'userGroups', userId], userRule);
				}
			}
		};

		var container = templateManager.container;
		var aclEnable = container.aclEnable;
		aclEnable = aclEnable !== undefined ? aclEnable : true;
		changeNotificationVisibility(aclEnable);

		if(aclEnable){
			updateUserRules(container);
			var containerRule = applyRule(container);
			if(verifyContainerRule(containerRule.rule)){
				updateItemAcl(container, ZhuAclConfig.containerType);
			}
			loadedContainers[container.id] = true;
		}
	};

	var convertObjectRulesToArray = function(userId, objRules){
		var arrayRules = [];
		Object.keys(objRules).forEach(function(ruleId){
			var objRule = objRules[ruleId];
			var userRule = objRule.userGroups[userId];
			if(userRule){
				var rule = {
					"UNIQUE_ID": ruleId,
					"NAME": objRule.itemName,
					"RULE": userRule.rule
				};
				arrayRules.push(rule);
			}
		});
		return arrayRules;
	};

	var convertArrayRulesToObject = function(userId, arrayRules){
		var objRules = {};
		arrayRules.forEach(function(rule){
			var profileObjects = {};
			profileObjects[userId] = {
				'rule': rule.RULE,
				'undefinedGroups': rule.GROUP_ID || []
			};
			objRules[rule.UNIQUE_ID] = {
				'itemName': rule.NAME,
				'userGroups': profileObjects
			};
		});
		return objRules;
	};

	var subscribeContainerEvent = function(rules, groupIds, userId){
		eventAggregator.subscribe(EventAggregator.ON_CONTAINER_OPEN, function applyACL(){
			var containerId = templateManager.container.id;
			if(!loadedContainers[containerId]){
				updateContainerAcl(rules, groupIds, userId);
			}
		});

	};

	var updateProjectAcl = function(loginRules, defaultRule, groupIds){
		var rules = {};
		var userId = user.getUserId();
		if(Util.isObject(loginRules)){
			rules = loginRules;
			loginRules = convertObjectRulesToArray(userId, rules);
		} else if(Util.isArray(loginRules)){
			rules = convertArrayRulesToObject(userId, loginRules);
		}

		templateManager.project.aclDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;
		templateManager.userId = userId;

		templateManager.aclRules = rules;
		updateContainerAcl(rules, groupIds, userId);
		subscribeContainerEvent(rules, groupIds, userId);
	};

	var createUser = function createUser(userId, projectId, organizationId, defaultRule){
		user = new User(userId, projectId, organizationId, defaultRule);
	};

	this.setUserRules = function(groupIds, projectId, userId, organizationId, options){
		var returnPromise;
		loadedContainers = {};
		options = options || {};

		if(options.canConfigureAcl){
			ZhuAccessButtonService.enableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		} else {
			ZhuAccessButtonService.disableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		}

		if(options.rules){
			createUser(userId, projectId, organizationId, options.defaultRule);
			updateProjectAcl(options.rules, options.defaultRule, groupIds);
			returnPromise = ZHPromise.when(templateManager.aclRules);
		} else {
			returnPromise = ZhuAclService.getZhuAclRules(projectId, groupIds).then(function(response){
				createUser(userId, projectId, organizationId, response.defaultRule);
				updateProjectAcl(response.aclRules, response.defaultRule, groupIds);
				return templateManager.aclRules;
			}).catch(function(err){
				MessageService.showMessageByCode(err.code);
			});
		}
		return returnPromise;
	};

	this.setElementsListFilter = function(container, groupId, widgetName){
		widgetName = widgetName || ZhuAclConfig.menuElementsListWidgetName;
		var elementsWidget = container.getWidget(widgetName);
		elementsWidget.dataSourceFilter = ZhuAclConfig.getDefaultDataSourceFilter(groupId, user.getProjectId(), user.getOrganizationId());
		user.setCurrentGroupId(groupId);
	};

	var getFactoredWidget = function(container, widgetName, pushInContainer){
		var widget = container.getWidget(widgetName);
		if(widget){
			return ZHPromise.when(widget);
		} else {
			return ZhuAclService.getWidgetMetadata(widgetName).then(function(metadata){
				var factoredWidget = metaDataFactory.widgetFactory(metadata, container);
				if(pushInContainer){
					container.widgets.push(factoredWidget);
				}
				return factoredWidget;
			});
		}
	};
	this.getFactoredWidget = getFactoredWidget;

	this.insertElementsListWidget = function(container, parentWidget, extraElementsPath){
		getFactoredWidget(container, ZhuAclConfig.menuElementsListWidgetName).then(function(factoredWidget) {
			if(parentWidget && !factoredWidget.parent){
				factoredWidget.parent = parentWidget;
				parentWidget.widgets = parentWidget.widgets || [];
				parentWidget.widgets.push(factoredWidget);
				factoredWidget.extraElementsPath = extraElementsPath;
			}
		});
	};

}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclController', ZhuAclController);
});


// FILE: assets/js/controllers/ZhuAclCRUDController.js
/* global Configuration, ZhuAclMessages */
function ZhuAclCRUDController(ScreenService, ZHPromise, ZhuAclConfig, ZhuAclController, MessageService){

	function defineQuestionToUse(rule, messageConfig){
		var question = ZhuAclMessages.VISIBLE;
		if(rule == ZhuAclConfig.RULE_INVISIBLE){
			question = ZhuAclMessages.INVISIBLE;
		} else if(rule == ZhuAclConfig.RULE_ALLOW){
			question = ZhuAclMessages.EDITABLE;
		}
		messageConfig.newVisibility = ScreenService.i18n(question);
		return messageConfig;
	}

	function deleteChildrensRules(elementsWidget, parentRow, groupId, parentRule){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					if(row.extraElement){
						row.RULE = parentRule;
					} else {
						row.__isDeleted = true;
						row.RULE_SAVE_CONTAINER = parentRule;
					}
					row.changed = true;
					row.GROUP_ID = groupId;
					deleteChildrensRules(elementsWidget, row, groupId, parentRule);
				}
			});
		}
	}

	function confirmChildsVisibility(elementsWidget, ruleType, messageConfig){
		var returnPromise = ZHPromise.when();
		if(elementsWidget.currentRow._childCount){
			var currentRow = elementsWidget.currentRow;
			var defer = ZHPromise.defer();
			var actions = [
				{
					label: ScreenService.i18n(ZhuAclMessages.YES),
					code: function(){
						currentRow.changed = true;
						deleteChildrensRules(elementsWidget, currentRow, currentRow.GROUP_ID, currentRow.RULE);
						defer.resolve();
					}
				},
				{
					label: ScreenService.i18n(ZhuAclMessages.NO),
					code: function(){
						var messageConfig = {
							type: 'confirmation',
							icon: 'question'
						};
						MessageService.showMessageByCode(ZhuAclMessages.CONFIRMATION_MESSAGE, {}, messageConfig);
						defer.reject();
					}
				}
			];
			var message = ZhuAclMessages.CONFIRM_CHILDS_QUESTION;
			var dialogConfig = {
				type: 'confirmation',
				icon: 'question',
				actions: actions
			};

			MessageService.showCustomDialogByCode(message, defineQuestionToUse(ruleType, messageConfig), dialogConfig);
			returnPromise = defer.promise;
		}

		return returnPromise;
	}

	function reloadChildrensRules(elementsWidget, parentRow, updatedRules, deletedRules){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					var rowId = row.UNIQUE_ID;
					row.isVisible = parentRow.isVisible;
					row.isEditable = parentRow.isEditable;

					if(deletedRules.indexOf(rowId) != -1){
						delete row.RULE_ID;
					}
					if(updatedRules[rowId] && updatedRules[rowId].containerId){
						row.CONTAINER_RULE_ID = updatedRules[rowId].containerId;
					}

					reloadChildrensRules(elementsWidget, row, updatedRules, deletedRules);
				}
				delete row.__isDeleted;
				delete row.changed;
			});
		}
	}

	function fixZeedhiBehavior(widget){
		widget.dataSource.data.forEach(function(data){
			data.isEditable = (data._childCount && data.isVisible) || data.isEditable;
		});
	}

	function isFatherVisible(parentRow, widget){
		return widget.dataSource.data.some(function(row){
			return (row.PARENT_ID == parentRow.UNIQUE_ID) && (row.isVisible || row.isEditable);
		});
	}

	function reloadParentsRules(row, widget){
		var parentRow = row._parent;
		if(parentRow){
			if(row.isVisible){
				parentRow.isEditable = parentRow.isEditable || row.isEditable || row.isVisible;
				parentRow.isVisible = true;
			} else {
				var updateFather = isFatherVisible(parentRow, widget);
				parentRow.isEditable = updateFather;
				parentRow.isVisible = updateFather;
			}
			reloadParentsRules(row._parent, widget);
		}
	}

	function clearRowState(row){
		row.changed = false;
		row.__isDeleted = false;
	}

	function reloadDataSourceAfterSave(elementsWidget, newRules){
		var currentRow = elementsWidget.currentRow;
		var currentRowId = currentRow.UNIQUE_ID;
		var updatedRule = newRules.updated[currentRowId];
		var deletedRules = newRules.deleted || [];
		if(updatedRule){
			if(Util.isObject(updatedRule)){
				currentRow.RULE_ID = updatedRule.menuId;
				currentRow.CONTAINER_RULE_ID = updatedRule.containerId;
			} else {
				currentRow.RULE_ID = updatedRule;
			}
			reloadChildrensRules(elementsWidget, currentRow, newRules.updated, deletedRules);
			if(!currentRow.extraElement){
				reloadParentsRules(currentRow, elementsWidget);
				fixZeedhiBehavior(elementsWidget);
			}
		}

		var updatedRules = Object.keys(newRules.updated);
		if(updatedRules.length){
			//updated all extraElements
			elementsWidget.dataSource.data.forEach(function(row){
				var ruleId = newRules.updated[row.UNIQUE_ID];
				if(ruleId && row.extraElement){
					row.RULE_ID = ruleId;
					row.RULE = currentRow.RULE;
					row.isVisible = currentRow.isVisible;
					row.isEditable = currentRow.isEditable;
				}
				clearRowState(row);
			});
		} else {
			elementsWidget.dataSource.data.forEach(clearRowState);
		}
	}

	function cancelSave(widget){
		widget.reload();
	}

	function changeElementRule(currentRow, isVisible, isEditable){
		currentRow.RULE = ZhuAclConfig.getRuleType(isVisible, isEditable);
		currentRow.isVisible = isVisible;
		currentRow.isEditable = isEditable;
		currentRow.GROUP_ID = currentRow.GROUP_ID || ZhuAclController.getCurrentGroupId();
	}

	function clearInlineChangedStatus(row){
		delete row.changed_isEditable;
		delete row.changed_isVisible;
	}

	function saveRule(widget, isEditable, isVisible, messageConfig){
		var currentRow = widget.currentRow;
		var ruleType = ZhuAclConfig.getRuleType(isVisible, isEditable);

		clearInlineChangedStatus(currentRow);
		changeElementRule(currentRow, isVisible, isEditable);

		confirmChildsVisibility(widget, ruleType, messageConfig).then(function(){
			widget.saveRow().then(function(response){
				widget.reload();
				// reloadDataSourceAfterSave(widget, response.dataset.aclZhuAccessControlRule);
				MessageService.successNotificationByCode(ZhuAclMessages.SUC_RULE_INSERTED);
			}).catch(function(){
				cancelSave(widget);
				MessageService.notificationMessageByCode(ZhuAclMessages.ERR_RULE_INSERTED, {}, {type: "alert"});
			});
		}).catch(function(){
			cancelSave(widget);
			MessageService.notificationMessageByCode(ZhuAclMessages.RULE_NOT_SAVED, {}, {type: "alert"});
		});
	}

	this.changeIsEditable = function changeIsEditable(widget, row, messageConfig){
		var isEditable = row.isEditable;
		var isVisible = isEditable || row.isVisible;
		saveRule(widget, isEditable, isVisible, messageConfig);
	};

	this.changeIsVisible = function changeIsVisible(widget, row, messageConfig){
		var isVisible = row.isVisible;
		var isEditable = isVisible && row.isEditable;
		saveRule(widget, isEditable, isVisible, messageConfig);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclCRUDController', ZhuAclCRUDController);
});


// FILE: assets/js/controllers/ZhuAclExportRulesController.js
function ZhuAclExportRulesController(templateManager, MessageService, ScreenService, ZhuAclConfig, ZhuAclController, ZhuAclService){
	var self = this;
	var requestToExportRules = function(profileFrom, profilesToExport) {
		ZhuAclService.exportRules(profileFrom, profilesToExport)
			.then(self.closeExportRulesPopup)
			.catch(function(response){
				var errorCode = (response && (response.errorCode || response.error)) || ZhuAclMessages.ERR_RULES_EXPORTED;
				MessageService.errorNotificationByCode(errorCode);
			});
	};

	this.exportRules = function() {
		var exportRulesWidget = templateManager.container.getWidget(ZhuAclConfig.exportRulesModal);
		if(exportRulesWidget.isValid()){
			var currentRow = exportRulesWidget.currentRow;
			var dialogConfig = {
				type: 'CONFIRMATION',
				actions: [
					{
						label: ScreenService.i18n(ZhuAclMessages.YES),
						code: function() {
							requestToExportRules(currentRow.profileFrom, currentRow.profilesToExport);
						}
					},
					{ label: ScreenService.i18n(ZhuAclMessages.NO) }
				]
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.CONFIRM_EXPORT, {}, dialogConfig);
		}
	};

	this.profileActionEventExportRules = function(groupId) {
		var container = templateManager.container;
		return ZhuAclController.getFactoredWidget(container, ZhuAclConfig.exportRulesModal, true).then(function(exportRulesWidget){
			ScreenService.openPopup(exportRulesWidget);
			exportRulesWidget.isVisible = true;
			exportRulesWidget.currentRow = {
				profileFrom: groupId
			};
			exportRulesWidget.fields[0].dataSourceFilter = [{
				name: 'ID',
				operator: '<>',
				value: groupId
			}];
		});
	};

	this.closeExportRulesPopup = function() {
		ScreenService.closePopup();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclExportRulesController', ZhuAclExportRulesController);
});

// FILE: assets/js/controllers/ZhuContainerElementsController.js
/* global Util, Configuration, ZhuAclMessages */
function ZhuContainerElementsController(templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService){
	var self = this;
	var idErrorMessages = "";

	var getContainerElementsTreeWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsTreeWidgetName);
	};

	var getContainerElementsListWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = "";
	};

	var showIdErrorsMessage = function(){
		if(idErrorMessages){
			var messageConfig = {
				type: 'alert',
				icon: 'exclamation'
			};
			MessageService.showMessageByCode(ZhuAclMessages.NSHOW_WITHOUT_ID, {elementsIds: idErrorMessages}, messageConfig);
		}
	};

	var addParentMenuProperties = function(rules, item, itemToInsert){
		var parentMenuId = item.parentMenuId;
		var parentMenuLabel = item.parentMenuLabel || item.parentMenuName;
		if(parentMenuId && parentMenuLabel){
			var parentMenuRule = ZhuAclController.getElementRule(rules, parentMenuId);
			itemToInsert.PARENT_MENU_ID = parentMenuId;
			itemToInsert.PARENT_MENU_NAME = item.parentMenuName || ZhuAclController.prepareLabelToName(parentMenuLabel);
			itemToInsert.PARENT_MENU_LABEL = parentMenuLabel;
			itemToInsert.PARENT_MENU_RULE_ID = parentMenuRule.ID;
		}
	};

	var getPlannedContainer = function(rules, item, itemType, parentId){
		var plannedItems = [];
		itemType = itemType || ZhuAclConfig.containerType;
		if(item && !item.isZhuAclElement && (!!item.label || ['container', 'widgets'].includes(itemType))){
			var uniqueId = item.id;
			if(uniqueId){
				var aclType = ZhuAclConfig.getAclMap(itemType);
				var itemToInsert = ZhuAclController.getPlannedItemToInsertInDataSet(rules, item, aclType.aclTypeCod, parentId);
				addParentMenuProperties(rules, item, itemToInsert);
				plannedItems.push(itemToInsert);
				aclType.childs.forEach(function(type){
					if(Util.isArray(item[type])){
						item[type].forEach(function(child){
							plannedItems = plannedItems.concat(getPlannedContainer(rules, child, type, uniqueId));
						});
					}
				});
			} else {
				var nameToShow = item.label || item.name;
				idErrorMessages += "<br> " + nameToShow + ";";
			}
		}
		return plannedItems;
	};

	this.reloadContainerElementsModal = function(){
		getContainerElementsListWidget().reload();
	};

	this.reloadContainerElementsTree = function(){
		getContainerElementsTreeWidget().reload();
	};

	this.selectAccessOnChange = function(selectedRow){
		if(selectedRow && selectedRow.SELECT_ACCESS){
			ZhuAclController.setElementsListFilter(templateManager.container, selectedRow.ID, ZhuAclConfig.containerElementsTreeWidgetName);
		}
		self.reloadContainerElementsTree();
	};

	var copyItem = function(itemToCopy, itemType, parentId){
		var item;
		if(itemToCopy && !itemToCopy.isZhuAclElement && !itemToCopy.ignoreAclElementsConfig){
			var uniqueId = itemToCopy.id;
			item = {
				id: uniqueId,
				name: itemToCopy.name,
				label: itemToCopy.label,
				parentMenuId: itemToCopy.parentMenuId,
				parentMenuName: itemToCopy.parentMenuName,
				parentMenuLabel: itemToCopy.parentMenuLabel,
			};
			itemType = itemType || ZhuAclConfig.containerType;
			var aclType = ZhuAclConfig.getAclMap(itemType);
			aclType.childs.forEach(function(type){
				if(Util.isArray(itemToCopy[type])){
					item[type] = itemToCopy[type].map(function(child){
						return copyItem(child, type, uniqueId);
					}).filter(function(copiedChild){
						return copiedChild;
					});
				}
			});
		}
		return item;
	};

	this.containerElementsTreeAfterLoadDataSource = function(){
		clearIdErrorsMessage();
		var widget = getContainerElementsTreeWidget();
		var elementsListWidget = getContainerElementsListWidget();
		var elementsListRow = elementsListWidget && elementsListWidget.currentRow;
		var rulesData = Util.clone(widget.dataSource.data);
		widget.dataSource.data = [];

        elementsListWidget.getField('disableDescription').isVisible = false;
		if(elementsListRow && elementsListRow.SELECT_ACCESS){
			var rules = ZhuAclController.getRulesFromDatasource(rulesData);
			var container = copyItem(templateManager.container);
			if(container){
				aclEngine.processAcl(container, rules, templateManager.userId, templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule(), true);
				widget.dataSource.data = getPlannedContainer(rules, container);
				showIdErrorsMessage();
                checkEditContainerRules(container, rules, widget, elementsListWidget);
			}
		}
	};

    checkEditContainerRules = function(container, rules, widget, elementsListWidget) {
        var isVisibleField   = widget.getField('isVisible');
        var isEditableField  = widget.getField('isEditable');
        var disableDescField = elementsListWidget.getField('disableDescription');
        if(rules.hasOwnProperty(container.id) && rules[container.id].ruleRow && ['A', 'R'].includes(rules[container.id].ruleRow.RULE)) {
            isVisibleField.readOnly    = false;
            isEditableField.readOnly   = false;
            disableDescField.isVisible = false;
        } else {
            isVisibleField.readOnly    = true;
            isEditableField.readOnly   = true;
            disableDescField.isVisible = true;
        }
    };

	this.isEditableOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		var messageConfig = {
			type: 'container',
			subType: 'elementos filhos'
		};
		ZhuAclCRUDController.changeIsEditable(widget, currentRow, messageConfig);
	};

	this.isVisibleOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		var messageConfig = {
			type: 'container',
			subType: 'elementos filhos'
		};
		ZhuAclCRUDController.changeIsVisible(widget, currentRow, messageConfig);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuContainerElementsController', ZhuContainerElementsController);
});

// FILE: assets/js/controllers/ZhuMenuElementsController.js
function ZhuMenuElementsController(ScreenService, templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService, restEngine, ZHPromise){
	var MENU = "M";
	var GROUP_MENU = "G";
	var IGNORED_MENUS = ['zhHistory'];
	var idErrorMessages = {"errorMenus": "", "errorContainers": ""};

	var getWidgetElements = function(){
		return templateManager.container.getWidget(ZhuAclConfig.menuElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = {"errorMenus": "", "errorContainers": ""};
	};

	var showIdErrorsMessage = function(){
		var messageCode;
		var errorMenus = idErrorMessages.errorMenus;
		var errorContainers = idErrorMessages.errorContainers;
		var messageParameters = {
			elementsIds: errorMenus,
			elementsWithoutContainer: errorContainers
		};

		if(errorMenus && errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG;
		} else if(errorMenus){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID;
		} else if(errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_CONTAINER_CONFIG;
		}

		if(messageCode){
			MessageService.showMessageByCode(messageCode, messageParameters, { type: 'alert' });
		}
	};

	var addErrorMenuId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorMenus += "<br> " + nameToShow + ";";
	};

	var addErrorContainerId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorContainers += "<br> " + nameToShow + ";";
	};

	this.elementsActionEventEdit = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			var messageConfig = {
				type: 'menu',
				subType: 'submenus'
			};
			currentRow.isEditable = !currentRow.isEditable;
			ZhuAclCRUDController.changeIsEditable(elementsWidget, currentRow, messageConfig);
		}
	};

	this.elementsActionEventView = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			var messageConfig = {
				type: 'menu',
				subType: 'submenus'
			};
			currentRow.isVisible = !currentRow.isVisible;
			currentRow.isEditable = currentRow.isVisible;
			ZhuAclCRUDController.changeIsVisible(elementsWidget, currentRow, messageConfig);
		}
	};

	this.elementsActionEventBack = function(){
		var elementsWidgetParent = getWidgetElements().parent;
		return ScreenService.closeSwipe().then(function(){
			elementsWidgetParent.list();
			elementsWidgetParent.activate();
		});
	};

	var addContainerProperties = function(item, menuItem, rules){
		var containerId = menuItem.containerId;
		var containerLabel = menuItem.containerLabel || menuItem.containerName;
		if(containerId && containerLabel){
			var containerRule = ZhuAclController.getElementRule(rules, containerId);
			item.CONTAINER_ID = containerId;
			item.CONTAINER_NAME = menuItem.containerName || ZhuAclController.prepareLabelToName(containerLabel);
			item.CONTAINER_LABEL = containerLabel;
			item.CONTAINER_RULE_ID = containerRule.RULE_ID;
		} else if(menuItem.windowName && !menuItem.buildedMenu){
			addErrorContainerId(menuItem);
		}
	};

	var planningMenuItem = function(menuItem, parentMenuItem, rules){
		var item;
		var uniqueId = menuItem.id;
		if(uniqueId){
			var parentId = parentMenuItem && parentMenuItem.id;
			var itemType = !parentId ? GROUP_MENU : MENU;
			item = ZhuAclController.getPlannedItemToInsertInDataSet(rules, menuItem, itemType, parentId);
			item.buildedMenu = menuItem.buildedMenu;
			addContainerProperties(item, menuItem, rules);
		} else {
			addErrorMenuId(menuItem);
		}

		return item;
	};

	var addMenusInDataSource = function(widget, rules, menus, parentMenu){
		menus.forEach(function(menu, index){
			var item = planningMenuItem(menu, parentMenu, rules);
			if(item){
				widget.dataSource.data.push(item);
				if (menu.menus && menu.menus.length > 0){
					addMenusInDataSource(widget, rules, menu.menus, menu);
				}
			}
		});
	};

	var getProjectMenu = function(){
		var projectMenu = Util.clone(templateManager.project.groupMenu);
		return projectMenu.filter(function(menu){
			return IGNORED_MENUS.indexOf(menu.name) == -1;
		});
	};

	var extraRulesPromise;

	var loadExtraRules = function(extraElementsPath){
		if(!extraRulesPromise){
			if(extraElementsPath){
				extraRulesPromise = restEngine.requestMetaData(Util.getRightPath(extraElementsPath), {});
			} else {
				extraRulesPromise = ZHPromise.when();
			}
		}
	};

	var addExtraElements = function(widget, projectRules, defaultRule){
		var getRule = function(id){
			return (projectRules[id] && projectRules[id].ruleRow) || {};
		};

		var getVisibility = function(rule){
			return (rule || defaultRule) !== 'I';
		};

		extraRulesPromise.then(function(response){
			if(response && response.data){
				response.data.forEach(function(row){
					var rule = getRule(row.UNIQUE_ID);
					row.RULE = rule.RULE;
					row.RULE_ID = rule.ID;
					row.isVisible = getVisibility(rule.RULE);
					row.extraElement = true;
				});
				widget.dataSource.data = widget.dataSource.data.concat(response.data);
			}
		});
	};

	this.loadPlanningTree = function(){
		var elementsWidget = getWidgetElements();
		var userId = templateManager.userId;
		var defautRule = templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule();
		var rules = ZhuAclController.getRulesFromDatasource(elementsWidget.dataSource.data);
		var newMenu = getProjectMenu();

		clearIdErrorsMessage();
		loadExtraRules(elementsWidget.extraElementsPath);
		aclEngine.processMenuAcl(newMenu, rules, templateManager.containers, userId, defautRule);
		elementsWidget.dataSource.data = [];
		addMenusInDataSource(elementsWidget, rules, newMenu);
		addExtraElements(elementsWidget, rules, defautRule);
		showIdErrorsMessage();
	};

	this.reloadElementsWidget = function(){
		getWidgetElements().reload();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuMenuElementsController', ZhuMenuElementsController);
});

// FILE: assets/js/adapter/config.js
ZeedhiApp.run(function(templateManager, ZhuAccessButtonService) {
	templateManager.projectPromise.then(function(project){
		if(templateManager.project.aclEnable){
			ZhuAccessButtonService.insertButtonInProject(templateManager.project);
		}
	});
});
